SUBROUTINE avecty( mot, longm, long, ideb, iend, kode, &
                   nomv, iv, minv, maxv, nind, maxx,   &
                   nx, icol, jcol, lerr,               &
                   dx, rx, ix, lx, cx )
!***********************************************************************
!     - avecty -
!     affecte une liste de valeurs lues au vecteur dx/rx/ix/lx/cx (maxx)
!     et retourne dans vecteur iv(minv:maxv) une liste d'indices entiers
!
!     recherche d'une liste d'indices d'objets et d'une liste de valeurs
!     dans une structure du type: mot(.....)=......  la liste d'indices
!     figurant a gauche du signe "=" et entre parentheses, la liste des
!     valeurs etant a droite du "=". la recherche se fait dans la zone
!     car(ideb:iend).
!     les indices sont donnes soit explicitement (i.e. une valeur
!     entiere), soit implicitement par l'intermediaire d'un nom
!     correspondant a cet indice: on convertira ce nom en son indice
!     dans le vecteur caracteres nomv(minv:maxv)
!***********************************************************************
!
!     MOT   - ENTREE. LE MOT-CLE                              - CAR
!     LONGM - ENTREE. SA LONGUEUR MAXIMALE                    - ENTIER
!     LONG  - ENTREE. SA LONGUEUR MINIMALE                    - ENTIER
!     IDEB  - ENTREE. DEBUT DE ZONE DE RECHERCHE DANS CAR     - ENTIER
!     IEND  - ENTREE. FIN   DE ZONE DE RECHERCHE DANS CAR     - ENTIER
!     KODE  - ENTREE. CODE DU TYPE DE VECTEUR :               - ENTIER
!                        1 = REEL DOUBLE PRECISION
!                        2 = REEL SIMPLE PRECISION
!                        3 = ENTIER
!                        4 = LOGIQUE
!                        5 = CARACTERE
!                        6 = REEL DOUBLE PRECISION CONTENANT
!                            DES VALEURS CARACTERES
!     NOMV  - ENTREE. LES NOMS POSSIBLES DES INDICES          - CAR
!     IV    - SORTIE. VECTEUR DES INDICES                     - ENTIER
!                     (DIMENSION MAXV-MINV+1)
!     MINV  - ENTREE. BORNE INFERIEURE POUR DIMENSION DE NOMV - ENTIER
!     MAXV  - ENTREE. BORNE SUPERIEURE POUR DIMENSION DE NOMV - ENTIER
!     NIND  - SORTIE. NOMBRE D'INDICES MIS DANS IV            - ENTIER
!     MAXX  - ENTREE. NOMBRE MAXIMUM DE VALEURS A STOCKER     - ENTIER
!     NX    - SORTIE. LE NOMBRE DE VALEURS STOCKEES           - ENTIER
!                     (DOIT ETRE =< MAXX)
!     ICOL  - SORTIE. POSITION ABSOLUE DANS CAR DU MOT CLE    - ENTIER
!                     DANS CAR(IDEB:IEND) (0 SI NON TROUVE)
!     JCOL  - SORTIE. POSITION ABSOLUE DANS CAR DU SIGNE DE   - ENTIER
!                     FIN DE PHRASE (SEP2) CORRESPONDANT
!     LERR  - SORTIE. .TRUE. SI ERREUR                        - LOGIQUE
!     DX    - SORTIE. LES VALEURS ASSOCIEES AU MOT-CLE (OPT.) - REAL*8
!     RX    - SORTIE. LES VALEURS ASSOCIEES AU MOT-CLE (OPT.) - REEL
!     IX    - SORTIE. LES VALEURS ASSOCIEES AU MOT-CLE (OPT.) - ENTIER
!     LX    - SORTIE. LES VALEURS ASSOCIEES AU MOT-CLE (OPT.) - LOGIQUE
!     CX    - SORTIE. LES VALEURS ASSOCIEES AU MOT-CLE (OPT.) - CAR
!
!     REMARQUE: ON NE BOUCLE PAS SUR LA RECHERCHE DU MOT CLE:
!               SEULE LA PREMIERE OCCURRENCE EST PRISE EN COMPTE
!
!     APPELLE DNUM, INDVEY, INUM, KARIN, POSITM
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: longm, long, ideb, iend, kode, minv, maxv, maxx
   CHARACTER(LEN=*), INTENT(IN) :: mot
   CHARACTER(LEN=*), DIMENSION(minv:maxv), INTENT(IN) :: nomv
   INTEGER, INTENT(OUT) :: nind, nx, icol, jcol
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, DIMENSION(maxv-minv+1), INTENT(OUT) :: iv
   REAL(KIND=8)    , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: dx
   REAL            , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: rx
   INTEGER         , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: ix
   LOGICAL         , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: lx
   CHARACTER(LEN=*), DIMENSION(maxx), INTENT(OUT), OPTIONAL :: cx

   !  Declarations locales
   !  --------------------

   REAL(KIND=8) :: a, b, c, dval
   INTEGER :: i, ijk, irep, ival, i1
   INTEGER :: ja, jb, jc, kol, kola, kolz, kol1, kol2, kol3, kol4, k3
   INTEGER :: larg, ll, nrep
   LOGICAL :: lgl, lexit
   CHARACTER(LEN=1) :: car3
   CHARACTER(LEN=4) :: numero
   CHARACTER(LEN=255) :: ca, nomb

   INCLUDE 'avectyp.'

   !--------------------------------------------------------------------

   nind = 0
   nx   = 0
   icol = 0
   jcol = 0
   lerr = .false.

   i1 = ideb

   ! Positionnement sur le mot cle
   call positm( mot, longm, long, i1, iend, kol, kol1, kol2, lerr )
   if ( lerr ) return
   if ( kol <= 0 ) return

   icol = kol
   jcol = kol2

   ! Determination des indices du vecteur
   kol = kol1
   call indvey( mot, nomv, minv, maxv, kol, jcol, nind, kol1, iv, maxv-minv+1, lerr )
   if ( lerr ) return

   ! kol1 = col. du signe "=" precedant la liste
   ! jcol = col. de la fin de liste (sep2)
   ! kol3 = col. du 1er separateur d'elements dans la liste
   ! car3 = carcatere attendu avant l'element examine

   car3 = '='

   if ( kode < 5 ) then

      ! Analyse d'un vecteur numerique ou logique
      ! -----------------------------------------

      do             ! Boucle sur les elements de la liste

         kol3 = index( car(kol1+1:jcol), sep1 ) + kol1
         if ( kol3 > kol1 ) then      ! Plusieurs valeurs
            lexit = .false.
         else                         ! Une seule valeur (ou boucle)
            kol3 = jcol
            lexit = .true.
         end if
         nrep = 1
         ! Boucle ?
         kol = index( car(kol1+1:kol3), to(:ncto) ) + kol1
         if ( kol > kol1 ) then      ! Boucle (a>b:c)
            c  = 1.0_8
            k3 = kol3 - 1
            if ( car(k3:k3) == par2 ) k3 = k3 - 1
            call dnum( k3, kol, -1, by, ncby, to, ncto, .true., ll, lgl, b, lerr )
            if ( lerr ) return
            if ( lgl ) then
               c = b
               if ( c == 0.0_8 ) c = 1.0_8
               call dnum( ll-1, kol, -1, to, ncto, to, ncto, .true., ll, lgl, b, lerr )
               if ( lerr ) return
            end if
            call dnum( ll-1, kol1, -1, par1, 1, car3, 1, .true., ll, lgl, a, lerr )
            if ( lerr ) return
            ! Correction eventuelle du pas
            if ( a > b .and. c > 0.0_8 ) c = -c
            ! Repetition n* ?
            if ( lgl .and. car(ll-1:ll-1) == repet ) then
               call inum( ll-2, kol1,   -1, car3, 1, car3, 1, .true., ll, lgl, nrep, lerr )
               if ( lerr ) return
            end if
         else
            ! Structure simple a ou n*a
            call dnum( kol3-1, kol1, -1, repet, 1, car3, 1, .true., ll, lgl, a, lerr )
            if ( lerr ) return
            if ( lgl ) then
               call inum( ll-1, kol1, -1, car3, 1, car3, 1, .true., ll, lgl, nrep, lerr )
               if ( lerr ) return
            end if
            b = a
            c = 1.0_8
         end if
         ! Mise a jour du vecteur
         select case ( kode )
         case ( 1 )      ! Vecteur double precision
            do irep = 1, nrep
               do dval = a, b, c
                  nx = nx + 1
                  if ( nx > maxx ) goto 100
                  dx(nx) = dval
               end do
            end do
         case ( 2 )      ! Vecteur simple precision
            do irep = 1, nrep
               do dval = a, b, c
                  nx = nx + 1
                  if ( nx > maxx ) goto 100
                  rx(nx) = dval
               end do
            end do
         case ( 3 )      ! Vecteur entier
            do irep = 1, nrep
               do ival = nint(a), nint(b), nint(c)
                  nx = nx + 1
                  if ( nx > maxx ) goto 100
                  ix(nx) = ival
               end do
            end do
         case ( 4 )      ! Vecteur logique
            do irep = 1, nrep
               do ival = nint(a), nint(b), nint(c)
                  nx = nx + 1
                  if ( nx > maxx ) goto 100
                  lx(nx) = ival /= 0
               end do
            end do
         end select

         ! Mise a jour pour analyse de l'element suivant
         kol1 = kol3
         car3 = sep1
         if ( lexit ) exit

      end do

   else

      ! Analyse d'un vecteur caractere
      ! ------------------------------

      do             ! Boucle sur les elements de la liste

         kol4 = kol1
         do
            kol3 = index( car(kol4+1:jcol), sep1 ) + kol4
            if ( kol3 > kol4 ) then
               if ( com(kol3) ) then
                  kol4 = kol3
                  cycle
               end if
               ! Plusieurs valeurs
               lexit = .false.
            else
               ! Une seule valeur
               kol3 = jcol
               lexit = .true.
            end if
            exit
         end do
         if ( kol3 <= (kol4+1) ) then
            kola = max( kol4-25, icol )
            kolz = min( kol4+25, ncar )
            write(l6,*) mrker1
            write(l6,11)  typer, car(kola:kolz), flag(:kol4-kola+1)
            write(l6,555) car(kol4:kol4)
            write(l6,*) mrker2
            lerr = .true.
            return
         end if
         kola = max( kol3-65, icol )
         kol4 = kol1
      10 continue
         kol  = index( car(kol4+1:kol3-1), to(:ncto) ) + kol4
         if ( kol > kol4 ) then
            if ( com(kol) ) then
               kol4 = kol
               goto 10
            end if
            kol = kol + ncto
            ! C'est une boucle sur les noms
            ! Analyse du dernier terme: nomb suivi du numero jb
            jc = 1
            k3 = kol3 - 1
            if ( car(k3:k3) == par2 ) k3 = k3 - 1
            call inum( k3, kol, -1, ' ', 1, ' ', 1, .false., ll, lgl, jb, lerr )
            if ( lerr ) return
            if ( .not. com(ll) .and. car(ll-ncby+1:ll) == by(:ncby) ) then
               jc = jb
               if ( jc == 0 ) jc = 1
               ll = ll - ncby + 1
               call inum( ll-1, kol, -1, ' ', 1, ' ', 1, .false., ll, lgl, jb, lerr )
               if ( lerr ) return
            end if
            if ( ll+1 <= kol ) then
               kola = max( ll-25, icol )
               kolz = min( ll+25, ncar )
               write(l6,*) mrker1
               write(l6,11) typer, car(kola:kolz), flag(:ll-kola)
               write(l6,111)
               write(l6,*)  car(kol1:kol3)
               write(l6,*) mrker2
               lerr = .true.
               return
            end if
            nomb = car(kol:ll)
            larg = ll - kol + 1
            ! Analyse du 1er terme: nomb suivi de ja
            call inum( kol-ncto-1, kol1-1, -1, ' ', 1, ' ', 1, .false., ll, lgl, ja, lerr )
            if ( lerr ) return
            if ( car(ll-larg+1:ll) /= nomb ) then
               kolz = min( kol3, kola+64 )
               write(l6,*) mrker1
               write(l6,11) typer, car(kola:kolz)
               write(l6,222)
               write(l6,*)  car(kol1:kol3)
               write(l6,*) mrker2
               lerr = .true.
               return
            end if
            ! Verif et repetition ?
            ll   = ll - larg - 1
            nrep = 1
            if ( ll+1 /= kol1 ) then
               if ( car(ll+1:ll+1) /= par1 ) then
                  kola = max( ll-25, icol )
                  kolz = min( ll+25, ncar )
                  write(l6,*) mrker1
                  write(l6,11) typer, car(kola:kolz), flag(:ll+1-kola)
                  write(l6,333)
                  write(l6,*)  car(kol1:kol3)
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               else
                  if ( car(ll:ll) == repet ) then
                     call inum( ll-1, kol1, -1, car3, 1, car3, 1, .false., ll, lgl, nrep, lerr )
                     if ( lerr ) return
                  end if
                  if ( ll /= kol1 ) then
                     kola = max( ll-25, icol )
                     kolz = min( ll+25, ncar )
                     write(l6,*) mrker1
                     write(l6,11) typer, car(kola:kolz), flag(:ll-kola)
                     write(l6,444)
                     write(l6,*)  car(kol1:kol3)
                     write(l6,*) mrker2
                     lerr = .true.
                     return
                  end if
               end if
            end if
            ! Correction eventuelle du pas
            if ( ja > jb .and. jc > 0 ) jc = -jc
            ! Construction des noms: nomb//ja a jb
            if ( kode == 5 ) then
               do irep = 1, nrep
                  do i = ja, jb, jc
                     nx = nx + 1
                     if ( nx > maxx ) goto 100
                     call karin( i, numero, ijk )
                     cx(nx) = nomb(:larg) // numero
                  end do
               end do
            else
               do irep = 1, nrep
                  do i = ja, jb, jc
                     nx = nx + 1
                     if ( nx > maxx ) goto 100
                     call karin( i, numero, ijk )
                     ca(:8) = nomb(:larg) // numero
                     read(ca,666) dx(nx)
                  end do
               end do
            end if

         else
            ! Pas de boucle
            ! Repetition ?
            kol = index( car(kol1+1:kol3-1), repet ) + kol1
            if ( kol > kol1 .and. .not. com(kol) ) then
               call inum( kol-1, kol1+1, -1, ' ', 1, ' ', 1, .false., ll, lgl, nrep, lerr )
               if ( lerr ) return
               ca   = car(kol+1:kol3-1)
            else
               nrep = 1
               ca   = car(kol1+1:kol3-1)
            end if
            if ( kode == 5 ) then
               do irep = 1, nrep
                  nx = nx + 1
                  if ( nx > maxx ) goto 100
                  cx(nx) = ca
               end do
            else
               read(ca,666) dval
               do irep = 1, nrep
                  nx = nx + 1
                  if ( nx > maxx ) goto 100
                  dx(nx) = dval
               end do
            end if

         end if

         ! Mise a jour pour analyse de l'element suivant
         kol1 = kol3
         car3 = sep1
         if ( lexit ) exit

      end do

   end if

   ! Effacement de la zone analysee
   if ( car(icol:icol) == sep2 ) then
      car(icol+1:jcol-1) = ' '
   else
      car(icol:jcol-1) = ' '
   end if

   return         ! Retour OK

   100 continue   ! Message d'erreur
   write(l6,*) mrker1
   write(l6,11) typer, car(icol:min(jcol,icol+64))
   write(l6,777)
   write(l6,*) mrker2
   900 continue   ! Retour en erreur
   lerr = .true.

   INCLUDE 'anaf11.'
   INCLUDE 'avectxyf.'
   INCLUDE 'avectyf.'

END SUBROUTINE avecty
