SUBROUTINE cmprss( ideb, iend )

!***********************************************************************
!     - cmprss -
!     compresse car (chaine de caracteres) et com (tableau logique)
!     (rappel: com(j) = .true. quand car(j:j) en commentaire) :
!     efface dans la sous-chaine car(ideb:iend) les "commentaires"
!     et les blancs, et remplace plusieurs sep2 successifs par un
!     seul (s'il ne reste qu'un seul caractere et que c'est sep2,
!     alors il est egalement supprime)
!     en entree ncar = nbre de caracteres "significatifs" de car
!     en sortie ncar = ncar - nbre caracteres effaces
!***********************************************************************
!
!     IDEB  - ENTREE. DEBUT DE LA ZONE A COMPRESSER           - ENTIER
!     IEND  - ENTREE. FIN   DE LA ZONE A COMPRESSER           - ENTIER
!
!-----------------------------------------------------------------------

   USE carl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ideb, iend

   !  Declarations locales
   !  --------------------

   INTEGER :: i, ia, j, jend, n, ncol
   LOGICAL :: lc, ls

   !-------------------------------------------------------------------

     ! jend donne la derniere colonne de la zone analysee,
     ! il change donc au fur et a mesure des effacements
     ! ia donne la colonne du debut d'un effacement

   ! Suppression des commentaires
   ! ----------------------------
   if ( ncar <= 0 ) return
   jend = iend
   lc   = .false.
   i    = ideb
   do
      if ( com(i) ) then
         if ( .not. lc ) then
            ia = i
            lc = .true.
         end if
      else if ( lc ) then     ! Compression
         ncol = ia - i + ncar
         car(ia:ncol) = car(i:ncar)
         do j = 0, ncar-i
            com(ia+j) = com(i+j)
         end do
         ncar = ncol
         jend = ia - i + jend
         i    = ia - 1
         lc   = .false.
      end if
      i = i + 1
      if ( i > jend ) exit
   end do

   if ( lc ) then   ! La derniere compression reste a faire
      if ( jend == ncar ) then
         ncar = ia - 1
         jend = ncar
      else
         ncol = ia + ncar - jend - 1
         car(ia:ncol) = car(jend+1:ncar)
         do j = 0, ncar-jend-1
            com(ia+j) = com(jend+1+j)
         end do
         ncar = ncol
         jend = ia - 1
      end if
   end if

   if ( jend < ideb ) return

   ! Suppression des blancs et remplacement de sep2 successifs
   ! ---------------------------------------------------------
   ! ou parmi des blancs par un seul sep2
   ! ------------------------------------
   n  = 0
   ia = 0
   lc = .false.
   ls = .false.
   i  = ideb
   do
      if ( car(i:i) == sep2 .or. car(i:i) == ' ' ) then
         if ( car(i:i) == sep2 ) ls = .true.
         n = n + 1
         if ( .not. lc ) then
            ia = i
            lc = .true.
         end if
      else
         if ( ls ) then
            car(ia:ia) = sep2
            n  = n - 1
            ia = ia + 1
            ls = .false.
         end if
         if ( n > 0 ) then      ! Compression
            ncol = ncar - n
            car(ia:ncol) = car(i:ncar)
            do j = 0, ncar - i
               com(ia+j) = com(i+j)
            end do
            ncar = ncol
            jend = jend - n
            i    = ia - 1
            n    = 0
         end if
         lc = .false.
      end if
      i = i + 1
      if ( i > jend ) exit
   end do

   if ( lc ) then
         ! La derniere compression reste a faire
         ! S'il ne doit rester qu'un caractere et que c'est sep2
         ! alors on le supprime aussi
      if ( ls .and. ia > ideb ) then
         car(ia:ia) = sep2
         n  = n - 1
         ia = ia + 1
      end if
      if ( n > 0 ) then
         if ( jend == ncar ) then
            ncar = ia - 1
         else
            ncol = ncar - n
            car(ia:ncol) = car(jend+1:ncar)
            do j = 0, ncar-jend-1
               com(ia+j) = com(jend+1+j)
            end do
            ncar = ncol
         end if
      end if
   end if

END SUBROUTINE cmprss
