SUBROUTINE leccom( cart, lcart, lprt, ilec, lrec, cfin, lfin, ifin, icom, kart, nread, eof, lerr )

!***********************************************************************
!     - leccom -
!     lecture de commandes avec suppression des blancs qui ne sont
!     pas entre "'" et des "'" delimiteurs.
!     les commandes sont rangees dans la chaine car (en common).
!     cfin = mot-cle indiquant la fin des commandes (provoque le
!     retour normal).
!     ce mot de fin est conserve dans car.
!     icom = rang iachar du caractere indiquant des commentaires: la
!     zone de cart a partir de ce caractere est ignoree.
!     ce caractere de commentaire n'est pas conserve dans car.
!     retour par * en cas d'erreur.
!***********************************************************************
!
!     CART  - ENTREE. UN ENREGISTREMENT SUR ILEC, DE LONGUEUR - CAR
!                     LREC (DEJA LU SI LCART=.TRUE.)
!           - SORTIE. DERNIER ENREGISTREMENT LU SUR ILEC
!     LCART - ENTREE. .TRUE. SI CART A ETE LU AVANT L'APPEL   - LOGIQUE
!                     DE A LECCOM
!     LPRT  - ENTREE. .TRUE. POUR IMPRIMER LES COMMANDES LUES - LOGIQUE
!     ILEC  - ENTREE. UNITE DE LECTURE DES COMMANDES          - ENTIER
!     LREC  - ENTREE. LONGUEUR DES ENREGISTREMENTS SUR ILEC   - ENTIER
!     CFIN  - ENTREE. MOT-CLE INDIQUANT LA FIN DES COMMANDES  - CAR
!     LFIN  - ENTREE. NOMBRE DE CARACTERES DU MOT-CLE DE FIN  - ENTIER
!     IFIN  - ENTREE. RANG "IACHAR" DU 1ER CARACTERE DU       - ENTIER
!                     MOT-CLE DE FIN                          - ENTIER
!     ICOM  - ENTREE. RANG "IACHAR" DU CARACTERE DE DEBUT     - ENTIER
!                     D'UNE ZONE COMMENTAIRE
!                     QUAND CE CARACTERE EST PRESENT TOUTE
!                     LA ZONE, A PARTIR DE CELUI-CI, DANS
!                     L'ENREGISTREMENT CART EN COURS EST
!                     IGNOREE (I.E. ON IGNORE CART(K:LREC)
!                     K DESIGNANT LA POSITION DU CARACTERE)
!                     POUR INTERDIRE CETTE POSSIBILITE, IL
!                     SUFFIT DE DONNER UNE VALEUR NEGATIVE
!     KART  - SORTIE. POSITION DANS CART DU DERNIER CARACTERE - ENTIER
!                     DU MOT-CLE DE FIN DES COMMANDES
!                     (0 SI FIN DE FICHIER ATTEINTE EN BATCH)
!     NREAD - SORTIE. NOMBRE D'ENREGISTREMENTS LUS SUR ILEC   - ENTIER
!                     AU COURS DE L'APPEL
!     EOF   - SORTIE. .TRUE. SI FIN DE FICHIER ATTEINTE EN    - LOGIQUE
!                      BATCH AVANT LE MOT-CLE DE FIN
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!                     (TROP DE COMMANDES OU FIN DE FICHIER
!                     EN BATCH ET "'" NON EQUILIBREES)
!
!         Module :
!     CAR   - ENTREE. CHAINE DE CARACTERES CONTENANT LES COMMANDES
!             SORTIE.
!     COM   - ENTREE. COM(I)=.TRUE. SI CAR(I:I) EST ENTRE APOSTROPHES
!             SORTIE.
!     NCAR  - ENTREE. NOMBRE DE CARACTERES DEJA ENTRES DANS CAR
!             SORTIE. NCAR + NOMBRE DE CARACTERES AJOUTES DANS CAR
!-----------------------------------------------------------------------

   USE carl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: ilec, lrec, lfin, ifin, icom
   LOGICAL, INTENT(IN)  :: lcart, lprt
   CHARACTER(LEN=*), INTENT(IN) :: cfin
   CHARACTER(LEN=*), INTENT(INOUT) :: cart
   INTEGER, INTENT(OUT) :: kart, nread
   LOGICAL, INTENT(OUT) :: eof, lerr

   !  Declarations locales
   !  --------------------

   INTEGER :: i, icf, j, jcf, ncf
   LOGICAL :: llcart, lqte, prec, blanc, lend
   INTEGER, PARAMETER :: iblan=IACHAR(" "), iquote=IACHAR("'")

   !--------------------------------------------------------------------

   kart  = 0
   nread = 0
   eof   = .false.
   lerr  = .false.
   lqte  = .false.
   prec  = .false.
   lend  = .false.
   if ( lprt ) write(l6,222)
   llcart = lcart

   ! Boucle sur les enregistrements de l'unite ilec
   ! ----------------------------------------------
   enreg: do
      ! Si cart a deja ete lu on saute sa lecture
      if ( llcart ) then
         llcart = .false.
      else
         read(ilec,'(a)',end=10) cart(:lrec)
         nread = nread + 1
      end if
      if (lprt) write(l6,*) cart(:lrec)

      ! Suppression des blancs sauf dans les chaines de caracteres
      ! lqte et com(i) = .true. si la colonne i est entre "'"
      ! prec = .true. si la colonne precedente est "'"
      ! lend = .true. si la colonne actuelle peut faire partie du
      !         mot-cle de fin

      ! Boucle sur les caracteres de l'enregistrement cart
      ! --------------------------------------------------
      do i = 1, lrec

         j = iachar( cart(i:i) )

         if ( j == iquote ) then

            lqte = .not. lqte
            if ( lend ) lend = .false.
            if ( prec ) then
               ! C'est un "'" double donc on le garde dans car
               ncar = ncar + 1
               if ( ncar > incar ) then
                  write(l6,*) mrker1
                  write(l6,111) incar
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               end if

               car(ncar:ncar) = cart(i:i)
               com(ncar) = lqte
               prec = .false.
            else
               prec = .true.
            end if

         else

            prec = .false.
            if ( .not. lqte ) then
               ! On est dans une zone "active"
               ! (si c'est un blanc on le saute)

               if ( j /= iblan ) then
                  ! Si c'est le debut d'une zone commentaire on
                  ! ignore la fin de l'enregistrement et on passe
                  ! a l'enregistrement suivant
                  if ( j == icom ) cycle enreg

                  ncar = ncar + 1
                  if ( ncar > incar ) then
                     write(l6,*) mrker1
                     write(l6,111) incar
                     write(l6,*) mrker2
                     lerr = .true.
                     return
                  end if

                  car(ncar:ncar) = cart(i:i)
                  com(ncar) = lqte
                  ! Possibilite du mot de fin ?
                  if ( lend ) then
                     ncf = ncf + 1
                     if ( ncf == lfin ) then
                        if ( car(icf:ncar) == cfin(:lfin) ) then
                           ! C'est fini, retour normal
                           kart = i
                           return
                          !------!
                        end if
                        ! Debut du mot de fin en icf+1,icf+2,...?
                        jcf = index( car(icf+1:ncar), cfin(1:1) )
                        if ( jcf <= 0 ) then
                           lend = .false.
                        else
                           icf = icf + jcf
                           ncf = ncf - jcf
                        end if
                     end if
                    ! Caractere de debut du mot de fin ?
                  else if ( j == ifin ) then
                            ! Si un seul caractere de fin:
                            ! c'est fini, retour normal
                            if ( lfin == 1 ) then
                               kart = i
                               return
                              !------!
                            end if
                            icf  = ncar
                            ncf  = 1
                            lend = .true.
                  end if

               end if

           else
               ! On est dans une zone entre quotes
               ncar = ncar + 1
               if ( ncar > incar ) then
                  write(l6,*) mrker1
                  write(l6,111) incar
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               end if

               car(ncar:ncar) = cart(i:i)
               com(ncar) = lqte
            end if

         end if

      end do

      ! Fin de l'examen de l'enregistrement en cours, lecture d'un nouvel enregistrement
   end do enreg

10 continue      ! Fin de fichier en batch
   eof = .true.
   ! Si les "'" ne sont pas apparies, on considere que c'est une erreur
   if ( lqte ) then
      write(l6,*) mrker1
      write(l6,333)
      write(l6,*) mrker2
      lerr = .true.
   end if

   INCLUDE 'leccomf.'

END SUBROUTINE leccom
