SUBROUTINE consec( x, n, out, maxi, nout, imax )

!***********************************************************************
!     - consec -
!     monotonically transforms a vector of integers into consecutive
!     integers starting from OUT+1 (ties are left tied).
!     Values lower than or equal to OUT on input are not transformed
!***********************************************************************
!
!     X     - INPUT . Vector of values to be transformed      - INTEGER
!             OUTPUT. Vector of transformed values
!     N     - INPUT . Length of X                             - INTEGER
!     OUT   - INPUT . Limit above which values are to be      - INTEGER
!                     transformed
!     MAXI  - INPUT . A value strictly greater than every     - INTEGER
!                     element in X
!     NOUT  - OUTPUT. Number of elements in X which are > OUT
!     IMAX  - OUTPUT. Maximum value after transformation      - INTEGER
!                     (obviously IMAX =< NOUT)
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        COUNT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n, out, maxi
   INTEGER, INTENT(INOUT) :: x(n)
   INTEGER, INTENT(OUT) :: nout, imax

   !  Local declarations
   !  ------------------

   INTEGER :: i, iout, mini

   !--------------------------------------------------------------------

   iout = out

   do    ! Loop while there is a gap
         ! (i.e. until a minimum > current iout is found)

         ! Search for a new minimum
      mini = maxi
      do i = 1, n
         if ( x(i) > iout .and. x(i) < mini ) mini = x(i)
      end do

      if ( mini < maxi ) then   ! Values of concern are decreased
         do i = 1, n
            if ( x(i) > iout ) x(i) = x(i) - mini + (iout+1)
         end do
         iout = iout + 1

      else                      ! End of transformations. Compute nout and exit
         imax = iout
         nout = count( x > out )
         exit

      end if

      ! The cycle is to be continued
   end do

END SUBROUTINE consec
