CHARACTER(LEN=6) FUNCTION dfmtb( nc, nn, x, zertol )

!***********************************************************************
!     - dfmtb -
!     format adjustment according to the value of X (dble prec. type):
!     . if 10**NN<|X|<10**(NC-1) (or <10**(NC-2) if X<0) F-type format
!     with NC characters: Fnc.m (m computed from the value of X)
!     . if not, E-type format with NC+4 characters: Enc+4.nc-2
!       (or Enc+4.nc-3 if X < 0)
!     The character string receiving DFMTB must be at least 6 characters
!     long and NC must be > 3
!***********************************************************************
!
!     NC     - INPUT . Number of characters for printing of X - INTEGER
!                      (must be > 3)
!     NN     - INPUT . (must be stictly lower than NC)        - INTEGER
!                      the upper limit for power of 10 under  - INTEGER
!                      which E-type format is used rather than
!                      F-type. Normally NN is negative and
!                      serves to avoid only zeros printed when
!                      X is small in absolute value (e.g. NN=-4
!                      means E-type is used if |X| < 10**-4 )
!                      NN is inactive if |X| < ZERTOL
!     X      - INPUT . The value to be printed                - REAL*8
!     ZERTOL - INPUT . A tolerance for zero : if |X| < ZERTOL - REAL*8
!                      X is printed with F-type format. This
!                      is usefull when one prefers having, e.g.
!                      3*10**-31, printed as 0.000000 rather
!                      than 0.3E-30
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  LOG10  MAX  SIGN
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: x, zertol
   INTEGER, INTENT(IN) :: nc, nn

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, onepls=1.0_8+1.0e-13_8
      ! onepls must be slightly greater than 1, in order to
      ! counter-balance such things as log10(100) = 1.99999,
      ! so that int( onepls + log10(100) ) = 3

   REAL(KIND=8) :: ax, algx
   INTEGER :: idec, ineg, lgx
   CHARACTER(LEN=6) :: temp
      ! Note that temp is used to avoid direct writing into dfmtb
      ! which causes some trouble with some compilers

   !--------------------------------------------------------------------

   if ( x < zero ) then
      ineg = 1
   else
      ineg = 0
   end if
   ax = abs( x )
   if ( ax < zertol ) then
      write(temp,1) 'F', nc, nc-2-ineg
   else
      algx = log10( ax )
      lgx  = algx + sign( onepls, algx )
      idec = max(1,lgx) + 1 + ineg
      if ( idec > nc. or. lgx+1 <= nn ) then
         write(temp,1) 'E', nc+4, nc-2-ineg
      else
         write(temp,1) 'F', nc, nc-idec
      end if
   end if
   dfmtb = temp

   1 format(a1,i2,'.',i2)

END FUNCTION dfmtb
