INTEGER FUNCTION ifromd( x, eps, maxint, ok, n )

!***********************************************************************
!     - ifromd -
!     converts a real*8 real number to the nearest integer
!***********************************************************************
!
!     X      - INPUT . The real number to be converted        - REAL*8
!                      No conversion will be done and zero
!                      will be returned if Abs(X) > MAXINT
!     EPS    - INPUT . The relative error used to compare X   - REAL*8
!                      to an integer (see OK below)
!     MAXINT - INPUT . The greatest possible integer          - INTEGER
!                      (in absolute value)
!     OK     - OUTPUT. .true. if X can be considered as an    - LOGICAL
!                      actual integer (i.e. only with 0's in
!                      its decimal part), that is if
!                      Abs(X-IFROMD) / Abs(X)  <=  EPS
!                      .false. otherwise
!     N      - OUTPUT. Number of digits of integer returned   - INTEGER
!                      Irrelevant if OK is .false.
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  LOG10  NINT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: x, eps
   INTEGER, INTENT(IN) :: maxint
   INTEGER, INTENT(OUT) :: n
   LOGICAL, INTENT(OUT) :: ok

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, onepls=1.0_8+1.0e-13_8
      ! onepls must be slightly greater than 1, in order to
      ! counter-balance such things as log10(100) = 1.99999,
      ! so that int( onepls + log10(100) ) = 3

   REAL(KIND=8) :: ax, bx

   !--------------------------------------------------------------------

   ok = .false.
   n  = 0
   ax = abs( x )
   bx = maxint

   if ( ax <= bx ) then
      ifromd = nint( x )
      if ( abs(x-ifromd) <= eps*ax ) then
         ok = .true.
         if ( abs(ifromd) > 0 ) then
            n = log10( ax ) + onepls
            if ( ifromd < 0 ) n = n + 1
         else
            n = 1
         end if
      end if
   else
      ifromd = zero
   end if

END FUNCTION ifromd
