SUBROUTINE mabc( a, b, c, l, m, n )

!***********************************************************************
!     - mabc -
!     multiplies general matrix A by general matrix B
!     to form resultant general matrix C
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     B     - INPUT . General matrix (columnwise)             - REAL*8
!     C     - OUTPUT. Resultant general matrix (columnwise)   - REAL*8
!     L     - INPUT . Number of rows in A and C               - INTEGER
!     M     - INPUT . Number of columns in A and rows in B    - INTEGER
!     N     - INPUT . Number of columns in B and C            - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: l, m, n
   REAL(KIND=8), INTENT(IN) :: a(l*m) , b(m*n)
   REAL(KIND=8), INTENT(OUT) :: c(l*n)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ij, ik, j, k, kj

   !--------------------------------------------------------------------

   ij = 0
   kj = -m

   do j = 1, n
      kj = kj + m

      do i = 1, l
         ik = i - l
         ij = ij + 1
         sumx = zero

         do k = 1, m
            ik = ik + l
            sumx = sumx + a(ik) * b(k+kj)
         end do

         c(ij) = sumx
      end do

   end do

END SUBROUTINE mabc
