SUBROUTINE masc( a, s, c, m, n )

!***********************************************************************
!     - masc -
!     multiplies general matrix A by symmetric matrix S (stored in
!     triangular form) to form resultant general matrix C
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     C     - OUTPUT. Resultant general matrix (columnwise)   - REAL*8
!     M     - INPUT . Number of rows in A and C               - INTEGER
!     N     - INPUT . Number of rows and columns in S,        - INTEGER
!                     of columns in A and C
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: a(m*n), s(n*(n+1)/2)
   REAL(KIND=8), INTENT(OUT) :: c(m*n)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, is, j, l, li, lj

   !--------------------------------------------------------------------

      ! Note: the elements of the triangle of the symmetric matrix
      ! are accessed in their sequential order

   do l = 1, m
      li = l
      is = 1
      c(l) = a(l) * s(1)

      do i = 2, n
         li = li + m
         lj = l - m
         c(li) = zero

         do j = 1, i-1
            lj = lj + m
            is = is + 1
            c(li) = c(li)  +  a(lj) * s(is)
            c(lj) = c(lj)  +  a(li) * s(is)
         end do

         is = is + 1
         c(li) = c(li)  +  a(li) * s(is)
      end do

   end do

END SUBROUTINE masc
