SUBROUTINE mastaz( a, s, z, m, n )

!***********************************************************************
!     - mastaz -
!     computes the product  A * S * transpose(A)  where A is a general
!     matrix, S a symmetric matrix (stored in triangular form) and
!     result is a symmetric matrix Z (stored in triangular form)
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     Z     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     M     - INPUT . Number of rows in A, of rows and        - INTEGER
!                     columns in Z
!     N     - INPUT . Number of columns in A, of rows and     - INTEGER
!                     columns in S
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: a(m*n), s(n*(n+1)/2)
   REAL(KIND=8), INTENT(OUT) :: z(m*(m+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ij, ik, imm, j, jj, jjj, jk, k
   REAL(KIND=8) :: wrk(n)

   !--------------------------------------------------------------------

   ij = 0

   do i = 1, m
      imm = i - m
      jj = 0

      do j = 1, n
         ik = imm
         jjj = jj + j
         sumx = zero

         do jk = jj+1, jjj
            ik = ik + m
            sumx = sumx + a(ik) * s(jk)
         end do

         jj = jjj
         jk = jj + j

         do k = j+1, n
            ik = ik + m
            sumx = sumx + a(ik) * s(jk)
            jk = jk + k
         end do

         wrk(j) = sumx
      end do

      do j = 1, i
         ij = ij + 1
         jk = j - m
         sumx = zero

         do k = 1, n
            jk = jk + m
            sumx = sumx + wrk(k) * a (jk)
         end do

         z(ij) = sumx
      end do

   end do

END SUBROUTINE mastaz
