SUBROUTINE matas( a, s, m, n )

!***********************************************************************
!     - matas -
!     post-multiplies general matrix A by its transpose to form
!     resultant symmetric matrix S (stored in triangular form)
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     S     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     M     - INPUT . Number of rows in A,                    - INTEGER
!                     of rows and columns in S
!     N     - INPUT . Number of columns in A                  - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: a(m*n)
   REAL(KIND=8), INTENT(OUT) :: s (m*(m+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx

   !--------------------------------------------------------------------

   nm = (n-1)*m
   is = 0

   do i = 1, m

      do j = 1, i
         sumx = zero

         do k = 0, nm, m
            sumx = sumx + a(i+k) * a(j+k)
         end do

         is = is + 1
         s(is) = sumx
      end do

   end do

END SUBROUTINE matas
