SUBROUTINE mrsz( r, s, z, n )

!***********************************************************************
!     - mrsz -
!     multiplies square matrix R by symmetric matrix S (stored in
!     triangular form) to form resultant symmetric matrix Z
!***********************************************************************
!
!     R     - INPUT . Square matrix (columnwise)              - REAL*8
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     Z     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     N     - INPUT . Order of R, S, Z                        - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n
   REAL(KIND=8), INTENT(IN) :: r(n*n), s(n*(n+1)/2)
   REAL(KIND=8), INTENT(OUT) :: z(n*(n+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ij, ik, j, jk, k, kj

   !--------------------------------------------------------------------

   ij = 0

   do i = 1, n
      jk = 0

      do j = 1, i
         ij  = ij + 1
         ik  = i
         kj  = j
         sumx = zero

         do k = 1, j
            sumx = sumx + r(ik) * s(jk+k)
            ik  = ik + n
            kj  = kj + k
         end do
         do k = j+1, n
            sumx = sumx + r(ik) * s(kj)
            ik  = ik + n
            kj  = kj + k
         end do

         z(ij) = sumx
         jk = jk + j
      end do

   end do

END SUBROUTINE mrsz
