SUBROUTINE mskiz( s, z, m, ii )

!***********************************************************************
!     - mskiz -
!     takes "direct" (Kroneker) product of symmetric matrix S (stored
!     in triangular form) by identity matrix to form resultant symmetric
!     matrix Z (stored in triangular form)
!***********************************************************************
!
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     Z     - OUTPUT. Resultant symmetric matrix, of order    - REAL*8
!                     M*II (in triangular form)
!     M     - INPUT . Order of S                              - INTEGER
!     II    - INPUT . Order of the identity matrix            - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------
!
!     Note: the "direct" product of S by I is taken as follows
!     (assuming, e.g., order of S is 3 and order of I is 2):
!        s(1,1)
!          0     s(1,1)
!        s(2,1)    0     s(2,2)
!          0     s(2,1)    0     s(2,2)
!        s(3,1)    0     s(3,2)    0     s(3,3)
!          0     s(3,1)    0     s(3,2)    0     s(3,3)
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, ii
   REAL(KIND=8), INTENT(IN) :: s(m*(m+1)/2)
   REAL(KIND=8), INTENT(OUT) :: z(m*ii*(m*ii+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, ij, i0, i1, j, k, l

   !--------------------------------------------------------------------

   do i = 1, m*ii*(m*ii+1)/2
      z(i) = zero
   end do

   ij = 0
   i1 = 0
   do i = 1, m
      i0 = i1*(i1+1)/2
      do j = 1, i
         ij = ij + 1
         k  = i0 + (j-1)*ii - i1
         do l = 1, ii
            k = k + i1 + l
            z(k) = s(ij)
         end do
      end do
      i1 = i1 + ii
   end do

END SUBROUTINE mskiz
