SUBROUTINE mssz( s, z, n )

!***********************************************************************
!     - mssz -
!     multiplies symmetric matrix S (stored in triangular form) by
!     itself to form resultant symmetric matrix Z (stored in triangular
!     form)
!***********************************************************************
!
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     Z     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     N     - INPUT . Number of rows and columns in S, Z      - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n
   REAL(KIND=8), INTENT(IN) :: s(n*(n+1)/2)
   REAL(KIND=8), INTENT(OUT) :: z(n*(n+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ij, ik, j, jk, k, ki, kj

   !--------------------------------------------------------------------

   ij = 0
   ik = 0

   do i = 1, n
      jk = 0

      do j = 1, i
         ij  = ij + 1
         kj  = j
         sumx = zero

         do k = 1, j
            sumx = sumx + s(ik+k) * s(jk+k)
            kj  = kj + k
         end do
         do k = j+1, i
            sumx = sumx + s(ik+k) * s(kj)
            kj  = kj + k
         end do
         ki = ik + i + i
         do k = i+1, n
            sumx = sumx + s(ki) * s(kj)
            ki  = ki + k
            kj  = kj + k
         end do

         z(ij) = sumx
         jk    = jk + j
      end do

      ik = ik + i
   end do

END SUBROUTINE mssz
