SUBROUTINE mszr( s, z, r, n )

!***********************************************************************
!     - mszr -
!     multiplies symmetric matrix S (stored in triangular form) by
!     symmetric matrix Z (stored in triangular form) to form resultant
!     square matrix R
!***********************************************************************
!
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     Z     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     R     - OUTPUT. Resultant square matrix (columnwise)    - REAL*8
!     N     - INPUT . Order of S, Z, R                        - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n
   REAL(KIND=8), INTENT(IN) :: s(n*(n+1)/2), z(n*(n+1)/2)
   REAL(KIND=8), INTENT(OUT) :: r(n*n)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ii, ij, ik, j, jj, k, kj, kk

   !--------------------------------------------------------------------

   ij = 0
   jj = 0

   do j = 1, n
      ii = 0

      do i = 1, n
         ij  = ij + 1
         kk  = 0
         sumx = zero

         do k = 1, n
            if( i .ge. k )then
               ik = ii + k
            else
               ik = kk + i
            endif
            if( k .ge. j )then
               kj = kk + j
            else
               kj = jj + k
            endif
            sumx = sumx + s(ik) * z(kj)
            kk  = kk + k
         end do

         r(ij) = sumx
         ii = ii + i
      end do

      jj = jj + j
   end do

END SUBROUTINE mszr
