SUBROUTINE mtadas( a, d, s, m, n )

!***********************************************************************
!     - mtadas -
!     computes the product  transpose(A) * D * A  where A is a general
!     matrix, D a vector representing a diagonal matrix
!     and result is a symmetric matrix S (stored in triangular form)
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     D     - INPUT . Vector of diagonal elements             - REAL*8
!     S     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     M     - INPUT . Number of rows in A, of terms in D      - INTEGER
!     N     - INPUT . Number of columns in A, of rows and     - INTEGER
!                     columns in S
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SUM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: a(m*n), d(m)
   REAL(KIND=8), INTENT(OUT) :: s(n*(n+1)/2)

   !  Local declarations
   !  ------------------

   INTEGER :: i, is, j, ki, kj

   !--------------------------------------------------------------------

   is = 0
   ki = -m

   do i = 1, n
      ki = ki + m
      kj = -m

      do j = 1, i
         kj  = kj + m
         is  = is + 1
         s(is) = sum( a(ki+1:ki+m) * a(kj+1:kj+m) * d(1:m) )
      end do

   end do

END SUBROUTINE mtadas
