SUBROUTINE norm1( x, ni, nj )

!***********************************************************************
!     - norm1 -
!     normalizes to unity a set of vectors
!***********************************************************************
!
!     X     - INPUT . The matrix of vectors,                  - REAL*8
!                     2-dimensioned (NI,NJ)
!                     (the vectors are stored as columns)
!             OUTPUT. Vectors of unit norm
!                     (null vectors are left unchanged)
!     NI    - INPUT . Number of coordinates of the vectors    - INTEGER
!                     (number of rows in X)
!     NJ    - INPUT . Number of vectors to normalize          - INTEGER
!                     (number of columns in X)
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SQRT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ni, nj
   REAL(KIND=8), INTENT(INOUT) :: x(ni,nj)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: a
   INTEGER :: j

   !--------------------------------------------------------------------

   do j = 1, nj
      a = sum( x(1:ni,j)**2 )
      if ( a > zero ) then
         a = one / sqrt( a )
         x(1:ni,j) = x(1:ni,j) * a
      end if
   end do

END SUBROUTINE norm1
