      CHARACTER*(*) FUNCTION RFMTB ( NC    , NN    , X     , ZERTOL )
!***********************************************************************
!     - RFMTB -
!     format adjustment according to the value of X (real type) :
!     . if 10**NN<|X|<10**(NC-1) (or <10**(NC-2) if X<0) F-type format
!     with NC characters : Fnc.m (m computed from the value of X)
!     . if not, E-type format with NC+4 characters : Enc+4.nc-2
!       (or Enc+4.nc-3 if X < 0)
!     The character string receiving RFMTB must be at least 6 characters
!     long and NC must be > 3
!***********************************************************************
!
!     NC     - INPUT . Number of characters for printing of X - INTEGER
!                      (must be > 3)
!     NN     - INPUT . (must be stictly lower than NC)        - INTEGER
!                      the upper limit for power of 10 under  - INTEGER
!                      which E-type format is used rather than
!                      F-type. Normally NN is negative and
!                      serves to avoid only zeros printed when
!                      X is small in absolute value (e.g. NN=-4
!                      means E-type is used if |X| < 10**-4 )
!                      NN is inactive if |X| < ZERTOL
!     X      - INPUT . The value to be printed                - REAL
!     ZERTOL - INPUT . A tolerance for zero : if |X| < ZERTOL - REAL
!                      X is printed with F-type format. This
!                      is usefull when one prefers having, e.g.
!                      3*10**-31, printed as 0.000000 rather
!                      than 0.3E-30
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  LOG10  MAX  SIGN
!
!-----------------------------------------------------------------------
!
!
!        ARGUMENTS
!        ---------
      REAL              X     , ZERTOL
      INTEGER           NC    , NN
!
!        LOCAL DECLARATIONS
!        ------------------
      REAL              AX    , ALGX  , ONEPLS, ZERO
      INTEGER           IDEC  , INEG  , LGX
      CHARACTER*6       TEMP
!
      PARAMETER (ONEPLS=1.0E0+1.0E-6, ZERO=0.0E0)
!        ONEPLS must be slightly greater than 1, in order to
!        counter-balance such things as log10(100) = 1.99999,
!        so that Int( ONEPLS + log10(100) ) = 3
!
!        Note that TEMP is used to avoid direct writing into RFMTB
!        which causes some trouble with some compilers
!
!     ------------------------------------------------------------------
!
      IF( X .LT. ZERO )THEN
         INEG = 1
      else
         INEG = 0
      end if
      AX = ABS( X )
      IF( AX .LT. ZERTOL )THEN
         WRITE(TEMP,1) 'F', NC, NC-2-INEG
      else
         ALGX = LOG10( AX )
         LGX  = ALGX + SIGN( ONEPLS, ALGX )
         IDEC = MAX(1,LGX) + 1 + INEG
         IF( IDEC .GT. NC. OR. LGX+1 .LE. NN )THEN
            WRITE(TEMP,1) 'E', NC+4, NC-2-INEG
         else
            WRITE(TEMP,1) 'F', NC, NC-IDEC
         end if
      end if
      RFMTB = TEMP
!
    1 FORMAT(A1,I2,'.',I2)
!
      END
