SUBROUTINE boum( nbreva, formul, cmmnt, x, nivop, adr, adrv, nomvar, lonvar, lonv, numv, lerr )

!***********************************************************************
!     - boum -
!     atomizes the formula, builds arrays  x, nivop, adr, adrv
!***********************************************************************
!
!     NBREVA- INPUT . Number of different variables           - INTEGER
!     FORMUL- INPUT . Formula (with delimiting apostrophs     - CHAR
!                     removed)
!     CMMNT - INPUT . A logical vector such that CMMNT(I) is  - LOGICAL
!                     .true. if FORMUL(I:I) was inbeded in
!                     apostrophs, and .false. otherwise
!     X     - OUTPUT. Values of atoms (dimension -1:MAXAT)    - REAL*8
!     NIVOP - OUTPUT. Actual hierarchical level of operators  - INTEGER
!                     in the formula (dimension MAXOP)
!     ADR   - OUTPUT. (dimension  4, MAXOP)                   - INTEGER
!                     ADR(1,.)  : codes of operators
!                     ADR(2,.)  : index in X of operators
!                     ADR(3/4,.): index in X of operands
!     ADRV  - OUTPUT. (dimension  2, MAXVA)                   - INTEGER
!                     ADRV(1,.) : index in NOMVAR of variables
!                     ADRV(2,.) : index in X of variables
!     NOMVAR- INPUT . Names of variables  (dimension NBREVA)  - CHAR
!     LONVAR- INPUT . Length of the names      "        "     - INTEGER
!     LONV  - WORK ARRAY of length NBREVA                     - INTEGER
!     NUMV  - WORK ARRAY of length NBREVA                     - INTEGER
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     opprm : natom   noper   nvarf
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        DNUM0   FTRIV   VARID1  VARID2
!     External functions called:
!        SAMSTR
!     Fortran  functions called:
!        ACHAR  IACHAR  INDEX   LEN     MAX     MIN
!
!-----------------------------------------------------------------------

   USE is_digit
   USE oplhn
   USE opprm

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nbreva
   CHARACTER(LEN=*), INTENT(IN) :: formul
   INTEGER, DIMENSION(nbreva), INTENT(IN) :: lonvar
   LOGICAL, DIMENSION(LEN(formul)), INTENT(IN) :: cmmnt
   CHARACTER(LEN=*), DIMENSION(nbreva), INTENT(IN) :: nomvar
   INTEGER, DIMENSION(nbreva), INTENT(INOUT) :: lonv, numv
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, DIMENSION(maxop), INTENT(OUT) :: nivop
   INTEGER, DIMENSION(4,maxop), INTENT(OUT) :: adr
   INTEGER, DIMENSION(2,maxva), INTENT(OUT) :: adrv
   REAL(KIND=8), DIMENSION(-1:maxat), INTENT(OUT) :: x

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: samstr
   REAL(KIND=8) :: dval
   INTEGER :: i, iact, icar, ik, ioksep, ipre, j, jcol, jj
   INTEGER :: joksep, kold, kolf, ll, lonf, mop, nivo
   LOGICAL :: lap, lop, ltexa, ltexo, ltexp
   CHARACTER(LEN=8) :: dollar='       $'
   CHARACTER(LEN=4) :: fmta='(A )'
   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER, PARAMETER :: imns=IACHAR("-"), ipls=IACHAR("+"), kblan=IACHAR(" ")


   ! LTEXA, LTEXO and LTEXP flag use of the special text function
   ! The argument of this special function is a character string
   ! (without any apostroph), and the real fonction value returned
   ! is the double precision representation of the 1st 8 characters
   ! of the character string (i.e. the argument read with An format
   ! where n is <= 8)
   ! ldigit is used to detect digits according to ASCII code

   !.................... USER MANAGEABLE SECTION .......................

   INTEGER, PARAMETER :: meme=2

   INCLUDE 'boump.'

   LOGICAL, DIMENSION(10,10) :: serr=reshape( (/                                       &
      .true. ,.true. ,.false.,.true. ,.false.,.false.,.false.,.true. ,.false.,.false., &
      .false.,.false.,.true. ,.true. ,.true. ,.true. ,.true. ,.false.,.true. ,.true. , &
      .false.,.false.,.true. ,.true. ,.true. ,.true. ,.true. ,.false.,.true. ,.true. , &
      .true. ,.true. ,.false.,.true. ,.false.,.false.,.false.,.true. ,.false.,.false., &
      .false.,.false.,.true. ,.true. ,.true. ,.true. ,.true. ,.false.,.true. ,.true. , &
      .true. ,.true. ,.false.,.true. ,.false.,.false.,.false.,.true. ,.false.,.true. , &
      .true. ,.true. ,.false.,.false.,.false.,.false.,.false.,.true. ,.false.,.false., &
      .false.,.false.,.true. ,.true. ,.true. ,.true. ,.true. ,.false.,.true. ,.true. , &
      .false.,.false.,.true. ,.true. ,.true. ,.true. ,.true. ,.false.,.true. ,.true. , &
      .true. ,.true. ,.true. ,.true. ,.false.,.true. ,.false.,.true. ,.false.,.true. /), &
                                              (/ 10, 10 /) )

   ! MEME  is the maximum number of operator names that can share
   ! a same prefix (e.g. SIN and SINH) (furthermore, it is assumed
   ! these names are contiguous in the list of the operator names)
   !
   ! IPLS and IMNS are used to distinguish unary and binary +
   ! and - (see below the detection of operators)
   !
   ! KOPER is array of row/column number in SERR of each operator;
   !          i.e. type of each operator. Note that special text
   !          operator acts as sign operators.
   ! COP   is array of messages associated to SERR.
   ! LCOP  is array of length of messages in COP.
   !
   ! SERR  is the matrix of errors for 2 successive elements in the
   ! formula. In row: type of previous element, in column: type
   ! of current element.
   ! SERR(i,j) is .true. if an i-type element may not precede a
   ! j-type element, and is .false. otherwise.
   ! The number of different types of elements is only 10 because
   ! constants and variables have same succession-rules.
   ! Order of rows (and columns) in SERR:
   ! constant (and variable); prefixed operator with 1 operand;
   ! operator with 2 operands (1 pre., 1 post.); postfixed operator;
   ! logical operator (except NOT); logical NOT operator;
   ! left parenthesis; right parenthesis; separator; sign.

   !................. END OF USER MANAGEABLE SECTION ...................

   !--------------------------------------------------------------------

   lerr = .false.

   lonf = len( formul ) - 1      ! Length of formula

   do j = 1, nbreva
      numv(j) = j
      lonv(j) = lonvar(j)
   end do
   ! Descending sort of variable-name lengths,
   ! numv(indices of variables) is reordered accordingly
   call ftriv( nbreva, lonv, numv )

   x(0)  = zero
   natom = 0
   noper = 0
   nvarf = 0
   nivo  = 0
   lap   = .false.
   ltexo = .false.
   ltexp = .false.
   ltexa = .false.
   ioksep= 0
   joksep= 0
   ik    = 0
   i     = 1
   iact  = 9
   ! ioksep & joksep are used to (roughly) check that the
   ! number of separators in the formula is correct

   do                                ! Loop on characters of formula
      ipre = iact
      icar = iachar( formul(i:i) )

      ! Is the preceding position the last one of a zone within
      ! apostrophes ?
      if ( lap .and. (.not. cmmnt(i) .or. i > lonf) ) then
         ll = i - kold
         if ( ll <= 0 ) then
            write(io,*) ewmrk1
            write(io,111) formul(max(1,i-7):min(i+7,lonf)), dollar(9-min(8,i):8)
            write(io,222)
            write(io,*) ewmrk2
            lerr = .true.
            return
         end if
         lap  = .false.
         kolf = i - 1
         if ( ltexo .and. ltexp ) then
            ! Preceding zone is the argument of the special text function
            ltexa = .true.
         else
            ! Preceding zone is supposed to be a variable name
            call varid1( nbreva, nomvar, lonvar, adrv, numv, formul(kold:kolf), ll, ik, lerr )
            if ( lerr ) return
         end if
         ! Now don't go back but proceede on current character
         ipre = 1
      end if

      ! End ?
      if ( i > lonf ) then
         if ( icar == kfin .and. .not. cmmnt(i) ) then
!!!!???     if ( lap ) then
!!!!???     end if
            iact = 9
            if ( serr(ipre,iact) ) goto 90
            ! Check parenthesis-level is 0
            if ( nivo /= 0 ) then
               write(io,*) ewmrk1
               write(io,333)
               write(io,*) ewmrk2
            else
               ! Check number of separators
               if ( ioksep == 0 .and. joksep == 0 ) return
               write(io,*) ewmrk1
               write(io,444)
               write(io,*) ewmrk2
            end if
         else
            write(io,*) ewmrk1
            write(io,555)
            write(io,*) ewmrk2
         end if
         lerr = .true.
         return
      end if

      ! Within delimiting apostrophes ?
      if ( cmmnt(i) ) then
         if ( .not. lap ) then
            iact = 1
            if ( serr(ipre,iact) ) goto 90
            lap  = .true.
            kold = i
         end if
         i = i + 1
         cycle
      end if

      ! The argument of the special text function ?
      if ( ltexo .and. ltexp ) then
         natom = natom + 1
         if ( natom > maxat ) then
            write(io,*) ewmrk1
            write(io,111) tooel
            write(io,*) ewmrk2
            lerr = .true.
            return
         end if
         if ( .not. ltexa ) then
            ! The right parenthesis delimits the end of the text
            kold = i
            do
               kolf = index( formul(i:lonf), achar(kpar2) ) + i - 1
               if ( kolf >= i .and. cmmnt(kolf) ) then
                  i    = kolf + 1
                  kolf = 0
                  if ( i <= lonf ) cycle
               end if
               exit
            end do
            if ( kolf-kold <= 0 ) then
               write(io,*) ewmrk1
               write(io,111) formul(max(1,kold-7):min(kold+7,lonf)), dollar(9-min(8,kold):8)
               write(io,333)
               write(io,666)
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            iact = 8
            nivo = nivo - nivpar
            i    = kolf + 1
            kolf = kolf - 1
         end if
         ! Store the text argument
         write(fmta(3:3),'(i1)') min(kolf-kold+1,8)
         read(formul(kold:kolf),fmta) x(natom)
         ltexo = .false.
         ltexp = .false.
         ! Go back for next character if right parenthesis already
         ! found; otherwise proceede to the analysis of current
         ! character which follows the text argument
         if ( .not. ltexa ) cycle
         ltexa = .false.
      end if

      if ( icar == kblan ) then                          ! Blank
         i = i + 1

      else if ( icar == kpar1 ) then                     ! Left parenthesis
         iact = 7
         if ( serr(ipre,iact) ) goto 90
         ltexp = ltexo
         nivo  = nivo + nivpar
         i     = i + 1

      else if ( icar == kpar2 ) then                     ! Right parenthesis
         iact = 8
         if ( serr(ipre,iact) ) goto 90
         nivo = nivo - nivpar
         i    = i + 1

      else if ( ldigit(icar) .or. icar == kpnt ) then    ! Constant
         iact = 1
         if ( serr(ipre,iact) ) goto 90
         call dnum0( io, ewmrk1, ewmrk2, formul, i, lonf, jcol, dval, lerr )
         if ( lerr ) return
         natom = natom + 1
         if ( natom > maxat ) then
            write(io,*) ewmrk1
            write(io,111) tooel
            write(io,*) ewmrk2
            lerr = .true.
            return
         end if
         x(natom) = dval
         i = jcol

      else if ( icar == ksep ) then                      ! Separator
         iact = 9
         if ( serr(ipre,iact) ) goto 90
         if ( ioksep <= 0 .or. joksep <= 0 ) then
            write(io,*) ewmrk1
            write(io,111) formul(max(1,i-7):min(i+7,lonf)), dollar(9-min(8,i):8)
            write(io,777)
            write(io,*) ewmrk2
            lerr = .true.
            return
         end if
         ioksep = ioksep - 1
         joksep = joksep - nivo
         i = i + 1

      else                                               ! Operator ?
         ! Once it has been found, we look at the meme-1 following
         ! ones to see if they have same prefix (such operators
         ! must be contiguous in the list of operators)
         lop = .false.
         jj  = 9999
         do j = idbop, ifnop
            if ( samstr( formul(i:min(i+lonop(j)-1,lonf)), nomop(j)(:lonop(j)) ) ) then
               lop = .true.
               jj  = j
            end if
            if ( j+1 >= jj+meme ) exit
         end do
         if ( lop ) then
            ! It is operator no. jj
            ! Distinction of unary + - (signs) :
            if ( (icar == imns .or. icar == ipls) .and. (ipre == 9 .or. ipre == 7 .or. ipre == 5) ) &
               jj = nbreop - (icar/imns)
            iact = koper(jj)
            if ( serr(ipre,iact) ) goto 90
            ! Expected number of separators is updated
            mop    = max( 0, nopop(jj)-1 )
            ioksep = ioksep + mop
            joksep = joksep + mop*(nivo+nivpar)

            natom = natom + 1
            noper = noper + 1
            if ( natom > maxat ) then
               write(io,*) ewmrk1
               write(io,111) tooel
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            if ( noper > maxop ) then
               write(io,*) ewmrk1
               write(io,111) tooop
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            x(natom)     = zero
            adr(2,noper) = natom
            adr(1,noper) = jj
            nivop(noper) = hieop(jj) + nivo
            ltexo = jj == itxop
            i     = i + lonop(jj)

         else                                            ! Variable ?
            ik = i
            call varid2( nbreva, nomvar, lonvar, adrv, numv, formul(i:), lonf, ik, lerr )
            if ( lerr ) return
            iact = 1
            if ( serr(ipre,iact) ) goto 90
            i = ik
         end if
      end if

   end do

   ! Error in succession of elements
90 continue
   write(io,*) ewmrk1
   write(io,111) formul(max(1,i-7):min(i+7,lonf)), dollar(9-min(8,i):8)
   write(io,888) cop(iact)(:lcop(iact)), cop(ipre)(:lcop(ipre))
   write(io,*) ewmrk2
   lerr = .true.

   INCLUDE 'boumf.'

END SUBROUTINE boum
