SUBROUTINE dnum0( io, m1, m2, car, i1, i2, ind, dval, lerr )

!***********************************************************************
!     - dnum0 - (a simplified version of subroutine dnum from anacom)
!     analyzes a real*8 number, reading from left to right only.
!     The value is stored in mem and then reread with fortran format
!     fmtc (f75.0) -> same value as classical fortran reading
!     dnum0 can interpret reals with e or d exponents
!***********************************************************************
!
!     io    - INPUT . Unit number for error message outputs   - INTEGER
!     m1    - INPUT . Mark for begining of message            - CHAR
!     m2    - INPUT . Mark for end of message                 - CHAR
!     car   - INPUT . Character string containing number      - CHAR
!     i1    - INPUT . Beginning column in CAR of number       - INTEGER
!     i2    - INPUT . Ending column in CAR of analyzed zone   - INTEGER
!                     ( i2 > i1 )
!     ind   - OUTPUT. Column immediately following number     - INTEGER
!     dval  - OUTPUT. Returned value                          - REAL*8
!     lerr  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        IACHAR  MAX  MIN
!
!-----------------------------------------------------------------------

   USE is_digit

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: io, i1, i2
   CHARACTER(LEN=*), INTENT(IN) :: m1, m2, car
   INTEGER, INTENT(OUT) :: ind
   REAL(KIND=8), INTENT(OUT) :: dval
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER :: i, ii1, im1, j, m
   LOGICAL :: lpoint, lexp, scan
   INTEGER, PARAMETER :: imns=IACHAR('-'), ipls=IACHAR('+'), ipnt=IACHAR('.')
   INTEGER, PARAMETER :: ieex=IACHAR('E'), idex=IACHAR('D')
   INTEGER, PARAMETER :: jeex=IACHAR('e'), jdex=IACHAR('d')

   INTEGER, PARAMETER :: maxc=75, maxc1=maxc+1
   ! maxc = maximum number of characters used to write a number
   !        (format fmtc must be set accordingly)
   CHARACTER(LEN=maxc) :: mem
   CHARACTER(LEN=8) :: fmtc='(F75.0) '

   INCLUDE 'dnum0p.'

   !--------------------------------------------------------------------

   lerr = .false.

   if ( i2 < i1 ) then
      write(io,*) m1
      write(io,111) nozone
      write(io,*) m2
      lerr = .true.
      return
   end if
   mem = ' '
   ii1 = i1
   m   = maxc
   lexp   = .false.
   lpoint = .false.
   scan   = .true.

   do i = i1, i2
      j = iachar(car(i:i))
      if ( ldigit(j) ) then  ! The character is a digit
         m = m - 1
      else
         if ( i > ii1 .or. (j /= imns .and. j /= ipls) ) then
            if ( .not. lpoint .and. j == ipnt ) then   ! Decimal point
               lpoint = .true.
            else
               ! Take account of exponent if any
               if ( .not. lexp .and. (j == ieex .or. j == idex .or. j == jeex .or. j == jdex) ) then
                  lexp = .true.
                  ii1  = i + 1
               else
                  ind  = i
                  scan = .false.
                  exit
               end if
            end if
         end if
      end if
   end do
   if ( scan ) ind = i2 + 1
   if ( m == maxc ) then
      im1 = max(1,i1-1)
      write(io,*) m1
      write(io,111) car(min(i1,i2):), nonumb, car(im1:im1)
      write(io,*) m2
      lerr = .true.
      return
   end if

   ! The value in car(i1:ind-1) is stored and read in mem
   mem(maxc1-ind+i1:maxc) = car(i1:ind-1)
   read(mem,fmtc) dval

   INCLUDE 'dnum0f.'

END SUBROUTINE dnum0
