SUBROUTINE gmpads( pend, send, k1 )

!***********************************************************************
!     - gmpads -
!     adds last end-of-sentence symbol in command paragraphs, if missing
!***********************************************************************
!
!     PEND  - INPUT . End of paragraph symbol                 - CHAR
!     SEND  - INPUT . End of sentence symbol                  - CHAR
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed (all paragraphs in CAR, from
!                     K1 to NCAR, are treated)
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl : car
!     lcar : ncar    com
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        LOCAL
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX
!
!-----------------------------------------------------------------------

   USE carl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1
   CHARACTER(LEN=1), INTENT(IN) :: pend, send

   !  Local declarations
   !  ------------------

   INTEGER :: i, ioc, i1, j
   INTEGER, DIMENSION(1) :: loc

   ! The loc array is a pseudo array only used to ensure consistency
   ! with arguments of routine local from ANACOM7

   !--------------------------------------------------------------------

   i1 = max( k1, 2 )

   do         ! Do while pend is found
      if ( i1 > ncar .or. ncar >= incar ) return

      call local( pend, 1, i1, ncar, .false., 1, ioc, loc )
      if ( ioc <= 0 ) return

      j = loc(1)
         ! N.B. if successive pend's, only the 1st one is considered
      if ( car(j-1:j-1) /= send .and. car(j-1:j-1) /= pend ) then
         do i = ncar, j, -1
            car(i+1:i+1) = car(i:i)
            com(i+1)     = com(i)
         end do
         car(j:j) = send
         com(j)   = .false.
         ncar     = ncar + 1
         j        = j + 1
      end if

      i1 = j + 1
   end do

END SUBROUTINE gmpads
