SUBROUTINE gmpcom( k1, kn, lerr, maxr )

!***********************************************************************
!     - GMPCOM -
!     analyzes COMM paragraph, used to change input file of commands,
!     screen width, output page width and height
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!     MAXR  - INPUT . Maximum record length of input file of  - INTEGER
!                     commands
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lec     lrecl   lrecp   linep   lines
!     gmplgl : linter
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CHKUNT  CMPRSS  CSCALX  CVUCT  FILOPN  GMPWRN
!     External functions called:
!        ISCALX  LSCALX
!     Fortran  functions called:
!        ADJUSTL  LEN_TRIM  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpint
   USE gmplgl
   USE systm
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE chkunt( lerr,iout,iunt,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12 )
         IMPLICIT NONE
         INTEGER, INTENT(IN) :: iout, iunt
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11, u12
         LOGICAL, INTENT(OUT) :: lerr
      END SUBROUTINE chkunt
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1, maxr
   INTEGER, INTENT(INOUT) :: kn
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: iscalx
   LOGICAL, EXTERNAL :: lscalx

   INTEGER :: i, icof, icol, icou, iunt, jcol, kolp
   INTEGER :: lek, lenc, linp, ncol, nkar, nladd
   LOGICAL :: lrwd, newp
   CHARACTER(LEN=72) :: filcmd
   CHARACTER(LEN=7)  :: tmpcmd
   CHARACTER(LEN=7), PARAMETER :: console='CONSOLE'

   INCLUDE 'gmpcomp.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc

   !--------------------------------------------------------------------

   lerr = .false.

      ! Name of new command file
   filcmd = console
   call cscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, filcmd, icof, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      return
   end if
   filcmd = adjustl( filcmd )
   lenc = len_trim( filcmd )
      ! Convert file name to capitals if not unix system
   if ( .not. lunix .and. icof > 0 .and. lenc > 0 ) call cvuct( filcmd )

   linter = lscalx( motc(3), lngm(3,1), lngm(3,2), k1, kn, linter, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      return
   end if
   if ( icol > 0 ) then
      if ( linter ) then
         lrecl = ncsci
         lrecp = ncsco
         linep = nlscr - nprmpt
      else
         lrecp = ncprt
         linep = nlprt
      end if
   end if

      ! Unit number for new command file
   iunt = iscalx( motc(4), lngm(4,1), lngm(4,2), k1, kn, lec1, icou, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
      return
   end if

      ! Record length of command file
   ncol = iscalx( motc(5), lngm(5,1), lngm(5,2), k1, kn, lrecl1, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
      return
   end if
   lrecl = min( ncol, maxr )
   if ( lrecl <= 0 ) lrecl = lrecl1

      ! Number of columns per page on output
   kolp = iscalx( motc(6), lngm(6,1), lngm(6,2), k1, kn, ncprt, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(6:), nmotc-5 )
      return
   end if
   if ( icol > 0 .and. kolp >= 79 ) lrecp = kolp

      ! Number of lines per page on output
   linp = iscalx( motc(7), lngm(7,1), lngm(7,2), k1, kn, nlprt, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(7:), nmotc-6 )
      return
   end if
   if ( icol > 0 .and. linp >= 10 ) then
      linep = linp
      if ( linter ) linep = linep - nprmpt
   end if

   lrwd = lscalx( motc(8), lngm(8,1), lngm(8,2), k1, kn, .false., icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(8:), nmotc-7 )
      return
   end if

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   ! If given, update unit number for input file of commands and open it
   ! -------------------------------------------------------------------
   if ( icof > 0 .or. icou > 0 ) then
      lek = lec   ! Save old unit number (to be restored in case of error)
      tmpcmd = filcmd(:7)
      if ( lunix ) call cvuct( tmpcmd )
      if ( iunt == lec1 .or. tmpcmd == console ) then
            ! Standard input (keyboard if interactive)
         if ( lec /= lec1 ) close (lec)
         if ( icou > 0 .or. tmpcmd == console ) then
            lec = lec1
         else
            lec = lec2
            call filopn( 12, lec, filcmd, 'OLD', 'FORMATTED', .true., lerr )
            if ( lerr ) then
               lec = lek      ! Restore previous unit number
               return
            end if
            if ( lrwd ) rewind lec
         end if
      else
            ! Alternate input
            ! Check unit number is permitted
         call chkunt( lerr, imp, iunt, iu1, iu2, iu3, iu4, imp, ksave )
         if ( lerr ) return
         if ( lec /= lec1 ) close (lec)
         lec = iunt
         call filopn( 12, lec, filcmd, 'OLD', 'FORMATTED', .true., lerr )
         if ( lerr ) then
            lec = lek      ! Restore previous unit number
            return
         end if
         if ( lrwd ) rewind lec
      end if
   end if

   INCLUDE 'gmpf111.'

END SUBROUTINE gmpcom
