SUBROUTINE gmpdir( card, lcmdl, lcmd1, lprt, fltrp, kstop )

!***********************************************************************
!     - gmpdir -
!     directs analysis of general paragraphs
!     (COMM, PROB, OBJ, SAUV, CATEG, TRAN)
!***********************************************************************
!
!     CARD  - INPUT . A record on LEC, already READ if LCMD1  - CHAR
!                     is .true.
!             OUTPUT. Last record read on LEC
!     LCMDL - INPUT . .true. if the complete set of commands  - LOGICAL
!                     has already been read
!     LCMD1 - INPUT . .true. if the first record of commands  - LOGICAL
!                     has already been read (in CARD)
!     LPRT  - INPUT . If .true. commands read are printed     - LOGICAL
!     FLTRP - INPUT . If .true. commands are checked for      - LOGICAL
!                     unused or unrecognized paragraphs
!     kstop - OUTPUT. Error code, also  indicating if the     - INTEGER
!                     program is to be stopped immediately on
!                     return from this subroutine.
!                      0 = no error
!                     >0 = no error, stop execution
!                     <0 = error, stop execution
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car     ncar    com
!     gmpint : lines   nvarid  nvarmx  nvaru   nvmxsp
!              nvust   label   ifrqcy  nvsav
!     gmplgl : ltran
!     memst1 : all
!     memsta : all
!     trancb : nbrefo  xtra to lcond
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG   CVUCS   CVUCT   FILTER  GMPADS  GMPCAT  GMPCMD
!        GMPCOM  GMPEXT  GMPGFM  GMPGFP  GMPINI  GMPOBJ  GMPPRO
!        GMPSAV  LOCAL   TRANSA  TRANSN
!     External functions called:
!        NAMAB
!     Fortran  functions called:
!        ACHAR  ALLOCATE  ALLOCATED  DEALLOCATE  LEN  MAX  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memsta
   USE oplhn
   USE opprm
   USE spcnom
   USE spcval
   USE termnl
   USE trancb

   IMPLICIT NONE

   !  Arguments
   !  ---------

   LOGICAL, INTENT(IN) :: lcmdl, lcmd1, lprt, fltrp
   CHARACTER(LEN=*), INTENT(INOUT) :: card
   INTEGER, INTENT(OUT) :: kstop

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: namab

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, ikend, ioc, j
   INTEGER :: ka, kard, karn, karp, kar1, kb, kn, ktn, kt1, k1, k3
   INTEGER :: lto, maxr, mfrqcy, mlabel, mvarmx
   INTEGER :: nend, nkar, nladd, nrrd, nvals, nvartp, nvarts, nvartt, status
   LOGICAL :: leof, lerc, lero, lerr, lers, llcmdl, lprtb, lprti, lrd1, newp, oldvrs
   CHARACTER(LEN=16), PARAMETER :: blank='                '
   INTEGER, DIMENSION(10) :: loca

   INCLUDE 'gmpdirp.'

   ! Remark about the TRAN paragraph.
   ! Some difficulties may arise with conversion from lower to
   ! upper case letters when the special text function is used:
   ! The argument of this function must not be converted, but all
   ! other characters must be. So the conversion task is not
   ! straightforward, and to simplify it, it's here assumed that
   ! the name of the text function includes the equal sign but no
   ! letter.

   !--------------------------------------------------------------------

   kstop = 0

   maxr = len( card )   ! Maximum record length of input command file

      ! Copy gmpint elements to avoid double argument transmission when calling gmpcmd
   nend  = nfin
   ikend = ikfin

   mvarmx = nvarmx      ! nvarmx from previous problem

   lrd1  = lcmd1
   lprtb = lprt .and. .not.linter
   lprti = lprt .and. linter
   lerr  = .false.

   lto = lonop(itxop)

   llcmdl = lcmdl

   ! Loop in case of error in PROB (or COMM) and interactive run
   ! ----
10 continue

   ! Read paragraphs entered
   ! -----------------------
   if ( llcmdl ) then   ! If commands already read, skip reading
      llcmdl = .false.
   else
      lerr = .false.
      ncar = 0
      call gmpcmd( card, lrd1, lprtb, pfin, nend, ikend, kard, nrrd, leof, lerr )
      if ( lerr ) then
         write(lgu,*) ermrk1
         write(lgu,555)
         write(lgu,*) ermrk2
         kstop = -1
         return                                  !--- stop error ---!
      end if
      lrd1 = .false.
      if ( leof ) then
         write(lgu,*) ermrk1
         write(lgu,888)
         write(lgu,*) ermrk2
         kstop = -1
         return                                  !--- stop e-o-f ---!
      end if

   end if

   k1 = 1
   do
      if ( car(k1:k1) /= cfin ) exit
      k1 = k1 + 1
   end do

      ! Add last end-of-sentence symbol in paragraphs if missing
   call gmpads( cfin, sep2, k1 )

      ! Selective conversion of commands (keywords) to capitals
      ! throughout all paragraphs (keyword values not converted)
   call cvucs( car(k1:ncar), com(k1:ncar), '=', sep2 )

   ! Special COMM paragraph ?
   ! ------------------------
   k3 = k1 + mpcomm - 1
   if ( car(k1:k3) == pcomm(:mpcomm) ) then
      kn = ncar - nfin

         ! Print commands read if requested and interactive run
      if ( lprti ) then
         nladd = 3 + (kn-k1)/lrecp
         call chkpg( nladd, .false., newp )
         write(imp,777) pcomm(:mpcomm), msginp
         write(imp,*) car(k1:kn)
         lines = lines + nladd
      else
!!       call chkpg( 2, .false., newp )
         write(lgu,777) pcomm(:mpcomm)
!!       lines = lines + 2
      end if

      car(k1:k3) = blank(:mpcomm-1) // sep2
         ! Analyze COMM paragraph
      kn = kn + 1
      call gmpcom( k3, kn, lerr, maxr )
      lprtb = lprt .and. .not.linter
      lprti = lprt .and. linter
      if ( lerr ) then
         if ( linter .and. lec == lec1 ) then
               ! Clear screen
            call chkpg( 0, .true., newp )
            write(imp,666)
            write(imp,444) pfin(:nfin)
            lines = lines + 2
            goto 10         ! Back to paragraphs reading
         end if
         write(lgu,*) ermrk1
         write(lgu,555)
         write(lgu,*) ermrk2
         kstop = -1
         return                                  !--- stop error ---!
      end if
         ! Go back to read the paragraphs
      if ( linter .and. lec == lec1 ) then
            ! Prompt user
         call chkpg( 0, .true., newp )
         write(imp,4444)
         write(imp,444) pfin(:nfin)
         lines = lines + 2
      end if
      goto 10               ! Back to paragraphs reading

   end if

   ! No request ?
   ! ------------
   if ( car(k1:k1+nfin-2) == pfin(2:nfin) ) then
         ! Go back to read the paragraphs
      if ( linter .and. lec == lec1 ) then
            ! Prompt user
         call chkpg( 0, .true., newp )
         write(imp,4444)
         write(imp,444) pfin(:nfin)
         lines = lines + 2
      end if
      goto 10               ! Back to paragraphs reading
   end if

   ! End of run ?
   ! ------------
   if ( car(k1:k1+mpstop-1) == pstop(:mpstop) ) then
      write(imp,3333)
      kstop = 1
      return                                     !=== stop normal ===!
   end if

      ! PROB paragraph is expected
   if ( car(k1:k1+mpprob-1) /= pprob(:mpprob) ) then
      write(lgu,*) ermrk1
      write(lgu,2222)
      write(lgu,*) ermrk2
      if ( .not. linter .or. lec /= lec1 ) then
         kstop = -1
         return                                  !--- stop error ---!
      end if
         ! Clear screen
      call chkpg( 0, .true., newp )
      write(imp,999)
      write(imp,444) pfin(:nfin)
      lines = lines + 5
      goto 10               ! Back to paragraphs reading
   end if

   ! Loop when aborting current problem and turning to a new one
   ! ----
20 continue

   ! Analysis of PROB paragraph
   ! --------------------------
   kar1 = namab( car, pprob, npprob, mpprob, k1 )
      ! Search for the end of the paragraph
   nkar = ncar
   call local( cfin, 1, kar1+1, nkar, .false., 1, ioc, loca )
   karn = loca(1)

      ! Print commands read if requested and interactive run
   if ( lprti ) then
      nladd = 3 + (karn-k1)/lrecp
      call chkpg( nladd, .false., newp )
      write(imp,777) pprob(:mpprob), msginp
      write(imp,*) car(k1:karn)
      lines = lines + nladd
   else
!!    call chkpg( 2, .false., newp )
      write(lgu,777) pprob(:mpprob)
!!    lines = lines + 2
   end if

      ! Analysis with filtering
   car(k1:kar1) = blank(:kar1-k1) // sep2
   call gmppro( k1, karn, lerr )
   if ( lerr ) then
      if ( .not. linter .or. lec /= lec1 ) then
         kstop = -1
         return                                  !--- stop error ---!
      end if
         ! Clear screen
      call chkpg( 0, .true., newp )
      write(imp,999)
      write(imp,444) pfin(:nfin)
      lines = lines + 5
      goto 10               ! Back to paragraphs reading
   end if

   if ( .not. lcode ) then

      ! Input from standard (non-gmp) file: allocate mem1 and mema arrays
      ! -----------------------------------------------------------------
      nvarid = nvarin + nvadd
      nvarmx = max( nvarin, nvarid )
      nvmxsp = nvarmx + nvarsp
      nvmxst = nvmxsp + nvtemp

      l_xmis = nvarmx
      l_xmin = nvarmx
      l_xmax = nvarmx
      l_vuse = nvarmx
      l_lval = nvarmx
      l_lnom = nvarmx
      l_vsav = nvarmx
      l_valpha = nvarmx
      l_vals = 0
      l_vnom = nvmxst
      l_unom = nvarmx
      l_noms = 0

      if ( allocated(xmis) ) deallocate(xmis)
      if ( allocated(xmin) ) deallocate(xmin)
      if ( allocated(xmax) ) deallocate(xmax)
      if ( allocated(vuse) ) deallocate(vuse)
      if ( allocated(lval) ) deallocate(lval)
      if ( allocated(lnom) ) deallocate(lnom)
      if ( allocated(vsav) ) deallocate(vsav)
      if ( allocated(vals) ) deallocate(vals)
      if ( allocated(valpha) ) deallocate(valpha)
      if ( allocated(vnom) ) deallocate(vnom)
      if ( allocated(unom) ) deallocate(unom)
      if ( allocated(noms) ) deallocate(noms)
      allocate( xmis(l_xmis), xmin(l_xmin), xmax(l_xmax),   &
                vuse(l_vuse), lval(l_lval), lnom(l_lnom),   &
                vsav(l_vsav), valpha(l_valpha),             &
                vnom(0:l_vnom), unom(l_unom), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpDir_1)."
         write(lgu,*) ermrk2
         kstop = -2
         return                                  !--- stop error ---!
      end if

         ! Initialize
      call gmpini( vnom, xmis, xmin, xmax, vuse, lval, lnom, vsav, valpha, nvarmx, nvmxst )

   else

      ! Mem1 and mema already built or read from a new gmp file
      ! -------------------------------------------------------
      if ( lgmpf ) then   ! mem1, mema already built from previous gmp file
                          ! (thus nprob is > 1 and mvarmx exists)
         label  = mlabel
         ifrqcy = mfrqcy
         nvarid = nvarin + nvadd
         nvarmx = max( nvarin, nvarid )
         nvmxsp = nvarmx + nvarsp
         nvmxst = nvmxsp + nvtemp

      else                ! New gmp file on input
            ! Read parameters
         if ( lrwde ) rewind inpux
         call gmpgfp( oldvrs, mvarmx, mlabel, mfrqcy, lerr )
         if ( lerr ) then
            if ( linter .and. lec == lec1 ) then
                  ! Clear screen
               call chkpg( 0, .true., newp )
               write(imp,333)
               write(imp,444) pfin(:nfin)
               lines = lines + 2
               goto 10      ! Back to paragraphs reading
            end if
            write(lgu,*) ermrk1
            write(lgu,555)
            write(lgu,*) ermrk2
            kstop = -1
            return                               !--- stop error ---!
         end if
         label  = mlabel
         ifrqcy = mfrqcy
         nvarid = nvarin + nvadd
         nvarmx = max( nvarin, nvarid )
         nvmxsp = nvarmx + nvarsp
         nvmxst = nvmxsp + nvtemp
            ! Allocate mem1 and mema arrays
         if ( allocated(xmis) ) deallocate(xmis)
         if ( allocated(xmin) ) deallocate(xmin)
         if ( allocated(xmax) ) deallocate(xmax)
         if ( allocated(vuse) ) deallocate(vuse)
         if ( allocated(lval) ) deallocate(lval)
         if ( allocated(lnom) ) deallocate(lnom)
         if ( allocated(vsav) ) deallocate(vsav)
         if ( allocated(vals) ) deallocate(vals)
         if ( allocated(valpha) ) deallocate(valpha)
         if ( allocated(vnom) ) deallocate(vnom)
         if ( allocated(unom) ) deallocate(unom)
         if ( allocated(noms) ) deallocate(noms)
         allocate( xmis(l_xmis), xmin(l_xmin), xmax(l_xmax),   &
                   vuse(l_vuse), lval(l_lval), lnom(l_lnom),   &
                   vsav(l_vsav), valpha(l_valpha),             &
                   vals(l_vals),                               &
                   vnom(0:mvarmx+nvarsp+nvtemp), unom(l_unom), &
                   noms(l_vals), stat=status )
         if ( status /= 0 ) then   ! fatal error
            write(lgu,*) ermrk1
            write(lgu,101) " (GmpDir_2)."
            write(lgu,*) ermrk2
            kstop = -2
            return                                  !--- stop error ---!
         end if
            ! Read mem1 and mema arrays
         call gmpgfm( mvarmx, l_vals, xmis, xmin, xmax, lval, lnom, valpha, vals, vnom, noms )
            ! Set the special "no variable" name to blank and
            ! add special variable names
         vnom(0) = ' '
         vnom(mvarmx+1) = vstat
         vnom(mvarmx+2) = vrang
         vnom(mvarmx+3) = vmanq
      end if

         ! If needed, extension/compression of mem1 and mema arrays
      if ( nvarmx /= mvarmx ) then
         call gmpext( mvarmx, lerr )
         if ( lerr ) then
            kstop = -3
            return                               !--- stop error ---!
         end if
      end if
         ! Set vuse and vsav to default
      do i = 1, nvarmx
         vuse(i) = i
         vsav(i) = i
      end do
   end if

   nvaru = nvarmx
   nvust = nvmxsp
   nvsav = nvarid
   nvals = l_vals

                        !-------------!

      ! Throughout the rest of the routine, karp is starting location
      ! in car for searching of OBJ, SAUV, CATEG, TRAN paragraphs
      ! ktn is ending location of TRAN paragraph
   karp = k1
   ktn  = 0

   lero = .false.
   lers = .false.
   lerc = .false.

   ! Loop in case of error in OBJ, SAUV, CATEG and interactive run
   ! ----
   do

      ! Analysis of OBJ paragraph
      ! -------------------------
      nkar = ncar
      call local( cfin//pobje(:mpobje), 1+mpobje, karp, nkar, .false., 1, ioc, loca )
      if ( ioc > 0 ) then
         k1   = loca(1)
         kar1 = namab( car, pobje, npobje, mpobje, k1+1 )
            ! Search for the end of the paragraph
         call local( cfin, 1, kar1+1, nkar, .false., 1, ioc, loca )
         karn = loca(1)

            ! Print commands read if requested and interactive run
         if ( lprti ) then
            nladd = 3 + (karn-(k1+1))/lrecp
            call chkpg( nladd, .false., newp )
            write(imp,777) pobje(:mpobje), msginp
            write(imp,*) car(k1+1:karn)
            lines = lines + nladd
         else
!!          call chkpg( 2, .false., newp )
            write(lgu,777) pobje(:mpobje)
!!          lines = lines + 2
         end if

            ! Analysis with filtering
         car(k1:kar1) = blank(:kar1-k1) // sep2
         call gmpobj( k1, karn, lero, vuse, vnom, xmis, xmin, xmax, valpha )
      end if
         ! Filter vuse for label, ifrqcy and redundencies
      call filter( vuse, nvaru, label    )
      call filter( vuse, nvaru, ifrqcy )
         ! Build up unom (names of variables used in same order as vuse)
      unom(1:nvaru) = vnom(vuse(1:nvaru))

      ! Analysis of SAUV paragraph
      ! --------------------------
      nkar = ncar
      call local( cfin//psauv(:mpsauv), 1+mpsauv, karp, nkar, .false., 1, ioc, loca )
      if ( ioc > 0 ) then
         k1   = loca(1)
         kar1 = namab( car, psauv, npsauv, mpsauv, k1+1 )
            ! Search for the end of the paragraph
         call local( cfin, 1, kar1+1, nkar, .false., 1, ioc, loca )
         karn = loca(1)

            ! Print commands read if requested and interactive run
         if ( lprti ) then
            nladd = 3 + (karn-(k1+1))/lrecp
            call chkpg( nladd, .false., newp )
            write(imp,777) psauv(:mpsauv), msginp
            write(imp,*) car(k1+1:karn)
            lines = lines + nladd
         else
!!          call chkpg( 2, .false., newp )
            write(lgu,777) psauv(:mpsauv)
!!          lines = lines + 2
         end if

            ! Analysis with filtering
         car(k1:kar1) = blank(:kar1-k1) // sep2
         call gmpsav( k1, karn, lers, vsav, vnom )
      end if

      ! Analysis of CATEG paragraph
      ! ---------------------------
      nkar = ncar
      call local( cfin//pcate(:mpcate), 1+mpcate, karp, nkar, .false., 1, ioc, loca )
      if ( ioc > 0 ) then
         k1   = loca(1)
         kar1 = namab( car, pcate, npcate, mpcate, k1+1 )
            ! Search for the end of the paragraph
         call local( cfin, 1, kar1+1, nkar, .false., 1, ioc, loca )
         karn = loca(1)

            ! Print commands read if requested and interactive run
         if ( lprti ) then
            nladd = 3 + (karn-(k1+1))/lrecp
            call chkpg( nladd, .false., newp )
            write(imp,777) pcate(:mpcate), msginp
            write(imp,*) car(k1+1:karn)
            lines = lines + nladd
         else
!!          call chkpg( 2, .false., newp )
            write(lgu,777) pcate(:mpcate)
!!          lines = lines + 2
         end if

            ! Analysis with filtering
         car(k1:kar1) = blank(:kar1-k1) // sep2
         call gmpcat( k1, karn, lerc, nvals )

      end if

      ! Just find the TRAN paragraph
      ! ----------------------------
      nkar = ncar
      call local( cfin//ptran(:mptran), 1+mptran, karp, nkar, .false., 1, ioc, loca )
      if ( ioc > 0 ) then
         j   = loca(1)
         kt1 = namab( car, ptran, nptran, mptran, j+1 ) + 1
            ! Search for the end of the paragraph
         call local( cfin, 1, kt1, nkar, .false., 1, ioc, loca )
         ktn = loca(1) - 1
         car(j:kt1-1) = blank(:kt1-j-1) // sep2
         if ( kt1 >= ktn ) then
               ! Paragraph is empty: no transformations
            ltran = .false.
         else
            ltran = .true.
               ! See if the text function is used
            kb = index( car(kt1:ktn), nomop(itxop)(:lto) ) + kt1 - 1
            if ( kb < kt1 ) then
                  ! No text function whole paragraph converted to capitals
               call cvuct( car(kt1:ktn) )
            else
                  ! Text function: paragraph selectively converted
               ka = kt1
               outer1: do
                  call cvuct( car(ka:kb) )
                  inner1: do
                     ka = index( car(kb:ktn), achar(kpar2) ) + kb - 1
                     if ( ka >= kb ) then
                        if ( com(ka) ) then
                           kb = ka + 1
                           cycle inner1
                        end if
                        kb = index( car(ka:ktn), nomop(itxop)(:lto) ) + ka - 1
                        if ( kb >= ka ) cycle outer1
                        call cvuct( car(ka:ktn) )
                     end if
                     exit
                  end do inner1
                  exit
               end do outer1
            end if
         end if
      end if

      ! Filter unused or unrecognized paragraphs
      ! ----------------------------------------
      if ( fltrp .and. karp < ncar-nfin ) then
         call local( cfin, 1, karp, ncar-nfin, .false., 9, ioc, loca )
         if ( ioc > 0 ) then
            nladd = 5 + ioc
            call chkpg( nladd, .false., newp )
            write(imp,222)
            write(lgu,*) ermrk1//ermrk1
            write(lgu,222)
            loca(ioc+1) = ncar - nfin + 1
            do i = 1, ioc
               ka = loca(i)
               kb = min( ka+10, loca(i+1)-1 )
               if ( ltran .and. j > ka .and. j < kb ) kb = j - 1
               write(imp,111) car(ka:kb)
               write(lgu,111) car(ka:kb)
            end do
            write(lgu,*) ermrk2//ermrk2
            lines = lines + nladd
         end if
      end if

      ! Reenter paragraphs if error and interactive run
      ! -----------------------------------------------
      if ( lero .or. lers .or. lerc ) then
         if ( .not. linter .or. lec /= lec1 ) then
            write(lgu,*) ermrk1
            write(lgu,555)
            write(lgu,*) ermrk2
            kstop = -1
            return                                  !--- stop error ---!
         end if
         do
               ! Clear screen
            call chkpg( 0, .true., newp )
            write(imp,333)
            write(imp,444) pfin(:nfin)
            lines = lines + 2
               ! The set of corrected paragraphs is stored after the TRAN
               ! paragraph (if any), so that location of TRAN paragraph
               ! is unchanged by the addition of the new commands
            karp = ktn + 1
            ncar = karp
            car(karp:karp) = cfin
               ! N.B. cfin must be stored just before the new paragraph
               ! name because searching is done for the name preceded by
               ! cfin while user may begin directly with paragraph name
            call gmpcmd( card,.false., lprtb, pfin, nend, ikend, kard, nrrd, leof, lerr )
            if ( lerr ) then
               write(imp,555)
               kstop = -1
               return                               !--- stop error ---!
            end if
            if ( leof ) then
               write(imp,888)
               kstop = -1
               return                               !--- stop e-o-f ---!
            end if

            k1 = karp + 1
            if ( car(k1:k1) == cfin ) then
               karp = k1
               k1   = k1 + 1
            end if

               ! Add last end-of-sentence symbol in paragraphs if missing
            call gmpads( cfin, sep2, k1 )

               ! Selective conversion of commands (keywords) to capitals
               ! throughout all paragraphs (keyword values not converted)
            call cvucs( car(k1:ncar), com(k1:ncar), '=', sep2 )

               ! If no paragraph entered, prompt for one, otherwise exit
            if ( car(k1:k1+nfin-2) /= pfin(2:nfin) ) exit
         end do

            ! Give up current problem and turn to a new one ?
         if ( car(k1:k1+mpprob-1) == pprob(:mpprob) ) goto 20

         if ( car(k1:k1+mpstop-1) == pstop(:mpstop) ) then
            write(imp,3333)
            kstop = 1
            return                               !=== stop normal ===!
         end if

         cycle   ! Go back to analyze corrected paragraphs
      end if

      exit
   end do

                        !-------------!

      ! Now all paragraphs (except TRAN) have been read and analyzed
      ! Complete updates for mem1 and mema arrays

      ! If no codes/... nonetheless allocate vals and noms with 1 word
      ! to avoid the need, later, to test whether these arrays are allocated
   if ( .not. allocated(vals) .or. nvals == 0 ) then
      nvals = 0
      if ( allocated(vals) ) deallocate(vals)
      if ( allocated(noms) ) deallocate(noms)
      allocate( vals(1:1), noms(1:1), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpDir_3)."
         write(lgu,*) ermrk2
         kstop = -2
         return                                  !--- stop error ---!
      end if
      vals = zero
      noms = ' '
   end if

   l_vals = nvals
   l_ymin = nvaru
   l_ymax = nvaru
   l_noms = nvals

   ! Allocate and initialize ymin and ymax
   ! -------------------------------------
   if ( allocated(ymin) ) deallocate(ymin)
   if ( allocated(ymax) ) deallocate(ymax)
   allocate( ymin(nvaru), ymax(nvaru), stat=status )
   if ( status /= 0 ) then   ! fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (GmpDir_4)."
      write(lgu,*) ermrk2
      kstop = -2
      return                                     !--- stop error ---!
   end if
   ymin = xout
   ymax = -xout

   ! If no transformation this is a normal ending
   ! --------------------------------------------
   if ( .not. ltran ) return


   ! Analysis of transformations
   ! ---------------------------
   do
         ! Print commands read if requested and interactive run
      if ( lprti ) then
         nladd = 3 + (ktn-kt1)/lrecp
         call chkpg( nladd, .false., newp )
         write(imp,777) ptran(:mptran), msginp
         write(imp,*) car(kt1:ktn)
         lines = lines + nladd
      else
!!       call chkpg( 2, .false., newp )
         write(lgu,777) ptran(:mptran)
!!       lines = lines + 2
      end if

         ! See how many formulas are involved
      call transn( car(kt1:ktn), com(kt1:), ktn-kt1+1, nbrefo )
      nbrefo = nbrefo + 10      ! Just add a few, for security

         ! Allocate arrays used for the transformations
         ! (they will be deallocated in gmpprx after data reading and saving)
      allocate( xtra(-1:maxat,nbrefo),                      &
                adr(4,maxop,nbrefo),                        &
                adrv(2,maxva,nbrefo),                       &
                nato(nbrefo), nope(nbrefo), nvaf(nbrefo),   &
                next1(nbrefo), next2(nbrefo),               &
                pilif(nbrefo), pilel(nbrefo),               &
                lcond(nbrefo), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpDir_5)."
         write(lgu,5555)
         write(lgu,555)
         write(lgu,*) ermrk2
         kstop = -2
         return                                     !--- stop error ---!
      end if

      nvarts = nvmxsp   ! Copy to avoid double transmission with gmpint
      nvartt = nvmxst   ! Copy to avoid double transmission with gmpint
      call transa( car(kt1:ktn), com(kt1:ktn), ktn-kt1+1, sep2, to, ncto, by, ncby,   &
                   vnom(1:), nvarts, nvartt, nvartp, lerr )
      nvust = nvartp    ! Updates nvust
      if ( .not. lerr ) then
         ! OK, normal ending
         !------------------
            ! Update lines if temporary variables were used
         if ( nvust > nvmxsp ) lines = lines + 4 + (nvust-nvmxsp-1)/5
         return                                     !--- end normal ---!
      else                     ! Error in transa
                               ! ---------------
         if ( linter .and. lec == lec1 ) then
               ! Clear screen
            call chkpg( 0, .true., newp )
            write(imp,1111)
            write(imp,444) pfin(:nfin)
            lines = lines + 2
            ncar  = 0
               ! Read commands
            call gmpcmd( card,.false., lprtb, pfin, nend, ikend, kard, nrrd, leof, lerr )
            if ( lerr ) then
               write(imp,555)
               kstop = -1
               return                               !--- stop error ---!
            end if
            if ( leof ) then
               write(imp,888)
               kstop = -1
               return                               !--- stop e-o-f ---!
            end if
            k1 = 1
            if ( car(1:1) == cfin ) k1 = 2
               ! Add last end-of-sentence symbol in paragraphs if missing
            call gmpads( cfin, sep2, k1 )
               ! Selective conversion to capitals
            call cvucs( car(k1:ncar), com(k1:ncar), '=', sep2 )
            if ( car(k1:k1+mptran-1) /= ptran(:mptran) ) then
               if ( car(k1:k1+mpstop-1) == pstop(:mpstop) ) then
                  write(imp,3333)
                  kstop = 1
                  return                            !=== stop normal ===!
               end if
               kt1 = k1
            else
               kt1 = namab( car, ptran, nptran, mptran, k1 ) + 1
            end if
               ! Search for the end of the paragraph
            call local( cfin, 1, kt1, ncar, .false., 1, ioc, loca )
            ktn = loca(1) - 1
            if ( kt1 >= ktn ) then
                  ! Paragraph is empty: no transformation
               ltran = .false.
               return                               !--- end normal ---!
            else
                  ! See if the text function is used
               kb = index( car(kt1:ktn), nomop(itxop)(:lto) ) + kt1 - 1
               if ( kb < kt1 ) then
                     ! No text function whole paragraph converted to capitals
                  call  cvuct( car(kt1:ktn) )
               else
                     ! Text function: paragraph selectively converted
                  ka = kt1
                  outer2: do
                     call cvuct( car(ka:kb) )
                     inner2: do
                        ka = index( car(kb:ktn), achar(kpar2) ) + kb - 1
                        if ( ka >= kb ) then
                           if ( com(ka) ) then
                              kb = ka + 1
                              cycle inner2
                           end if
                           kb = index( car(ka:ktn), nomop(itxop)(:lto) ) + ka - 1
                           if ( kb >= ka ) cycle outer2
                           call cvuct( car(ka:ktn) )
                        end if
                        exit
                     end do inner2
                     exit
                  end do outer2
               end if
            end if
            cycle   ! Go back to analyze TRAN paragraph
         end if
      end if
   end do

   INCLUDE 'allocf.'
   INCLUDE 'gmpdirf1.'
   INCLUDE 'gmpdirf2.'

END SUBROUTINE gmpdir
