SUBROUTINE gmpgfw( nval )

!***********************************************************************
!     - gmpgfw -
!     writes dictionnary information on a new gmp file
!     (i.e. parameters and mem1 and mema arrays)
!***********************************************************************
!
!     NVAL  - INPUT . Max ( 1, length of vals and noms )      - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  NINT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memsta
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nval

   !  Local declarations
   !  ------------------
   INTEGER :: i, j, k, lmk, locn, locv
   INTEGER :: m, mabel, mfrqcy, n, nba, nb1, nwals

      ! The following work arrays are used only if savall is .false.
      ! (i.e. not all variables are saved)

   REAL(KIND=8), DIMENSION(nval) :: wals
   INTEGER, DIMENSION(nvarmx) :: lwal, lwom
   CHARACTER(LEN=nocw), DIMENSION(nval) :: woms

   !--------------------------------------------------------------------

      ! nb1 and nba are number of mem1 and mema arrays
   nb1 = 11
   nba = 3

   write(isave) codes
   write(isave) gmpvrs, xout, 1, 2, nocw, nb1, nba

   if ( savall ) then

      ! Save all variables
      ! ------------------
      write(isave) nvarmx, nvarsp, nvtemp, nvarid, label, ifrqcy
      write(isave) l_xmis, l_xmin, l_xmax, l_vals,   &
                   l_vuse, l_lval, l_lnom, l_vsav, l_valpha
      write(isave) l_vnom, l_unom, l_noms

      write(isave) vnom(1:nvarmx)
      write(isave) xmis, xmin, xmax
      write(isave) lval, lnom, valpha
         ! If given, write codes/cutpoints and their names
      if ( l_vals > 0 ) then
         write(isave) vals(1:l_vals)
         write(isave) noms(1:l_vals)
      endif

   else
      ! Save variables in the vsav list
      ! -------------------------------
         ! Reset parameters to values corresponding to variables saved
         ! and build up relevant vals and noms (here named wals and woms)
         ! locv and locn are starting indices in wals and woms for codes
         ! and names sub-arrays
      mabel  = 0
      mfrqcy = 0
      locv   = 3
      locn   = 3
      do i = 1, nvsav
         j = vsav(i)
         if ( j == label ) mabel = i
         if ( j == ifrqcy ) mfrqcy = i
            ! Fill in arrays lval (lwal) and vals (wals)
         k = lval(j)
         if ( k /= 0 ) then
            lwal(i)      = locv
            wals(locv-2) = i
            n = abs( nint( vals(k-1) ) )
            if ( n > kodtxt ) n = n - kodtxt
            lmk = locv - k
            do m = k-1, k-1+n
               wals(lmk+m) = vals(m)
            end do
            locv = locv + n + 2
         else
            lwal(i) = 0
         end if
            ! Fill in arrays lnom (lwom) and noms (woms)
         k = lnom(j)
         if ( k /= 0 ) then
            lwom(i) = locn
            write(woms(locn-2),1111) i
            read(noms(k-1),1111) n
            lmk = locn - k
            do m = k-1, k-1+n
               woms(lmk+m) = noms(m)
            end do
            locn = locn + n + 2
         else
            lwom(i) = 0
         end if
      end do
         ! nwals is number of items in wals and woms
         ! The value corresponding to nvarmx for save file is nvsav
         ! Now we can save the informations
      nwals = locv - 3
      write(isave) nvsav, nvarsp, nvtemp, nvsav, mabel, mfrqcy
      write(isave) nvsav, nvsav, nvsav, nwals, nvsav, nvsav, nvsav, nvsav
      write(isave) nvsav+nvarsp+nvtemp, nvsav, nwals

      write(isave) vnom(vsav(1:nvsav))
      write(isave) xmis(vsav(1:nvsav)), xmin(vsav(1:nvsav)), xmax(vsav(1:nvsav))
      write(isave) lwal(1:nvsav), lwom(1:nvsav), valpha(vsav(1:nvsav))
         ! If given write codes/cutpoints and their names
      if ( nwals > 0 ) then
         write(isave) wals(1:nwals)
         write(isave) woms(1:nwals)
      end if

   end if

   INCLUDE 'gmpnomsf.'

END SUBROUTINE gmpgfw
