SUBROUTINE gmpobj( k1, kn, lerr, vuse, vnom, xmis, xmin, xmax, valpha )

!***********************************************************************
!     - gmpobj -
!     analyzes OBJ paragraph, with commands filtering
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!     VUSE  - INPUT .    see definition of MEM1 arrays        - INTEGER
!             OUTPUT.
!     VNOM  - INPUT .    see definition of MEMA arrays        - CHAR
!             OUTPUT.
!     XMIS  - INPUT .    see definition of MEM1 arrays        - REAL*8
!             OUTPUT.
!     XMIN  - INPUT .    see definition of MEM1 arrays        - REAL*8
!             OUTPUT.
!     XMAX  - INPUT .    see definition of MEM1 arrays        - REAL*8
!             OUTPUT.
!     VALPHA- INPUT .    see definition of MEM1 arrays        - INTEGER
!             OUTPUT.
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines   nalpha  nvaru   label   ifrqcy
!     gmplgl : lcmpl
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        AVECTX  AVECTZ  CHKPG  CMPRSS  GMPWRN
!     External functions called:
!        LSCALX
!     Fortran  functions called:
!        MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE avectx( mot, longm, long, ideb, iend, kode, nomv, lnom, &
                         incr, minx, maxx, imin, imax, icol, jcol, lerr, &
                         dx, rx, ix, lx, cx )
         INTEGER, INTENT(IN)  :: longm, long, ideb, iend, kode, incr, minx, maxx
         LOGICAL, INTENT(IN)  :: lnom
         CHARACTER(LEN=*), INTENT(IN) :: mot
         CHARACTER(LEN=*), DIMENSION(minx:*), INTENT(IN) :: nomv
         INTEGER, INTENT(OUT) :: imin, imax, icol, jcol
         LOGICAL, INTENT(OUT) :: lerr
         REAL(KIND=8)    , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: dx
         REAL            , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: rx
         INTEGER         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: ix
         LOGICAL         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: lx
         CHARACTER(LEN=*), DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: cx
      END SUBROUTINE avectx
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1
   INTEGER, INTENT(INOUT) :: kn
   LOGICAL, INTENT(OUT) :: lerr
   REAL(KIND=8), DIMENSION(nvarmx), INTENT(INOUT) :: xmis, xmin, xmax
   INTEGER, DIMENSION(nvarmx), INTENT(INOUT) :: vuse, valpha
   CHARACTER(LEN=*), DIMENSION(0:nvarmx), INTENT(INOUT) :: vnom

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: lscalx
   INTEGER :: i, icol, idum, imax, imin, jcol, nkar, nladd
   LOGICAL :: newp
   INTEGER, DIMENSION(1) :: lab

   INCLUDE 'gmpobjp.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc

   !--------------------------------------------------------------------

   lerr = .false.

      ! Analysis of vnom
   call avectx( motc(1), lngm(1,1), lngm(1,2), k1, kn, 5,             &
                cnul, .false., 1, 1, nvarmx, imin, imax, icol, jcol,  &
                lerr, cx=vnom(1:nvarmx) )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      return
   end if

      ! Analysis of label
   call avectz( motc(2), lngm(2,1), lngm(2,2), k1, kn,   &
                vnom(0:), 0, nvarmx,                     &
                lab, 1, 1, idum, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
      return
   end if
   if ( icol > 0 ) label = lab(1)

      ! Analysis of vuse
   call avectz( motc(3), lngm(3,1), lngm(3,2), k1, kn,   &
                vnom(1:), 1, nvarmx,                     &
                vuse, 1, nvarmx, idum, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      return
   end if
   if ( icol > 0 ) nvaru = idum

      ! Analysis of missing and extreme values
   call avectx( motc(4), lngm(4,1), lngm(4,2), k1, kn, 1,              &
                vnom(1:),.true., 1, 1, nvarmx, imin, imax, icol, jcol, &
                lerr, dx=xmis )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
      return
   end if

   call avectx( motc(5), lngm(5,1), lngm(5,2), k1, kn, 1,              &
                vnom(1:),.true., 1, 1, nvarmx, imin, imax, icol, jcol, &
                lerr, dx=xmin )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
      return
   end if

   call avectx( motc(6), lngm(6,1), lngm(6,2), k1, kn, 1,              &
                vnom(1:),.true., 1, 1, nvarmx, imin, imax, icol, jcol, &
                lerr, dx=xmax )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(6:), nmotc-5 )
      return
   end if

      ! Analysis of ifrqcy
   call avectz( motc(7), lngm(7,1), lngm(7,2), k1, kn,   &
                vnom(0:), 0, nvarmx,                     &
                lab, 1, 1, idum, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(7:), nmotc-6 )
      return
   end if
   if ( icol > 0 ) ifrqcy = lab(1)

      ! Analysis of status of cases with incomplete data
   lcmpl = lscalx( motc(8), lngm(8,1), lngm(8,2), k1, kn, lcmpl, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(8:), nmotc-7 )
      return
   end if

      ! Analysis of valpha
   call avectz( motc(9), lngm(9,1), lngm(9,2), k1, kn,   &
                vnom(1:), 1, nvarmx,                     &
                valpha, 1, nvarmx, idum, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(9:), nmotc-8 )
      return
   end if
   if ( icol > 0 ) nalpha = idum

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   INCLUDE 'gmpf111.'

END SUBROUTINE gmpobj
