SUBROUTINE gmpprp( iout, vuse, unom, vnom, lchk )

!***********************************************************************
!     - gmpprp -
!     prints problem information
!***********************************************************************
!
!     IOUT  - INPUT . Unit number for printing                - INTEGER
!     VUSE  - INPUT .    see definition of MEM1 arrays        - INTEGER
!     UNOM  - INPUT .    see definition of MEMA arrays        - CHAR
!     VNOM  - INPUT .    see definition of MEMA arrays        - CHAR
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pgmgmp
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iout
   LOGICAL, INTENT(IN) :: lchk
   INTEGER, DIMENSION(nvaru), INTENT(IN) :: vuse
   CHARACTER(LEN=*), DIMENSION(nvaru) , INTENT(IN) :: unom
   CHARACTER(LEN=*), DIMENSION(nvarmx), INTENT(IN) :: vnom

   !  Local declarations
   !  ------------------

   INTEGER :: i, irw, jendf, jendt, l, lf, ll, lll, nladd, nlr, nvpl
   LOGICAL :: flnam, newp
   CHARACTER(LEN=8) :: labmis, labnom, labwgt
   INCLUDE 'gmpprpp.'

   !--------------------------------------------------------------------

   if ( nindv < maxint ) then
      write(cmxind,'(i10)') nindv
   else
      cmxind = allind
   end if
   if ( label > 0 ) then
      labnom = vnom(label)
   else
      labnom = ' '
   end if
   if ( ifrqcy > 0 ) then
      labwgt = vnom(ifrqcy)
   else
      labwgt = ' '
   end if
   if ( lcmpl ) then
      labmis = miselm
   else
      labmis = miskep
   end if
   if ( lrwde ) then
      irw = 1
   else
      irw = 2
   end if

      ! Determine number of lines to be printed
   jendt = max( len_trim( titre ), 1 )
   nladd = (jendt-1)/(lrecp-1) + 1
   nladd = nladd + 16
   flnam = filee(1:1) /= ' '
   if ( flnam ) then
      lf = max( len_trim( filee ), 1 )
      nladd = nladd + 3 + (lf-1)/(lrecp-1) + 1
   end if
   if ( lectx == 2 ) then
      jendf = max( len_trim( fmtin ), 1 )
      nladd = nladd - 1 + (jendf-1)/(lrecp-1) + 1
   end if
   if ( lchk ) call chkpg( nladd, .false., newp )

      ! Print problem title
   write(iout,111) nprob, (titre(i:min(i+lrecp-2,jendt)),i=1,jendt,lrecp-1)
      ! Print main parameters
   write(iout,222) nvarin, nvadd, nvarmx, cmxind, labnom, labwgt, labmis, inpux, rep(irw)
   if ( flnam ) then
      write(iout,333) (filee(i:min(i+lrecp-2,lf)),i=1,lf,lrecp-1)
   end if
      ! Print input format
   select case (lectx)
      case (1)
         write(iout,444) freefm
      case (2)
         write(iout,444) (fmtin(i:min(i+lrecp-2,jendf)),i=1,jendf,lrecp-1)
      case (3)
         lf = max( len_trim( codee ), 1 )
         write(iout,555) hdrgmp, codee(1:min(lrecp-1,lf))
      case (4)
         write(iout,444) unform//prdbl
      case (5)
         write(iout,444) unform//prsgl
   end select
   lines = lines + nladd

      ! Print variables used

   if ( lbref ) then
         ! Brief output (15 variable-numbers per line, without names)
         ! (it is supposed that the printed list never requires more
         ! than one page: no subtle count of lines printed)
      nladd = 4 + (nvaru-1)/15
      if ( lchk ) call chkpg( nladd, .false., newp )
      write(iout,666)
      write(iout,'(1x,15i4)') vuse
      lines = lines + nladd

   else
         ! Extended output (variable numbers and names)
         ! nvpl = number of variables printed per line (see format 777)
         ! nlr  = number of lines still to be printed
         ! nladd= number of lines to be printed in current page
      if ( lchk ) call chkpg( 4, .false., newp )
      write(iout,666)
      lines = lines + 3
      nvpl  = 4
      nlr   = (nvaru-1)/nvpl + 1
      lll   = 0
      do
         if ( lchk ) then
            nladd = min( nlr, max( linep-lines, 1 ) )
         else
            nladd = nlr
         end if
         do l = 1, nladd
            ll  = lll + 1
            lll = min( nvaru, lll+nvpl )
            write(iout,777) (vuse(i),unom(i),i=ll,lll)
         end do
         lines = lines + nladd
         nlr   = nlr   - nladd
         if ( nlr > 0 ) then
            if ( lchk ) call chkpg( 0, .true., newp )
            cycle
         end if
         exit
      end do

   end if

   INCLUDE 'gmpprpf.'

END SUBROUTINE gmpprp
