SUBROUTINE gmpsav( k1, kn, lerr, vsav, vnom )

!***********************************************************************
!     - gmpsav -
!     analyzes SAUVE paragraph, with commands filtering
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!     VSAV  - INPUT .    see definition of MEM1 arrays        - INTEGER
!             OUTPUT.
!     VNOM  - INPUT .    see definition of MEMA arrays        - CHAR
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpchr : contsv  files   codes   fmtsv
!     gmpint : lines   isave   kontsv  nvsav
!     gmplgl : lsave   lcods   lrwds   savall
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        AVECTZ  CHKPG  CHKUNT  CMPRSS  CSCALX  CVUCT  FILOPN  GMPWRN
!     External functions called:
!        ISCALX  LSCALX
!     Fortran  functions called:
!        ADJUSTL  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pgmgmp
   USE systm
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE chkunt( lerr,iout,iunt,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12 )
         IMPLICIT NONE
         INTEGER, INTENT(IN) :: iout, iunt
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11, u12
         LOGICAL, INTENT(OUT) :: lerr
      END SUBROUTINE chkunt
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1
   CHARACTER(LEN=*), DIMENSION(0:nvarmx), INTENT(IN) :: vnom
   INTEGER, INTENT(INOUT) :: kn
   INTEGER, DIMENSION(nvarmx), INTENT(INOUT) :: vsav
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: iscalx
   LOGICAL, EXTERNAL :: lscalx

   INTEGER :: i, icol, idum, jcol, nkar, nladd, numi
   LOGICAL :: lexis, lopen, newp
   CHARACTER(LEN=11) :: frm

   INCLUDE 'gmpsavp.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc

   !--------------------------------------------------------------------

   lerr = .false.

   isave = iscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, isave, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      return
   end if

   call cscalx( motc(2), lngm(2,1), lngm(2,2), k1, kn, files, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
      return
   end if
   files = adjustl( files )
      ! Convert file name to capitals if not unix system
   if ( .not. lunix .and. icol > 0 ) call cvuct( files )

   call cscalx( motc(3), lngm(3,1), lngm(3,2), k1, kn, codes, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      return
   end if

   call cscalx( motc(4), lngm(4,1), lngm(4,2), k1, kn, fmtsv, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
      return
   end if
      ! Convert format to capitals
   if ( icol > 0 ) call cvuct( fmtsv )

   call cscalx( motc(5), lngm(5,1), lngm(5,2), k1, kn, contsv, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
      return
   end if
      ! Reset kontsv code to proper value according to contsv
   if ( icol > 0 ) then
      call cvuct( contsv )
      if ( contsv(:3) /= zdata ) kontsv = 0
   end if

      ! Analysis of vsav
   call avectz( motc(6), lngm(6,1), lngm(6,2), k1, kn,  &
                vnom(0:), 0, nvarid,                    &
                vsav, 1, nvarid, idum, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(6:), nmotc-5 )
      return
   end if
   if ( icol > 0 ) then
      savall = .false.
      nvsav = idum
      if ( vsav(1) == 0 ) nvsav = 0
   end if

   lrwds = lscalx( motc(7), lngm(7,1), lngm(7,2), k1, kn, lrwds, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(7:), nmotc-6 )
      return
   end if

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   ! Prepare save file
   ! -----------------
   lcods = fmtsv(:3) == hdrgmp
   if ( lcods .or. fmtsv(1:3) == 'BIN' .or. fmtsv(2:4) == 'BIN' ) then
      frm = 'UNFORMATTED'
   else
      if ( fmtsv(1:1) /= '(' .and. fmtsv(1:3) /= zfree ) then
!!       call chkpg( 4, .false., newp )
         write(lgu,*) mrker1
         write(lgu,222) fmtsv(:40)
         write(lgu,*) mrker2
!!       lines = lines + 4
         lerr = .true.
         return
      end if
      frm = 'FORMATTED  '
   end if
   if ( isave >= 0 ) then
         ! Check unit number is correct
      call chkunt( lerr, imp, isave, iu1, iu2, iu3, iu4, imp, lec, lec1, lec2, inpux )
      if ( lerr ) return
      lsave = .true.
      if ( files(1:1) /= ' ' ) then
            ! Check the file already exists and is connected
         inquire (file=files,exist=lexis,opened=lopen,number=numi)
         if ( lopen .and. lexis .and. numi /= isave ) close (numi)
      end if
      call filopn( 31, isave, files, 'UNKNOWN', frm, .false., lerr )
      if ( lerr ) return
   else
      if ( files(1:1) /= ' ' ) then
            ! Check the file already exists and is connected
         inquire (file=files,exist=lexis,opened=lopen,number=numi)
            ! Use default save unit
         isave = ksave
         if ( lopen .and. lexis .and. numi /= isave ) close (numi)
         call filopn( 31, isave, files, 'UNKNOWN', frm, .false., lerr )
         if ( lerr ) return
      end if
      lsave = isave >= 0
      lcods = lcods .and. lsave
   end if

   INCLUDE 'gmpf111.'
   INCLUDE 'gmpsavf.'

END SUBROUTINE gmpsav
