SUBROUTINE gmpxin( nerr, nmis, nsav, kodsv )

!***********************************************************************
!     - gmpxin -
!     reads input data with possible saving in gmp file or user file.
!     data used for current problem are stored on iu1
!***********************************************************************
!
!     NERR  - OUTPUT. Number of cases with wrong categ. codes - INTEGER
!     NMIS  - OUTPUT. Number of cases with missing data among - INTEGER
!                     cases used
!     NSAV  - OUTPUT. Number of cases written in save file    - INTEGER
!     KODSV - INPUT . Code for data saving                    - INTEGER
!                     0 = no save
!                     1 = save in user-file, free format
!                     2 = save in user-file, Fortran format
!                     3 = save in GMP-file,  real*8 binary
!                     4 = save in user-file, real*8 binary
!                     5 = save in user-file, real binary
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines   nindv   nindvu
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  FMT_TYPE  GMP999  TRANSE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  ANY  MIN  NINT  REAL
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memsta
   USE spcval
   USE trancb

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: kodsv
   INTEGER, INTENT(OUT) :: nerr, nmis, nsav

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   INTEGER :: i, ifrq, ilab, iuse, j, k, kase, kf, kvn, kv1, l, ll
   INTEGER :: mis, nc, ni, nk, nlps, nter, num, svkod, xlect
   LOGICAL :: lsve, ltxt, newp, nolab
   CHARACTER(LEN=8), PARAMETER :: frqhdr='Freq.   '
   CHARACTER(LEN=8) :: tmplab

      ! Work arrays:
      ! x     is a work vector for data + special variables
      !                                 + temporary variables
      !                                 + labeling and frequency variables
      ! rin   is a work vector for input data (kind real)
      ! xchar is a work vector for list-directed character input/output data
      ! y     is a work vector for the (recoded) data used
      ! vtxt  is a work vector flagging character variables
      ! type  is a work vector for format type of variables

   REAL(KIND=8), DIMENSION(nvmxsp+nvtemp+2) :: x
   REAL(KIND=8), DIMENSION(nvaru) :: y
   REAL, DIMENSION(nvarin) :: rin
   INTEGER, DIMENSION(nvarmx) :: type
   LOGICAL, DIMENSION(nvarmx) :: vtxt
   CHARACTER(LEN=32), DIMENSION(nvarmx) :: xchar

   INCLUDE 'gmpxinp.'

   !--------------------------------------------------------------------

      ! iuse is the index of 1st special variable (vstat) in x
   iuse = nvarmx + 1
      ! Special internal missing value is attributed to last special variable
      ! (which must not be modified by transformations)
   x(nvmxsp) = xout
      ! Prepare labeling and frequency variables
   if ( label > 0 ) then
      ilab  = label
      nolab = .false.
   else
      ilab  = nvmxsp + nvtemp + 1
      nolab = .true.
   end if
!!!x(ilab) = xblan
   if ( ifrqcy > 0 ) then
      ifrq = ifrqcy
   else
      ifrq = nvmxsp + nvtemp + 2
   end if
   x(ifrq) = one

   lsve = kodsv > 0   ! See whether saving is requested or not

   svkod = kodsv      ! Save code
   xlect = lectx      ! Reading code

      ! Prepare array flagging character variables. Used if:
      ! . list-directed input includes character variables, and/or
      ! . list-directed saving includes character variables, and/or
      ! . data printout includes character variables
   vtxt = .false.   ! Initialize to no char. variables
      ! Look at "text" codes
   do i = 1, nvarmx
      k = lval(i)
      if ( k > 0 ) then
         nk = nint( vals(k-1) )
         if ( nk < -kodtxt ) vtxt(i) = .true.
      end if
   end do
      ! Complete with valpha list, if given
   do i = 1, nalpha
      vtxt(valpha(i)) = .true.
   end do
      ! Complete with format type, if Fortran format was given
   if ( xlect == 2 ) then
      call fmt_type( fmtin, nvarin, type )
      do i = 1, nvarin
         if ( type(i) == 1 ) vtxt(i) = .true.
      end do
   end if
   if ( label > 0 ) vtxt(label) = .true.
   if ( lectx == 1 .and. any(vtxt(1:nvarin)) ) xlect = 0   ! Update reading code
   if ( kodsv == 1 .and. any(vtxt(1:nvarmx)) ) svkod = 0   ! Update save code

      ! If data printout requested, update output formats and print
      ! names of variables printed (the ones used). Here ni is no. of
      ! variables printed per line, and nlps is no. of lines printed
      ! for each subject.
      ! The variable format used to print data is stored in car
      ! and kf is its relevant length.
   kf = 27
   if ( lpdat ) then
      ni = (lrecp-19) / 14
      write(fmtx(25:26),'(i2)') ni
      car( 1:27) = '(1x,i7,1x,a8,(t20, 4g14.7))'
      car(19:20) = fmtx(25:26)
      nlps = nvaru/ni + 1
         ! See if there are variables needing A-type format.
         ! If not, or if not enough room in car for the format,
         ! standard format is used (g-type for all variables, so that
         ! char. variables, if any, will have an horrible appearance).

         ! See if variables other than case label are char. type
      if ( label > 0 ) vtxt(label) = .false.
      ltxt = any(vtxt)
      if ( label > 0 ) vtxt(label) = .true.
      if ( ltxt ) then
            ! char. variables are present: whether explicitely declared
            ! or by means of A-specifications in a format or by "text"
            ! in the CATEG paragraph.
            ! Remember that the frequency variable will be printed first,
            ! before variables used; so 1+nvaru var. are to be printed.
         if ( incar >= 22+4*(1+nvaru/ni)+6*nvaru ) then
            car(14:22) = '2x,g14.7,'
            ll  = ni - 1
            kf  = 23
            kvn = 0
            do
               kv1 = kvn + 1
               kvn = min( kvn+ll, nvaru )
               do i = kv1, kvn
                  j = vuse(i)
                  if ( vtxt(j) ) then
                     car(kf:kf+5) = 'a9,5x,'
                  else
                     car(kf:kf+5) = 'g14.7,'
                  end if
                  kf = kf + 6
               end do
               if ( kvn < nvaru ) then
                  car(kf-1:kf+3) = '/19x,'
                  kf = kf + 4
                  ll = ni
                  cycle
               end if
               exit
            end do
            kf = kf - 1
            car(kf:kf) = ')'
         end if
      end if
      call chkpg( 15, .false., newp )
      write(imp,111)
      write(imp,fmtx) frqhdr, unom(1:nvaru)
      write(imp,*) '   '
      lines = lines + 5
   end if

      ! Note that inpux was rewinded, if relevant, in gmppro
      ! and  that isave was rewinded, if relevant, in gmpsav
   rewind iu1
   ni     = 0
   nindvu = 0
   nerr   = 0
   nmis   = 0
   nsav   = 0
   nter   = 0   ! nter is a counter for errors in transformations
                ! coded through a tran paragraph

   ! CASE READING LOOP
   ! -----------------

   case: do kase = 1, nindv

      select case (xlect)

         case (0)      ! List-directed input with char. variables
               ! Set variables to missing value (so that if some variables
               ! for the current case are skipped, they will be missing
               ! instead of being identical to those for the previous case)
            do i = 1, nvarin
               write(xchar(i),*) xmis(i)
            end do
            read(inpux,*,end=10) xchar(1:nvarin)
            do i = 1, nvarin
               if ( vtxt(i) ) then
                  read(xchar(i),'(a)') x(i)
               else
                  read(xchar(i),*) x(i)
               end if
            end do
         case (1)      ! List-directed input without char. variables
               ! Set variables to missing value (so that if some variables
               ! for the current case are skipped, they will be missing
               ! instead of being identical to those for the previous case)
            x(1:nvarin) = xmis(1:nvarin)
            read(inpux,*,end=10) x(1:nvarin)

         case (2)      ! Input data read with Fortran format
            read(inpux,fmtin,end=10) x(1:nvarin)

         case (3)      ! Input data through a gmp file (binary, real*8)
            read(inpux,end=10) num, x(1:nvarin)
            if ( num <= 0 ) goto 10

         case (4)      ! Input data as binary, real*8
            read(inpux,end=10) x(1:nvarin)

         case (5)      ! Input data as binary, real
            read(inpux,end=10) rin
               ! convert data from real to real*8
            x(1:nvarin) = rin

         case default  ! Unexpected lectx value
            call gmp999
            stop 888

      end select

      ni = kase
         ! Initialize special variables vstat and vrang
      x(iuse  ) = one
      x(iuse+1) = kase
         ! Create label with case number if no one provided
      if ( nolab ) then
         write(tmplab,'(i8.8)') kase
         read(tmplab,'(a8)') x(ilab)
      end if

      ! FORTRAN TRANSFORMATIONS BY SUBROUTINE TRANSF
      ! --------------------------------------------
!!!   call transf( nprob, nvarmx, ni, x, x(iuse), xout )

      ! COMMAND LANGUAGE TRANSFORMATIONS IF ANY
      ! ---------------------------------------
      if ( ltran ) call transe( x, nvust, kase, nter )

      ! SAVE (TRANSFORMED) INPUT DATA ON ISAVE IF REQUESTED
      ! ---------------------------------------------------
      if ( lsve .and. x(iuse) >= zero ) then
         select case (svkod)
            case (0)      ! Save in user-file, free format (char. var.)
               do i = 1, nvsav
                  j = vsav(i)
                  if ( vtxt(j) ) then
                     write(xchar(i),'(a)') x(j)
                  else
                     write(xchar(i),*) x(j)
                  end if
               end do
               write(isave,*) xchar(1:nvsav)
            case (1)      ! Save in user-file, free format (no char. var.)
               write(isave,*) x(vsav(1:nvsav))
            case (2)      ! Save in user-file, Fortran format
               write(isave,fmtsv) x(vsav(1:nvsav))
            case (3)      ! Save in gmp-file
               write(isave) kase, x(vsav(1:nvsav))
            case (4)      ! Save in user-file, real*8 binary
               write(isave) x(vsav(1:nvsav))
            case (5)      ! Save in user-file, real binary
               write(isave) real(x(vsav(1:nvsav)))
            case default  ! Unexpected kodsv value
               call gmp999
               stop 889
         end select
         nsav = nsav + 1
      end if

      ! THE CASE IS DROPPED IF REQUIRED
      ! -------------------------------
      if ( x(iuse) <= zero ) cycle case
         mis = 0

         ! PRINT DATA USED IF REQUESTED
         ! ----------------------------
         if ( lpdat ) then
            call chkpg( nlps, .false., newp )
            write(imp,car(:kf)) kase, x(ilab), x(ifrq), x(vuse(1:nvaru))
            lines = lines + nlps
         end if

         ! STORE VARIABLES USED (RECODED IF NEEDED) ON IU1
         ! -----------------------------------------------
         do i = 1, nvaru
            j    = vuse(i)
            y(i) = x(j)

            k = lval(j)
            if ( k > 0 ) then

                  ! Categorical data
               nk = nint( vals(k-1) )
               nc = abs( nk )
               if ( nc <= kodtxt ) then
                     ! Check for limits and missing value
                  if ( x(j) == xmis(j) .or. x(j) < xmin(j) .or. x(j) > xmax(j) .or. &
                           abs(x(j)) >= xout ) then
                     y(i) = xout
                        ! Case is dropped if complete data is needed
                     if ( lcmpl ) cycle case
                     mis = 1
                     cycle
                  end if
               else
                  nc = nc - kodtxt
               end if
                  ! Recode
               if ( nk < 0 ) then
                     ! Numerical or character codes
                  ltxt = .true.
                  ll = 1
                  do l = k, k+nc-1
                     if ( x(j) == vals(l) ) then
                        ltxt = .false.
                        exit
                     end if
                     ll = ll + 1
                  end do
                  if ( ltxt ) then
                        ! Error: none of stated codes; drop case
                     nerr = nerr + 1
                     cycle case
                  end if
                  y(i) = ll
               else
                     ! Cutpoints
                  ll = 1
                  do l = k, k+nc-2
                     if ( x(j) <= vals(l) ) exit
                     ll = ll + 1
                  end do
                  y(i) = ll
               end if

            else
                  ! Continuous data
                  ! Check for limits and missing value
               if ( x(j) == xmis(j) .or. x(j) < xmin(j) .or. x(j) > xmax(j) .or.   &
                     abs(x(j)) >= xout ) then
                  y(i) = xout
                     ! Case is dropped if complete data is needed
                  if ( lcmpl ) cycle case
                  mis = 1
                  cycle
               end if
            end if
               ! Compute extrema of (recoded) variable
            if ( y(i) < ymin(i) ) ymin(i) = y(i)
            if ( y(i) > ymax(i) ) ymax(i) = y(i)

         end do

         nindvu = nindvu + 1
         nmis   = nmis + mis
         write(iu1) kase, x(ilab), x(ifrq), y

   end do case

   ! END OF CASE READING LOOP
   ! ------------------------

10 continue   ! End-of-file

   nindv = ni

   if ( kodsv == 3 ) then
         ! Write ending record in gmp file, to be used as end-of-file
      kase = -1
      write(isave) kase, (xout,i=1,nvsav)
   end if

   if ( nter > 0 ) then
      write(imp,222) nter
      call chkpg( 0, .true., newp )
      write(lgu,*) ermrk1
      write(lgu,222) nter
      write(lgu,*) ermrk2
   end if

   INCLUDE 'gmpxinf.'

END SUBROUTINE gmpxin
