SUBROUTINE transe( xvar, nvatp, kase, nter )

!***********************************************************************
!     - transe -
!     executes formulas
!***********************************************************************
!
!     XVAR  - INPUT . Values of variables                     - REAL*8
!             OUTPUT. Updated values
!     NVATP - INPUT . Length of XVAR (number of permanent     - INTEGER
!                     + temporary variables)
!     KASE  - INPUT . Sequential number of current case       - INTEGER
!     NTER  - INPUT . Number of detected errors               - INTEGER
!             OUTPUT. Updated value
!
!     --------------------------------------------------------
!     Module elements modified:
!     opprm : natom   noper   nvarf
!
!----------------------------------------------------------------------
!
!     Subroutines called:
!        FORMAJ
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE opprm
   USE spcval
   USE trancb

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nvatp, kase
   INTEGER, INTENT(INOUT) :: nter
   REAL(KIND=8), DIMENSION(nvatp), INTENT(INOUT) :: xvar

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: foper   ! The function executing operations
   REAL(KIND=8) :: resul
   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, j, kond, numv
   INTEGER, PARAMETER :: mxmsg=10    ! Maximum number of error messages printed

   !--------------------------------------------------------------------

   kond = 0
   i = 1

   do         ! Loop on formulas

      natom = nato(i)
      noper = nope(i)
      nvarf = nvaf(i)
      call formaj( foper, nvatp, xvar, xtra(-1:,i), adr(1:,1:,i), adrv(1:,1:,i), resul, kase, i, nter )
      if ( lcond(i) ) then
            ! It's a condition
         kond = kond + 1
            ! Check its output is defined (see foper)
         if ( resul < xmaxim ) then
            if ( resul > zero ) then
               i = next1(i)
            else
               i = next2(i)
            end if
         else
               ! The condition is undefined, stop execution
            nter = nter + 1
            write(io,*) ewmrk1
            if ( nter <= mxmsg ) write(io,111) kase, kond
            write(io,333)
            write(io,*) ewmrk2
            if ( nter <= mxmsg ) write(iook,111) kase, kond
            write(iook,333)
            return
         end if
      else
            ! It's an algebraic formula
         numv = next2(i)
         if ( numv > 0 ) then
               ! Formula is on right hand size of equal sign
            xvar(numv) = resul
         else
               ! Formula is on left hand size of equal sign
               ! (i.e. it gives the index of the variable storing
               ! the result of next formula, that on right hand side)
               ! note that an epsilon is added to avoid such things
               ! as int(0.999999999....) = 0
            j = resul + 1.0e-12_8
            if ( j <= 0 .or. j > nvatp ) then
                  ! Index out of range, stop execution
               nter = nter + 1
               write(io,*) ewmrk1
               if ( nter <= mxmsg ) write(io,222) kase, i, j
               write(io,333)
               write(io,*) ewmrk2
               if ( nter <= mxmsg ) write(iook,222) kase, i, j
               write(iook,333)
               return
            end if
            next2(next1(i)) = j
         end if
         i = next1(i)
      end if

      if ( i > nform ) exit
   end do

   INCLUDE 'transef.'

END SUBROUTINE transe
