SUBROUTINE varid2( nbreva, nomvar, lonvar, adrv, numv, nom, lc, ic, lerr )

!***********************************************************************
!     - varid2 -
!     searches for a name of variable, the length of target name being
!     unknown (search is done by descending order of variable-name
!     lengths to avoid stopping on a name which would be the prefix of
!     other names), and updates array of addresses of variables
!     N.B. Search is insensitive to case type of letters
!***********************************************************************
!
!     NBREVA- INPUT . Number of variables                     - INTEGER
!     NOMVAR- INPUT . Names of variables                      - CHAR
!     LONVAR- INPUT . Their actual length                     - INTEGER
!     ADRV  - INPUT . Array of addresses of variables         - INTEGER
!             OUTPUT. Updated
!     NUMV  - INPUT . Array of indices of variables sorted    - INTEGER
!                     in descending order of name length
!     NOM   - INPUT . Name being tested                       - CHAR
!     LC    - INPUT . Maximum location (end of formula)       - INTEGER
!     IC    - INPUT . Location of first character of NOM      - INTEGER
!             OUTPUT. Location of character following last
!                     character of NOM
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     opprm : natom   nvarf
!
!----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        SAMSTR
!     Fortran  functions called:
!        MIN
!
!-----------------------------------------------------------------------

   USE opprm

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nbreva, lc
   CHARACTER(LEN=*), INTENT(IN) :: nom
   INTEGER, DIMENSION(nbreva), INTENT(IN) :: lonvar, numv
   CHARACTER(LEN=*), DIMENSION(nbreva), INTENT(IN) :: nomvar
   INTEGER, INTENT(INOUT) :: ic
   INTEGER, DIMENSION(2,maxva), INTENT(INOUT) :: adrv
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: samstr
   INTEGER :: i, j, ll
   INCLUDE 'varid12p.'

   !--------------------------------------------------------------------

   lerr = .false.

   ! N.B. Names are sorted in decreasing order of length
   ll = lc - ic + 1
   do j = 1, nbreva
      i = numv(j)
      if ( ll >= lonvar(i) ) then
         if ( samstr( nom(1:lonvar(i)), nomvar(i)(1:lonvar(i)) ) ) then
            natom = natom + 1
            nvarf = nvarf + 1
            if ( natom > maxat ) then
               write(io,*) ewmrk1
               write(io,222) msgato
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            if ( nvarf > maxva ) then
               write(io,*) ewmrk1
               write(io,222) msgvar
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            adrv(1,nvarf) = i
            adrv(2,nvarf) = natom
            ic = ic + lonvar(i)
            return
         end if
      end if
   end do

   write(io,*) ewmrk1
   write(io,111) nom(1:min(30,ll))
   write(io,*) ewmrk2
   lerr = .true.

   INCLUDE 'varid12f.'

END SUBROUTINE varid2
