SUBROUTINE getwgh( xpar, ax, ndim, nxp, k1, arith, wght )

!***********************************************************************
!     - getwgh -
!     GETs marginal part WeiGHts (metric terms to be used further for
!     orthonormalization), according to the relevant derivation.
!***********************************************************************
!
!     XPAR  - INPUT . X partition-vector, of length NDIM      - INTEGER
!     AX    - INPUT . Vector of weights, of length NDIM       - REAL*8
!     NDIM  - INPUT . Dimension of the set involved (V or W)  - INTEGER
!     NXP   - INPUT . Number of parts in XPAR                 - INTEGER
!     K1    - INPUT . Index of first non-zero element in XPAR - INTEGER
!     ARITH - INPUT . .true. if arithmetic mean is used to    - LOGICAL
!                     derived metric terms for
!                     orthogonalization;
!                     .false. if harmonic mean is used
!     WGHT  - OUTPUT. Vector of part weights, of length NXP   - REAL*8
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ndim, nxp, k1
   LOGICAL, INTENT(IN) :: arith
   INTEGER, INTENT(IN) :: xpar(ndim)
   REAL(KIND=8), INTENT(IN) :: ax(ndim)
   REAL(KIND=8), INTENT(OUT) :: wght(nxp)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   INTEGER :: j, k
   REAL(KIND=8) :: bwrk(ndim)

   !--------------------------------------------------------------------

   wght(1:nxp) = zero
   bwrk(1:nxp) = zero
   if ( arith ) then   ! N * arithmetic mean
      do k = k1, ndim
         j = xpar(k)
         if ( j > 0 ) wght(j) = wght(j) + ax(k)
      end do
   else                ! N * harmonic mean
      do k = k1, ndim
         j = xpar(k)
         if ( j > 0 ) then
            wght(j) = wght(j) + one/ax(k)
            bwrk(j) = bwrk(j) + one
         end if
      end do
      wght(1:nxp) = bwrk(1:nxp)*bwrk(1:nxp) / wght(1:nxp)
   end if

END SUBROUTINE getwgh
