INTEGER FUNCTION numgx( nvg, vg, gmn, gmx )

!***********************************************************************
!     - numgx -
!     computes group index, given the grouping variables of the case.
!     This index is the number of the cell (in which the case falls)
!     in the complete table (i.e. the table obtained by crossing all
!     grouping-variables), assuming the first grouping variable varies
!     the fastest, etc
!***********************************************************************
!
!     NVG   - INPUT . Number of grouping variables            - INTEGER
!     VG    - INPUT . Vector of grouping variables            - REAL*8
!     GMN   - INPUT . Vector of minimum code for the grouping - REAL*8
!                     variables
!     GMX   - INPUT . Vector of maximum code for the grouping - REAL*8
!                     variables
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NINT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nvg
   REAL(KIND=8), INTENT(IN) :: vg(nvg), gmn(nvg), gmx(nvg)

   !  Local declarations
   !  ------------------

   INTEGER :: i, ip

   !--------------------------------------------------------------------

   ip = 1
   numgx = nint(vg(1)) - nint(gmn(1)) + 1

   do i = 2, nvg
      ip = ip * ( nint(gmx(i-1)) - nint(gmn(i-1)) + 1 )
      numgx = numgx  +  ip * ( nint(vg(i)) - nint(gmn(i)) )
   end do

END FUNCTION numgx
