SUBROUTINE pac001

!***********************************************************************
!     - pac001 -
!     is a directing subroutine called by the main program and may be
!     viewed as a pseudo main program:
!     the actual main program only initialized some module elements
!***********************************************************************

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memst2
   USE memsta
   USE memstb
   USE pacint

   IMPLICIT NONE

   !  Local declarations
   !  ------------------

   INTEGER, PARAMETER :: cardl=256
   INTEGER :: ierr, kstop, nladd
   LOGICAL :: error, lcmd1, lcmdl, lcmdp, fltrp, newp
   CHARACTER(LEN=cardl) :: card
      ! card is a record on unit lec, in which commands are stored
   INCLUDE 'pac001p.'

   EXTERNAL pachdr

   !--------------------------------------------------------------------

      ! Stabilize TCPU
   call tcpu( lgu, .false., ' ' )
   call tcpu( lgu, .false., ' ' )

      ! UPDATE PAGE HEADER WITH DATE OF RUN
      ! -----------------------------------
   call datrun( pgmhdr(8:15) )

      ! FIRST RECORD OF LOG FILE IS DATE OF RUN
      ! ---------------------------------------
   if ( lgu /= imp ) then
      write(lgu,*) pgmhdr(:15)
      write(lgu,*) '  '
   end if

      ! GENERAL INITIALIZATIONS PERFORMED ONLY ONCE
      ! -------------------------------------------
   call gmpin1( pachdr, cardl, error )
   if ( error ) return

      ! SPECIFIC INITIALIZATIONS PERFORMED ONLY ONCE
      ! --------------------------------------------
   call pacin1( error )
   if ( error ) return

      ! SPECIFY READ AND PRINT PARAMETERS FOR COMMANDS READING
      ! ------------------------------------------------------
      ! lcmdl is true if a complete set of commands (i.e. including
      ! the ending paragraph) has been read on lec (but not analyzed)
      ! before calling gmpdir.
      ! lcmd1 is true if the first record of a set of commands has
      ! been read on lec (but not analyzed) before calling gmpdir.
      ! For the 1st problem lcmdl and lcmd1 are set to .false.
      ! (note that gmpdir lets them unchanged but reads all the
      ! commands so that they have to be reset to .false. on return),
      ! for succeding ones it depends on how the program-specific set
      ! manages commands reading.
      ! lcmdp is .true. if command-records read are to be printed.

   lcmdl = .false.
   lcmd1 = .false.
   lcmdp = .true.

      ! Set fltrp to true if commands are to be checked for unused or
      ! unrecognized paragraphs, to false otherwise.
   fltrp = .true.

      ! PROBLEM LOOP
      ! ------------

   do
      call tcpu( lgu, .false., ' ' )

         ! GENERAL INITIALIZATIONS FOR A NEW PROBLEM
         ! -----------------------------------------
      call gmpipb

      call chkpg( 0, .true., newp )   ! Get a new page
      if ( linter .and. .not. lcmdl ) then
         call chkpg( 3, .false., newp )
         write(imp,111) pfin(:nfin)   ! Prompt user to enter paragraphs
         lines = lines + 1
      end if

         ! READ AND ANALYZE GENERAL PARAGRAPHS
         ! -----------------------------------
      call gmpdir( card, lcmdl, lcmd1, lcmdp, fltrp, kstop )
      if ( kstop /= 0 ) exit
      lcmdl = .false.
      lcmd1 = .false.

         ! PRINT PROBLEM INFORMATION AND READ (AND SAVE) DATA
         ! --------------------------------------------------
      call gmpprx( ierr )

         ! If an error occurred, stop if batch, next problem otherwise
      if ( ierr /= 0 ) then
         if ( linter ) cycle
         exit
      end if

            ! PROGRAM-SPECIFIC SECTION
            ! ========================

      if ( nindvu <= 0 ) then   ! It's not worth continuing
         call chkpg( 2, .false., newp )
         write(imp,222)
         lines = lines + 2
         write(lgu,*) ermrk1
         write(lgu,222)
         write(lgu,*) ermrk2
         cycle                  ! Current problem is aborted
      end if

         ! SPECIFIC INITIALIZATIONS FOR A NEW PROBLEM
         ! ------------------------------------------
      call pacipb

         ! UPDATE CONTENT-CODE OF SAVE FILE IF RELEVANT
         ! --------------------------------------------
      if ( lsave .and. kontsv == 0 ) then
         if ( contsv(:5) == 'DERIV' ) kontsv = -1
      end if

         ! CALL DIRECTING ROUTINE OF PROGRAM-SPECIFIC SET
         ! ----------------------------------------------
      call pacdir( card, lcmdl, lcmd1, lcmdp, fltrp, kstop )

      call chkpg( 1, .false., newp )
      write(imp,333)
      lines = lines + 1
      call tcpu( lgu, .true., cpumsg )

         ! STOP OR LOOP FOR NEXT PROBLEM
         ! -----------------------------
      if ( kstop /= 0 ) exit
   end do

   INCLUDE 'pac001f.'

END SUBROUTINE pac001
