SUBROUTINE pacaf0( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, kfac, part, npar, allgt, lerr )
!***********************************************************************
!     - pacaf0 -
!     analyzes a "between parts-comparison" in the formula, which is
!     indicated by a factor (non-indexed letter) instead of modalities
!     (so that all modalities of the factor are concerned).
!     Creates the partition matrix associated to the comparison
!***********************************************************************
!
!     FORM  - INPUT . The formula                             - CHAR
!     NK    - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     KOL   - INPUT . Location in FORM of the character       - INTEGER
!                     following the factor-letter
!     IKAR  - INPUT . JCHAR rank of FORM(KOL:KOL), or IACHAR  - INTEGER
!                     rank of blank if the end of formula
!                     has been reached
!                     (JCHAR rank is IACHAR rank but with lower
!                     case letter returning same rank as the
!                     corresponding upper case letter)
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells are .true.
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     KFAC  - INPUT . The index of the factor found           - INTEGER
!                     (negative index for a T-factor)
!     PART  - INPUT . The partition matrix, initialized to 0  - INTEGER
!             OUTPUT. Updated
!     NPAR  - OUTPUT. Number of parts in the partition, i.e.  - INTEGER
!                     number of terms in the comparison
!                     (the complementary part is not counted)
!     ALLGT - INPUT . .true. if all cells in LEXS are .true.  - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CONSEC
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE is_digit
   USE pacint
   USE paclet
   USE pacrnk
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nk, kol, ikar, kfac
   LOGICAL, INTENT(IN) :: allgt
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   INTEGER, INTENT(INOUT) :: part(ntrt,ngrp)
   INTEGER, INTENT(OUT) :: npar
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER :: i, j, k, kfat, kola, kolz
   LOGICAL :: newp
   CHARACTER(LEN=1), PARAMETER :: mark='$'

   !--------------------------------------------------------------------

   lerr = .false.

   ! Update part
   ! -----------
      ! Note that categories coded as 0 or less are excluded from the
      ! partition (currently, only T-factors can have zero or negative
      ! category codes)
   if ( kfac > 0 ) then      ! It's a G-factor
      do j = 1, ngrp
         k = gstr(j,kfac)
!!!      if ( k > 0 ) then
            do i = 1, ntrt
               if ( lexs(i,j) ) part(i,j) = k
            end do
!!!      end if
      end do
      npar = gmod(kfac)

   else                      ! It's a T-factor
      kfat = -kfac
      do i = 1, ntrt
         k = tstr(i,kfat)
         if ( k > 0 ) then
            do j = 1, ngrp
               if ( lexs(i,j) ) part(i,j) = k
            end do
         end if
      end do
      npar = tmod(kfat)

   end if

   ! Check for expected character following the factor
   ! -------------------------------------------------
   if ( ikar == kblan .or. ikar == kinte .or. ikar == kprod .or. ikar == knest ) then

         ! End of between-comparison is found: normal return,
         ! but before, if gaps in part-numbers are suspected,
         ! suppress them.
         ! Gaps may arise when the cells in the design do not
         ! all exist (whether because originally empty or because
         ! a condition caused some groups/treatments to "disappear").
         ! On return from consec, npar is the actual number of
         ! parts in the partition-matrix

      if ( .not. allgt ) call consec( part, ntrt*ngrp, 0, maxint, k, npar )

   else
         ! An unexpected character has been found
      kola = max( kol-25, 1 )
      kolz = min( kol+25, nk )
!!    call chkpg( 6, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,111) form(kola:kolz), flag(:kol-kola+1), mark
!!    lines = lines + 4
      if ( llettr(ikar) ) then
         write(lgu,222)
!!       lines = lines + 1
      else if ( ldigit(ikar) ) then
         write(lgu,333)
!!       lines = lines + 2
      else
         write(lgu,444)
!!       lines = lines + 1
      end if
      write(lgu,*) ermrk2
      lerr = .true.

   end if

   INCLUDE 'pacaf0f.'

END SUBROUTINE pacaf0
