SUBROUTINE pacaf1( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                   flag, mat, kfac, vide, lerr )
!***********************************************************************
!     - pacaf1 -
!     analyzes a "simple part" in the formula,
!     updates the logical G&T-structure-matrix MAT.
!     A "simple part" is the subset of G&T corresponding to the
!     intersection of the modalities specified, thus the cells
!     indexed by these modalities are set to .true. and all others
!     to .false.
!***********************************************************************
!
!     FORM  - INPUT . The formula                             - CHAR
!     NK    - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     KOL   - INPUT . Starting location in FORM of the zone   - INTEGER
!                     analyzed
!             OUTPUT. Location in FORM of the character
!                     following the "simple part"
!     IKAR  - OUTPUT. JCHAR rank of FORM(KOL:KOL), or IACHAR  - INTEGER
!                     rank of blank if the end of formula
!                     has been reached
!                     (JCHAR rank is IACHAR rank but with lower
!                     case letter returning same rank as the
!                     corresponding upper case letter)
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     MAT   - INPUT . Logical G&T structure-matrix            - LOGICAL
!             OUTPUT. Cells not indexed by modalities in the
!                     simple part are reset to .false.
!     KFAC  - OUTPUT. If a factor (non-indexed letter) is     - INTEGER
!                     mentioned, then KFAC = IFAC(k) with
!                     k = IACHAR (factor symbol);
!                     0 otherwise
!     VIDE  - OUTPUT. .true. if all cells in MAT are .false.  - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        JCHAR
!     Fortran  functions called:
!        ACHAR  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE is_digit
   USE pacint
   USE paclet
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nk
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   INTEGER, INTENT(INOUT) :: kol
   LOGICAL, INTENT(INOUT) :: mat(ntrt,ngrp)
   INTEGER, INTENT(OUT) :: ikar, kfac
   LOGICAL, INTENT(OUT) :: vide, lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: jchar

   INTEGER :: i, j, kf, kola, kolz, n, nmax
   LOGICAL :: lfgt1, lgf, newp
   CHARACTER(LEN=1), PARAMETER :: mark='$'
   CHARACTER(LEN=1) :: c

   !--------------------------------------------------------------------

      ! First, check end of formula has not been reached yet
   if ( kol > nk ) then
      kola  = max( nk-25, 1 )
!!    call chkpg( 5, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,111) form(kola:nk), flag(:nk-kola+1), mark
      write(lgu,222)
      write(lgu,*) ermrk2
!!    lines = lines + 5
      lerr = .true.
      return
   end if

   lerr  = .false.
   ikar  = jchar( form(kol:kol) )
   vide  = .false.
   lfgt1 = .false.

   do   ! Looping on factor-symbols (letters)
        ! -----------------------------------

      c = form(kol:kol)
      kfac = ifac(ikar)
         ! Find what kind of factor (lgf) and which factor index (kf)
      if ( kfac > 0 ) then      ! It's a G-factor
         lgf = .true.
         kf  = kfac
      else
         if ( kfac < 0 ) then   ! It's a T-factor
            lgf = .false.
            kf  = -kfac
         else                   ! It's not a factor-symbol
            kola = max( kol-25, 1 )
            kolz = min( kol+25, nk )
!!          call chkpg( 5, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,111) form(kola:kolz), flag(:kol-kola+1), mark
            write(lgu,333)
            write(lgu,*) ermrk2
!!          lines = lines + 5
            lerr = .true.
            return
         end if
      end if

         ! Find n, the index of the associated modality
         ! --------------------------------------------
      n = 0
      newp = .true.
      do
         kol = kol + 1
         if ( kol <= nk ) then
            ikar = jchar( form(kol:kol) )
            if ( ldigit(ikar) ) then
               n = 10*n + ikar-krnk0
               newp = .false.
               cycle
            end if
         else
            ikar = kblan
         end if
         exit
      end do

      if ( newp ) then
            ! The factor is mentioned,
            ! then it must be the only one element in the simple part
         if ( lfgt1 .or. llettr(ikar) ) then
            kola = max( kol-25, 1 )
            kolz = min( kol+25, nk )
!!          call chkpg( 6, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,111) form(kola:kolz), flag(:kol-kola+1), mark
            write(lgu,444)
            write(lgu,*) ermrk2
!!          lines = lines + 6
            lerr = .true.
            return
         end if
         return
      else
            ! Check n is not out of range
         if ( lgf ) then
            nmax = gmod(kf)
         else
            nmax = tmod(kf)
         end if
         if ( n > nmax .or. n <= 0 ) then
            kola = max( kol-25, 1 )
            kolz = min( kol+25, nk )
!!          call chkpg( 5, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,111) form(kola:kolz), flag(:kol-kola+1), mark
            write(lgu,555) achar(jchar(c)), nmax
            write(lgu,*) ermrk2
!!          lines = lines + 5
            lerr = .true.
            return
         end if
      end if
      lfgt1 = .true.

         ! Update mat
         ! ----------
      if ( lgf ) then
            ! For all groups that are not indexed by the modality,
            ! set the T-vector to false (i.e. exclude it)
         do j = 1, ngrp
            if ( gstr(j,kf) /= n ) then
               do i = 1, ntrt
                  mat(i,j) = .false.
               end do
            end if
         end do

      else
            ! For all treatments that are not indexed by the modality,
            ! set the G-vector to false (i.e. exclude it)
         do i = 1, ntrt
            if ( tstr(i,kf) /= n ) then
               do j = 1, ngrp
                  mat(i,j) = .false.
               end do
            end if
         end do

      end if

         ! If next item is a letter, it is to be analyzed
         ! ----------------------------------------------
      if ( .not. llettr(ikar) ) exit
   end do

   kfac = 0

   ! Check the part is not empty
   ! ---------------------------
   do j = 1, ngrp
      do i = 1, ntrt
         if ( mat(i,j) ) return
      end do
   end do
      ! mat is empty (.false. for all cells)
   vide = .true.
   kola = max( kol-25, 1 )
   kolz = min( kol+25, nk )
   call chkpg( 5, .false., newp )
   write(imp,666) form(kola:kolz), flag(:kol-kola+1), mark
   lines = lines + 5
   write(lgu,*) ermrk1//ermrk1
   write(lgu,666) form(kola:kolz), flag(:kol-kola+1), mark
   write(lgu,*) ermrk2//ermrk2

   include 'pacaf1f.'

END SUBROUTINE pacaf1
