SUBROUTINE pacaf3( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, mat1, kfac, vide, lerr )
!***********************************************************************
!     - pacaf3 -
!     analyzes a "composition" in the formula,
!     i.e. a sequence   x&y&z etc   where x, y, z are "unions of simple
!     parts".
!     Updates the logical G&T structure-matrix MAT1.
!     A "composition" is the intersection of "unions of simple parts"
!***********************************************************************
!
!     FORM  - INPUT . The formula                             - CHAR
!     NK    - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     KOL   - INPUT . Starting location in FORM of the zone   - INTEGER
!                     analyzed
!             OUTPUT. Location in FORM of the character
!                     following the "composition"
!     IKAR  - OUTPUT. JCHAR rank of FORM(KOL:KOL), or IACHAR  - INTEGER
!                     rank of blank if the end of formula
!                     has been reached
!                     (JCHAR rank is IACHAR rank but with lower
!                     case letter returning same rank as the
!                     corresponding upper case letter)
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells are .true.
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     MAT1  - INPUT . Logical G&T structure-matrix            - LOGICAL
!             OUTPUT. Cells not indexed by modalities in the
!                     composition are reset to .false.
!     KFAC  - OUTPUT.    see definition in PACAF1             - INTEGER
!     VIDE  - OUTPUT. .true. if all cells in MAT1 are .false. - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACAF2
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ANY  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nk
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   INTEGER, INTENT(INOUT) :: kol
   LOGICAL, INTENT(INOUT) :: mat1(ntrt,ngrp)
   INTEGER, INTENT(OUT) :: ikar, kfac
   LOGICAL, INTENT(OUT) :: vide, lerr

   !  Local declarations
   !  ------------------

   INTEGER :: kola, kolz
   LOGICAL :: newp
   CHARACTER(LEN=1), PARAMETER :: mark='$'
   LOGICAL :: mat2(ntrt,ngrp)

   !--------------------------------------------------------------------

   lerr = .false.

   ! Analyze first term
   ! ------------------
   call pacaf2( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                lexs, flag, mat1, kfac, vide, lerr )
   if ( lerr ) return
      ! Check for empty union
   if ( vide ) then
      kola = max( kol-25, 1 )
      kolz = min( kol+25, nk )
      call chkpg( 5, .false., newp )
      write(imp,333) form(kola:kolz), flag(:kol-kola+1), mark
      lines = lines + 5
      write(lgu,*) ermrk1//ermrk1
      write(lgu,333) form(kola:kolz), flag(:kol-kola+1), mark
      write(lgu,*) ermrk2//ermrk2
   end if
      ! If complete factor or only one term, nothing to do here
   if ( kfac /= 0 .or. ikar /= kand ) return

   do   ! Looping on composition terms
        ! ----------------------------

         ! Analyze current term (resultant matrix is mat2)
         ! First, initialize mat2
      mat2 = lexs
      kol = kol + 1
      call pacaf2( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, mat2, kfac, vide, lerr )
      if ( lerr ) return
      if ( kfac /= 0 ) then
            ! If a complete factor appears here, it is an error
         kola = max( kol-26, 1 )
         kolz = min( kol+24, nk )
!!       call chkpg( 6, .false., newp )
         write(lgu,*) ermrk1
         write(lgu,111) form(kola:kolz), flag(:kol-kola), mark
         write(lgu,222)
         write(lgu,*) ermrk2
!!       lines = lines + 6
         lerr = .true.
         return
      end if
         ! Take the intersection of current term (in mat2) with
         ! previous ones (in mat1) (result in mat1)
      mat1 = mat1 .and. mat2
      vide = .not. any(mat1)
         ! Check for empty composition
      if ( vide ) then
         kola = max( kol-25, 1 )
         kolz = min( kol+25, nk )
         call chkpg( 5, .false., newp )
         write(imp,333) form(kola:kolz), flag(:kol-kola+1), mark
         lines = lines + 5
         write(lgu,*) ermrk1//ermrk1
         write(lgu,333) form(kola:kolz), flag(:kol-kola+1), mark
         write(lgu,*) ermrk2//ermrk2
      end if

         ! More terms ?
      if ( ikar /= kand ) exit
   end do

   INCLUDE 'pacaf3f.'

END SUBROUTINE pacaf3
