SUBROUTINE pacbay( k1, kn, lerr )

!***********************************************************************
!     - pacbay -
!     analyzes BAYes paragraph, with commands filtering
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines
!     pacchr : bayfil
!     pacint : ibayes
!     paclgl : lbayes  lrwdb
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CHKUNT  CMPRSS  CSCALX  CVUCT  FILOPN  GMPWRN
!     External functions called:
!        ISCALX  LSCALX
!     Fortran  functions called:
!        ADJUSTL  LEN_TRIM  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpint
   USE gmplgl
   USE pacchr
   USE pacint
   USE paclgl
   USE systm
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE chkunt( lerr,iout,iunt,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12 )
         IMPLICIT NONE
         INTEGER, INTENT(IN) :: iout, iunt
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11, u12
         LOGICAL, INTENT(OUT) :: lerr
      END SUBROUTINE chkunt
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1
   INTEGER, INTENT(INOUT) :: kn
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: iscalx
   LOGICAL, EXTERNAL :: lscalx

   INTEGER :: i, icol, jcol, nkar, nladd, numb
   LOGICAL :: lexis, lnamd, lnewb, lopen, newp

   INCLUDE 'pacbayp.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc

   !--------------------------------------------------------------------

   lerr = .false.

   ibayes = iscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, ibayes, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      lbayes = .false.
      return
   end if

   call cscalx( motc(2), lngm(2,1), lngm(2,2), k1, kn, bayfil, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
      lbayes = .false.
      return
   end if
   bayfil = adjustl( bayfil )
      ! Convert file name to capitals if not unix system
   if ( .not. lunix .and. icol > 0 ) call cvuct( bayfil )

   lrwdb = lscalx( motc(3), lngm(3,1), lngm(3,2), k1, kn, .false., icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      lbayes = .false.
      return
   end if

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   ! Prepare bayes file
   ! ------------------
   lbayes = .true.
   if ( ibayes >= 0 ) then

         ! Check unit number is permitted
      call chkunt( lerr, imp, ibayes, iu1, iu2, iu3, iu4, imp, lec, lec1, lec2, inpux, isave )
      if ( lerr ) then
         lbayes = .false.
         return
      end if
      if ( bayfil(1:1) == ' ' ) then
            ! Check unit is connected
         inquire (unit=ibayes,exist=lexis,opened=lopen,named=lnamd)
         lnewb = .not. (lopen .and. lexis)
         if ( lopen .and. lexis .and. lnamd ) then
            close (ibayes)
            lnewb = .true.
         end if
      else
            ! Check the file already exists
         inquire (file=bayfil,exist=lexis,opened=lopen,number=numb)
         lnewb = .not. (lopen .and. lexis)
         if ( lopen .and. lexis .and. numb /= ibayes ) then
!!          call chkpg( 5, .false., newp )
            i = min( lrecp-1, len_trim(bayfil) )
            write(lgu,*) mrker1
            write(lgu,222) bayfil(:i), numb, ibayes
            write(lgu,*) mrker2
!!          lines = lines + 5
            lerr = .true.
            lbayes = .false.
            return
         end if
      end if

   else

      if ( bayfil(1:1) /= ' ' ) then
            ! Use default bayes unit
         ibayes = kbayes
            ! Check the file already exists
         inquire (file=bayfil,exist=lexis,opened=lopen,number=numb)
         lnewb = .not. (lopen .and. lexis)
         if ( lopen .and. lexis .and. numb /= ibayes ) ibayes = numb
      else
         lbayes = .false.
         lnewb = .false.
         call chkpg( 3, .false., newp )
         write(imp,333)
         lines = lines + 3
         write(lgu,*) mrker1//mrker1
         write(lgu,333)
         write(lgu,*) mrker2//mrker2
      end if

   end if
   if ( lnewb ) then
      call filopn( 101, ibayes, bayfil, 'UNKNOWN', 'FORMATTED', .false., lerr )
      if ( lerr ) then
         lbayes = .false.
         return
      end if
   end if

   lrwdb = lnewb .or. lrwdb

   if ( lbayes .and. lrwdb ) rewind ibayes

   INCLUDE 'gmpf111.'
   INCLUDE 'pacbayf.'

END SUBROUTINE pacbay
