REAL(KIND=8) FUNCTION pacchi( x, iok )

!***********************************************************************
!     - pacchi -
!     returns the probability that a random variable distributed
!     according to a chi-square distribution is lower than or equal to x
!     The degrees of freedom parameter is passed through a module
!***********************************************************************
!
!     X     - INPUT . The value of the variable               - REAL*8
!     IOK   - OUTPUT. Return code                             - INTEGER
!                     0 = normal
!                     1 = invalid input argument
!                     2 = maximum number of iterations reached
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        CHI2CDF
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE pacdtr

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: x
   INTEGER, INTENT(OUT) :: iok


   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: chi2cdf
   REAL(KIND=8), PARAMETER :: dflimit=1.0e3_8

   !--------------------------------------------------------------------

   pacchi = chi2cdf( x, xdtr1, dflimit, iok )

END FUNCTION pacchi
