SUBROUTINE paccst( gsup, ag, at, v, w, udpw, nudpw, qcst, bcst, racst, &
                   chkall, sumv, sumw, samew, iupr )

!***********************************************************************
!     - paccst -  ????????????? revoir pour dl > 1 forme qcq
!     sets some ConSTants, according to V, W/U, and updates V if the
!     comparison is non product form
!***********************************************************************
!
!     GSUP  - OUTPUT.    see definition of MEM2 sub-arrays    - INTEGER
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     AT    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     V     - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!             OUTPUT. Updated if non product form comparison
!     W     - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     UDPW  - OUTPUT.    see definition of MEM2 sub-arrays    - REAL*8
!                     Used only if non product form
!                     Its usefull length will be NGSUP
!     NUDPW - INPUT . Actual length of UDPW                   - INTEGER
!                     (NGRP or 1 if not used)
!     QCST  - OUTPUT. Df of the joined comparison, i.e. sum   - REAL*8
!                     over G-comparison-support of (count-1)
!     BCST  - OUTPUT. A scalar such that  V'*D*V = BCST*Iv    - REAL*8
!                     where D is a diagonal matrix with
!                     1/AG(.) in diagonal, and Iv is the
!                     identity matrix of order NCV.
!                     Relevant only if the comparison is
!                     product form
!     RACST - OUTPUT. A scalar involved in computation of     - REAL*8
!                     sum of squares:
!                     If the comparison is product form it is
!                     the reciprocal of the scalar a such
!                     that  W'*d*W = a*Iw
!                     where d is a diagonal matrix with
!                     1/AT(.) in diagonal, and Iw is the
!                     identity matrix of order NCW.
!                     Otherwise it is the reciprocal of the
!                     square norm of the vector U
!     CHKALL- INPUT . If .true. checks SUMV and SUMW are      - LOGICAL
!                     performed over all vectors in V and W,
!                     if .false. only the 1st vector of
!                     each set is checked.
!                     Relevant only if the comparison is
!                     product form
!     SUMV  - OUTPUT. Whether zero or one depending on the    - REAL*8
!                     fact that the coefficients of the linear
!                     form(s) in V sum to zero or not
!                     If CHKALL is .false. only the 1st linear
!                     form is checked.
!                     Relevant only if the comparison is
!                     product form
!     SUMW  - OUTPUT. Whether zero or one depending on the    - REAL*8
!                     fact that the coefficients of the linear
!                     form(s) in W sum to zero or not
!                     If CHKALL is .false. only the 1st linear
!                     form is checked.
!                     Relevant only if the comparison is
!                     product form
!     SAMEW - INPUT . .true. if some results from previous    - LOGICAL
!                     comparison can be used, as yet checked
!                     Note that SAMEW is always .false. if
!                     the comparison is non product form
!             OUTPUT. .true. if .true. on input and current
!                     and previous comparisons have same
!                     GSUP and W
!     IUPR  - INPUT . Scratch unit number for storage of      - INTEGER
!                     previous results if SAMEW is .true.
!                     on input.
!                     Note that no rewind must be done here
!                     as managed by directing routine
!
!     --------------------------------------------------------
!     Module elements modified:
!     pacint : ngsup
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  ALL  SUM
!
!-----------------------------------------------------------------------

   USE pacint
   USE paclgl
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nudpw, iupr
   LOGICAL, INTENT(IN) :: chkall
   REAL(KIND=8), INTENT(IN) :: ag(ngrp), at(ntrt), w(ntrt,nszu)
   LOGICAL, INTENT(INOUT) :: samew
   REAL(KIND=8), INTENT(INOUT) :: v(ngrp,ncv)
   REAL(KIND=8), INTENT(OUT) :: qcst, bcst, racst, sumv, sumw
   REAL(KIND=8), INTENT(OUT) :: udpw(nudpw)
   INTEGER, INTENT(OUT) :: gsup(ngrp)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: temp
   INTEGER :: i, k, mgsup
   LOGICAL :: some
   REAL(KIND=8) :: wold(ntrt,nszu)   ! work arrays, used only if
   INTEGER      :: gsold(ngrp)       ! samew is .true. on input

   !--------------------------------------------------------------------

      ! mgsup is ngsup from previous comparison
   mgsup = ngsup
   qcst  = zero
   bcst  = zero
   racst = zero
   sumv  = zero
   sumw  = zero
   ngsup = 0

   if ( lupvw ) then

      ! Product form comparison
      ! -----------------------
      sumw = sum( w(:,1) )
      do i = 1, ntrt
         if ( at(i) > zero ) racst = racst + w(i,1)*w(i,1)/at(i)
      end do
      if ( abs(sumw) <= zerolk ) then
         if ( chkall ) then
            do k = 2, ncw
               sumw = sum( w(:,k) )
               if ( abs(sumw) > zerolk ) then
                  sumw = one
                  exit
               else
                  sumw = zero
               end if
            end do
         else
            sumw = zero
         end if
      else
         sumw = one
      end if

      outer: do k = 1, ngrp
         if ( abs(v(k,1)) > zerolk ) then
            ngsup   = ngsup + 1
            gsup(k) = ngsup
            qcst    = qcst + ag(k)
            bcst    = bcst + v(k,1)*v(k,1)/ag(k)
            sumv    = sumv + v(k,1)
            cycle outer
         else
            v(k,1)  = zero
         end if
         do i = 2, ncv
            if ( abs(v(k,i)) > zerolk ) then
               ngsup   = ngsup + 1
               gsup(k) = ngsup
               qcst    = qcst + ag(k)
               cycle outer
            else
               v(k,i)  = zero
            end if
         end do
         gsup(k) = 0
      end do outer
      if ( abs(sumv) <= zerolk ) then
         if ( chkall ) then
            do i = 2, ncv
               sumv = sum( v(1:ngrp,i) )
               if ( abs(sumv) > zerolk ) then
                  sumv = one
                  exit
               else
                  sumv = zero
               end if
            end do
         else
            sumv = zero
         end if
      else
         sumv = one
      end if

         ! If specific approach not requested, restore G-support set
         ! to whole G-set (and reset constants accordingly)
      if ( .not. lspecf ) then
         ngsup = ngrp
         qcst = sum( ag(1:ngrp) )
         do k = 1, ngrp
            gsup(k) = k
         end do
      end if

   else

      ! Non product form comparison (currently only 1 df)
      ! ---------------------------
      do k = 1, ngrp
         some = .false.
         temp = zero
         do i = 1, ntrt
            if ( abs(w(i,k)) > zerolk .and. at(i) > zero ) then
               some = .true.
               temp = temp + w(i,k)*w(i,k)/at(i)
            end if
         end do
         if ( some ) then
            v(k,1)      = one
            ngsup       = ngsup + 1
            gsup(k)     = ngsup
            racst       = racst + temp/ag(k)
            qcst        = qcst + ag(k)
            udpw(ngsup) = one / temp
         else
            v(k,1)      = zero
            gsup(k)     = 0
         end if
      end do

   end if

   racst = one / racst
   qcst  = qcst - ngsup

   ! Check results from previous comparison can be re-used
   ! -----------------------------------------------------
   samew = samew .and. ngsup == mgsup
   if ( samew ) then
      read(iupr) gsold
      samew = all( (gsup > 0) .eqv. (gsold > 0) )
      if ( .not. samew ) return
      read(iupr) wold
      samew = all( abs(w-wold) <= zerolk )
   end if

END SUBROUTINE paccst
