SUBROUTINE pacdpb( jus, kus, creg, zbar, nsuj, nwdep, nwcov )

!***********************************************************************
!     - pacdpb -
!     computes and saves the relevant Derived Protocol when covariates
!     are used
!***********************************************************************
!
!     JUS   - INPUT . Unit number for temporary storage of    - INTEGER
!                     input derived protocol
!     KUS   - INPUT . Unit number for temporary storage of    - INTEGER
!                     output derived protocol (i.e. if saving
!                     on user's save file not requested)
!     CREG  - INPUT . Matrix of covariate regression coeff.   - REAL*8
!                     dimensioned (NWCOV,NWDEP)
!     ZBAR  - INPUT . Vector for general means of covariates  - REAL*8
!                     multiplied by sum of W-coefficients
!                     of length NWCOV
!     NSUJ  - INPUT . Maximum number of cases on input        - INTEGER
!     NWDEP - INPUT . NCW * NDEP                              - INTEGER
!     NWCOV - INPUT . NCW * NCOVA                             - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SUM
!
!-----------------------------------------------------------------------

   USE gmpint
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: jus, kus, nsuj, nwdep, nwcov
   REAL(KIND=8), INTENT(IN) :: creg(nwcov,nwdep), zbar(nwcov)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: dwght, frq, rk
   INTEGER :: i, k, kase
   CHARACTER(LEN=incdw) :: slab
   REAL(KIND=8) :: xg(ngfac), yd(nwdep), yc(nwcov)
                 ! xg - work vector for grouping variables of case read
                 ! yd - work vector for dependent var. of case read
                 ! yc - work vector for covariates of case read

   !--------------------------------------------------------------------

      ! Note that ideriv was rewinded in pacder if relevant
   rewind jus
   if ( .not. lderiv ) rewind kus

   do kase = 1, nsuj

      read(jus,end=10) slab, frq, dwght, k, xg, yd, yc
         ! k is group-number according to "actual group-table"

         ! Compute derived protocol adjusted for covariates
      do i = 1, nwdep
         yd(i) = yd(i) - sum( (yc-zbar)*creg(:,i) )
      end do

         ! N.B. Correct weight not saved presently, replaced by 0.
      if ( lderiv ) then   ! Save on user's save-file
         rk = k
         write(ideriv,derfmt) slab, frq, zero, rk, xg, yd
      else                 ! Temporary storage
         write(kus) slab, frq, zero, k, xg, yd
      end if

   end do

10 continue

END SUBROUTINE pacdpb
