SUBROUTINE pacdpl( iudp, gsup, supg, ag, bar, nrb, ys2, labg, namy, ny )

!***********************************************************************
!     - pacdpl -
!     Lists the relevant Derived Protocol
!***********************************************************************
!
!     IUDP  - INPUT . Unit number for temporary storage of    - INTEGER
!                     derived protocol (to be used only if no
!                     user's save file, the derived protocol
!                     being otherwise saved on unit IDERIV
!                     with format DERFMT)
!     GSUP  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     SUPG  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     BAR   - INPUT . Matrix of means,                        - REAL*8
!                     two-dimensioned (NRB,NGSUP)
!     NRB   - INPUT . Number of rows in BAR                   - INTEGER
!     YS2   - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     LABG  - INPUT .    see definition of MEMB sub-arrays    - CHAR
!     NAMY  - INPUT . Array of names of derived variables     - CHAR
!                     of length NY
!     NY    - INPUT . Length of NAMY                          - INTEGER
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM  MIN  NINT  SQRT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iudp, nrb, ny
   REAL(KIND=8), INTENT(IN) :: ag(ngrp), bar(nrb,ngsup), ys2(nwdct)
   INTEGER, INTENT(IN) :: gsup(ngrp), supg(ngsup)
   CHARACTER(LEN=*), INTENT(IN) :: labg(ngrp), namy(ny)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: cwght, delt, frq, pmax, pmin, xk, yg, yi, ym
   INTEGER :: k, kase, kl, ks, lcmpt, mg, my
   CHARACTER(LEN=incdw) :: slab
   LOGICAL :: newp
   REAL(KIND=8) :: xg(ngfac), y(ny)
                 ! xg - work vector for grouping variables of case read
                 ! y  - work vector for derived protocol of case read

   INCLUDE 'pacdplp.'

   !--------------------------------------------------------------------

   lcmpt = min( lrecp-1, len_trim(cmptit) )

   call chkpg( 0, .true., newp )
   write(imp,111) cmptit(:lcmpt)
   write(imp,222)
   lines = lines + 4

   if ( lderiv ) then
      rewind ideriv
   else
      rewind iudp
   end if

      ! k     is group-number according to "actual group-table"
      ! frq   is case frequency
      ! cwght is case weight

   if ( lupvw .and. ny == 1 ) then     ! List and plot

      plot = ' '
         ! Calculate general mean (ym)
      yi = zero
      ym = zero
      yg = zero
      do ks = 1, ngsup
         k  = supg(ks)
         yi = yi + ag(k)
         ym = ym + ag(k)*bar(1,ks)
         yg = yg + bar(1,ks)
      end do
      if ( gpond == 1 ) then
         ym = yg / ngsup
      else
         ym = ym / yi
      end if

      if ( lrduce ) then
         yi = outy
      else
         yi = outy * sqrt( ys2(1) )
      end if
      pmin = ym - yi
      pmax = ym + yi
      delt = (yi+yi) / npts

      fmtn(33:34) = ' 2'
      fmty(19:20) = ' 2'
      write(imp,333)  cy, cg, cm, mtyp(gpond), ym, cp, yi, outy
      write(imp,fmtn) whdr, namy
      write(imp,111) '  '
      lines = lines + 4 + 2 + 1

      do kase = 1, nindvu
         if ( lderiv ) then
            read(ideriv,derfmt,end=10) slab, frq, cwght, xk, xg, y
            k = nint( xk )
         else
            read(iudp,end=10) slab, frq, cwght, k, xg, y
         end if
         ks = gsup(k)
         yg = bar(1,ks)
         yi = y(1)
         if ( yg >= pmax ) then
            mg = npt2
         else if ( yg <= pmin ) then
            mg = -npt2
         else
            mg = nint( (yg-ym)/delt )
         end if
         if ( yi >= pmax ) then
            my = npt2
         else if ( yi <= pmin ) then
            my = -npt2
         else
            my = nint( (yi-ym)/delt )
         end if
         plot(-npt2)= cp
         plot(npt2) = cp
         plot(0)    = cm
         plot(mg)   = cg
         plot(my)   = cy
         call chkpg( 1, .false., newp )
         write(imp,444) slab, labg(k), cwght, y, plot
         lines = lines + 1
            ! Restore
         plot(mg) = ' '
         plot(my) = ' '
      end do
   10 continue

   else                                ! List only
      k  = (lrecp-18) / 15
      kl = ny/k + 1
      write(fmtn(33:34),'(i2)') k
      fmty(19:20) = fmtn(33:34)
      write(imp,fmtn) whdr, namy
      write(imp,111) '  '
      lines = lines + 1 + kl + 1
      do kase = 1, nindvu
         if ( lderiv ) then
            read(ideriv,derfmt,end=20) slab, frq, cwght, xk, xg, y
            k = nint( xk )
         else
            read(iudp,end=20) slab, frq, cwght, k, xg, y
         end if
         call chkpg( kl, .false., newp )
         write(imp,fmty) slab, labg(k), cwght, y
         lines = lines + kl
      end do
   20 continue

   end if
   write(imp,*) ']]'   ! End mark of protocol

   INCLUDE 'pacdplf.'

END SUBROUTINE pacdpl
