SUBROUTINE pachmc( c, nrow, ncol, a )

!***********************************************************************
!     - pachmc -
!     normalizes a set of contrasts using the Harmonic Mean of square
!     norm of original Contrasts as the common square norm
!***********************************************************************
!
!     C     - INPUT . The matrix of contrasts coefficients    - REAL*8
!                     (columns are contrasts)
!             OUTPUT. Normalized contrasts
!     NROW  - INPUT . Number of rows in C (dimension of the   - INTEGER
!                     space)
!     NCOL  - INPUT . Number of columns (contrasts) in C      - INTEGER
!     A     - INPUT . The fundamental measure associated to   - REAL*8
!                     the contrasts, vector of length NROW
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SQRT  SUM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nrow, ncol
   REAL(KIND=8), INTENT(IN) :: a(nrow)
   REAL(KIND=8), INTENT(INOUT) :: c(nrow,ncol)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: r, s
   INTEGER :: j

   !--------------------------------------------------------------------

      ! Compute the common norm and reset each contrast to unit norm
   r = zero
   do j = 1, ncol
      s = sum( c(:,j)*c(:,j)/a )
      r = r + one/s
      s = one/sqrt(s)
      c(:,j) = c(:,j) * s
   end do
   r = sqrt( ncol/r )

      ! Rescale the contrasts using common norm
   c = c * r

END SUBROUTINE pachmc
