SUBROUTINE paclab( lab, symb, nmod, stru, nf, n, fsym )

!***********************************************************************
!     - paclab -
!     creates LABels for a set of elements corresponding to some factors
!     A label is the concatenation of factor-symbols (in lower case
!     letters) and category-numbers corresponding to the element, as far
!     as maximum possible label width is large enough to accomodate any
!     possible label; otherwise all labels reduce to a generic symbol
!     followed by the element-numbers
!***********************************************************************
!
!     LAB   - OUTPUT. Array of labels                         - CHAR
!     SYMB  - INPUT . Array of factors symbols                - CHAR
!     NMOD  - INPUT . Array of number of modalities per factor- INTEGER
!     STRU  - INPUT . Factor-structure matrix                 - INTEGER
!     NF    - INPUT . Number of factors                       - INTEGER
!     N     - INPUT . Number of elements                      - INTEGER
!     FSYM  - INPUT . Generic (type-of-factor) symbol         - CHAR
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ACHAR  IACHAR  INT  LOG10
!
!-----------------------------------------------------------------------

   USE gmpint   ! For parameter nocw: maximum label width

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nf, n
   CHARACTER(LEN=1), INTENT(IN) :: fsym
   INTEGER, INTENT(IN) :: nmod(nf), stru(n,nf)
   CHARACTER(LEN=*), INTENT(IN) :: symb(nf)
   CHARACTER(LEN=*), INTENT(OUT) :: lab(n)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: onepls=1.0_8+1.0e-13_8
      ! onepls must be slightly greater than 1, in order to
      ! counter-balance such things as log10(100) = 1.999999999999,
      ! so that int( onepls + log10(100) ) = 2
   REAL(KIND=8) :: t
   INTEGER, PARAMETER :: konv=iachar('A')-iachar('a') ! Conversion constant
   INTEGER, PARAMETER :: kzero=iachar('0')
   INTEGER :: i, j, k, l
   CHARACTER(LEN=29) :: frmt='(A,I1.1,A,I1.1,A,I1.1,A,I1.1)'
                      ! frmt must be at least 1+7*nocw/2 long

   !--------------------------------------------------------------------

   if ( nf <= nocw/2 ) then

         ! See whether or not label width is sufficient, according to
         ! number of digits for category-numbers
      l = nf
      k = 5
      do i = 1, nf
         t = nmod(i)
         j = int( log10(t) + onepls )
         l = l + j
         frmt(k  :k  ) = achar( j + kzero )
         frmt(k+2:k+2) = frmt(k:k)
         k = k + 7
      end do
      if ( l <= nocw ) then
            ! Ok, there is enough room
         do i = 1, n
            write(lab(i),frmt) (achar(iachar(symb(j))-konv), stru(i,j),j=1,nf)
         end do
         return
      end if

   end if

      ! Not enough room, so element-label is simply type-of-factor
      ! symbol followed by element-number
   do i = 1, n
      write(lab(i),"(a1,i3.3,'    ')") fsym, i
   end do

END SUBROUTINE paclab
