SUBROUTINE paclvo( outlev, maxou, noutlv, l1df1, baylev, hyp0, nomv )

!***********************************************************************
!     - paclvo -
!     updates variables controlling desired LeVel of Output
!     for the descriptive statistics of the derived protocol.
!     If the comparison has 1 df, the null hypothesis is printed
!***********************************************************************
!
!     OUTLEV- INPUT . Vector of keywords for descriptive      - CHAR
!                     statistics required, of length MAXOU
!             OUTPUT. May be converted to upper case letters
!     MAXOU - INPUT . Length of OUTLEV                        - INTEGER
!     NOUTLV- INPUT . Number of keywords entered in OUTLEV    - INTEGER
!             OUTPUT. Reset to zero
!     L1DF1 - INPUT . .true. if the comparison has 1 df,      - LOGICAL
!                     .false. otherwise
!     BAYLEV- INPUT . Bayes-probability level used for the    - REAL*8
!                     Bayes-fiducial statements
!     HYP0  - INPUT . Null hypothesis vector, of length NVTOT - REAL*8
!     NOMV  - INPUT . Names of the variables                  - CHAR
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     paclgl : levout
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG   CVUCT
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: baylev
   INTEGER, INTENT(IN) :: maxou
   LOGICAL, INTENT(IN) :: l1df1
   REAL(KIND=8), INTENT(IN) :: hyp0(nvtot)
   CHARACTER(LEN=*), INTENT(IN) :: nomv(nvtot)
   INTEGER, INTENT(INOUT) :: noutlv
   CHARACTER(LEN=*), INTENT(INOUT) :: outlev(maxou)

   !  Local declarations
   !  ------------------

!! REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, j
   LOGICAL :: gena, genb, lw1, newp
   CHARACTER(LEN=21) :: fmth="(  (1X,A8,'=',G14.7))"

   INCLUDE 'paclvop.'

   !--------------------------------------------------------------------

   !  Keywords may be individually entered (in which case non-entered
   !  options will be excluded from outputs; note that GST => STA and
   !  GCO => COR) or by means of a generic keyword with the following
   !  correspondence (where 1df means "yes if the comparison has only
   !  1 degree of freedom on treatments"):
   !       Keyword:  COE   STA   GST   SSP   COR   GCO   COV
   !       LEVOUT :  (1)   (2)   (3)   (4)   (5)   (6)   (7)
   !    Generic
   !     NONe        no    no    no    no    no    no    no
   !     BRIef       no    1df   1df   no    no    no    no
   !     NORmal      1df   1df   1df   no    1df   no    no
   !     DETail      1df   1df   1df   1df   1df   1df   no
   !     ALL         yes   yes   yes   yes   yes   yes   yes

   !--------------------------------------------------------------------

      ! If the comparison has 1 df, print the null hypothesis tested
   if ( l1df1 ) then
         ! Set number of values printed per line
      j = lrecp / (1+8+1+14)
      write(fmth(2:3),'(i2)') j
      j = (nvtot-1)/j + 1
      call chkpg( 2+j, .false., newp )
      write(imp,111)
      write(imp,fmth) (nomv(i),hyp0(i),i=1,nvtot)
      lines = lines + 2 + j
   end if

      ! No use updating if no change, just print output options
   if ( noutlv /= 0 ) then

         ! Scan for a generic keyword

         ! Convert 1st 3 letters of 1st entry to upper case
      call cvuct( outlev(1)(1:3) )

      lw1       = ncw == 1
      gena      = outlev(1)(:3) == gen(5)
      genb      = outlev(1)(:3) == gen(4)
      levout(7) = gena
      levout(6) = gena .or. (lw1 .and. genb)
      gena      = gena .or. genb
      genb      = outlev(1)(:3) == gen(3)
      levout(5) = levout(6) .or. (lw1 .and. genb)
      levout(4) = levout(6)
      gena      = gena .or. genb
      genb      = outlev(1)(:3) == gen(2)
      levout(3) = levout(5) .or. (lw1 .and. genb)
      gena      = gena .or. genb
      levout(2) = levout(3)
      levout(1) = levout(5)

      if ( .not. (gena .or. outlev(1)(:3) == gen(1)) ) then
            ! Scan for specific keywords

         outer: do i = 1, noutlv
               ! Convert 1st 3 letters of entry to upper case
            if ( i > 1 ) call cvuct( outlev(i)(1:3) )
            do j = 1, nkey
               if ( outlev(i)(1:3) == key(j) ) then
                  levout(j) = .true.
                  cycle outer
               end if
            end do
         end do outer
         if ( levout(6) ) levout(5) = .true.
         if ( levout(3) ) levout(2) = .true.

      end if

         ! Reset number of entries to 0
      noutlv = 0

   end if

!! call chkpg( 2+1+nkey+nkey2, .false., newp )
!! write(imp,222)
!! lines = lines + 2
!!    ! Print level for the partition matrix and associated means
!! if ( lpmean ) then
!!    if ( lptabl ) then
!!       write(imp,333) msg(0), noyes(1)
!!    else
!!       write(imp,333) msg(0), msgmo
!!    end if
!! else
!!    write(imp,333) msg(0), noyes(0)
!! end if
!! lines = lines + 1
!!    ! Print level for descriptive statistics on derived protocol
!! do i = 1, nkey
!!    if ( levout(i) ) then
!!       j = 1
!!    else
!!       j = 0
!!    end if
!!    write(imp,333) msg(i), noyes(j)
!! end do
!! lines = lines + nkey
!!    ! Print level for other options
!! tmp(1) = luniv
!! tmp(2) = lmulti
!! tmp(3) = lreduc
!! tmp(4) = limit .or. baylev > zero
!! tmp(5) = lanova
!! tmp(6) = listdp
!! call chkpg( nkey2, .false., newp )
!! do i = 1, nkey2
!!    if ( tmp(i) ) then
!!       j = 1
!!    else
!!       j = 0
!!    end if
!!    write(imp,333) msg(nkey+i), noyes(j)
!! end do
!! lines = lines + nkey2

   INCLUDE 'paclvof.'

END SUBROUTINE paclvo
