SUBROUTINE pacmis( ltmis, ctab, gapp, ag, lerr )

!***********************************************************************
!     - pacmis -
!     prints warning and makes correction if unauthorized MISsing data:
!     When count for at least one measurement in one group is neither
!     zero nor complete, it is reset to zero and warning is printed.
!     Note that when the missing data pattern is not the same for all
!     variables, this is considered as an error
!***********************************************************************
!
!     LTMIS - INPUT . .true. when at least one treatment is   - LOGICAL
!                     neither complete nor entirely missing
!                     for at least one group
!     CTAB  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!             OUTPUT.
!     GAPP  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     memst2 : ctab
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NINT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   LOGICAL, INTENT(IN) :: ltmis
   REAL(KIND=8), INTENT(IN) :: ag(ngrp)
   INTEGER, INTENT(IN) :: gapp(ngrp)
   INTEGER, INTENT(INOUT) :: ctab(ntrt,nvtot,ngrp)
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER :: i, j, jj, k, ko, n
   LOGICAL :: newp

   !--------------------------------------------------------------------

   lerr = .false.

   if ( ltmis ) then
      call chkpg( 8, .false., newp )
      write(imp,111)
      lines = lines + 7
      write(lgu,*) ermrk1//ermrk1
      write(lgu,111)
   end if

   do k = 1, ngrp
      ko = gapp(k)

      if ( ltmis ) then
            ! Correct unexpected missing data
         n = nint( ag(k) )
         do j = 1, nvtot

            do i = 1, ntrt
               if ( ctab(i,j,ko) > 0 ) then
                  if ( ctab(i,j,ko) /= n ) then
                     call chkpg( 1, .false., newp )
                     write(imp,222) k, n, j, i, ctab(i,j,ko)
                     lines = lines + 1
                     write(lgu,222) k, n, j, i, ctab(i,j,ko)
                     ctab(i,j,ko) = 0
                  end if
               end if
            end do

         end do
      end if

         ! Check the missing data pattern is the same for all variables
      do j = 2, nvtot
         jj = j - 1

         do i = 1, ntrt
            if ( ctab(i,j,ko) /= ctab(i,jj,ko) ) then
!!             call chkpg( 5, .false., newp )
               if ( ltmis ) write(lgu,*) ermrk2//ermrk2
               write(lgu,*) ermrk1
               write(lgu,333) k, j, i
               write(lgu,*) ermrk2
!!             lines = lines + 5
               lerr = .true.
               return
            end if
         end do

      end do

   end do
   if ( ltmis ) write(lgu,*) ermrk2//ermrk2

   INCLUDE 'pacmisf.'

END SUBROUTINE pacmis
