SUBROUTINE pacpla( k1, kn, lerr, unom, gsym, tsym, tusev, sizusv,  &
                   templ, lonce, lablg, lablt, iulid, lidfil )
!***********************************************************************
!     - pacpla -
!     analyzes PLAn paragraph, with commands filtering
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!     UNOM  - INPUT .    see definition of MEMA sub-arrays    - CHAR
!     GSYM  - OUTPUT. Symbols for G-factors                   - CHAR
!     TSYM  - OUTPUT. Symbols for T-factors                   - CHAR
!     TUSEV - INPUT . Temporary array for USEV                  INTEGER
!             OUTPUT.   (see definition of MEM2 sub-arrays)
!     SIZUSV- INPUT . Length of TUSEV                         - INTEGER
!     TEMPL - OUTPUT. Temporary array for G-labels            - CHAR
!                     of length NGUSER
!     LONCE - OUTPUT. .true. if covar. are given only once    - LOGICAL
!                     and thus are to be replicated over T
!     LABLG - INPUT . Whether or not user previously entered  - LOGICAL
!                     G-labels
!             OUTPUT. Set to .true. if user enters G-labels,
!                     otherwise reset to .false. unless none
!                     of keywords no. 3 to 8 is entered
!     LABLT - INPUT . Whether or not user previously entered  - LOGICAL
!                     T-labels
!             OUTPUT. Set to .true. if user enters T-labels,
!                     otherwise reset to .false. unless none
!                     of keywords no. 3 to 8 is entered
!     IULID - OUTPUT. Unit number for the EYELID save file    - INTEGER
!     LIDFIL- OUTPUT. Name of the EYELID save file            - CHAR
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines
!     memst2 : at  tstr  usev
!     memstb : nomv  nomt  labt
!     pacint : nuvar  ndep  ncova  nvtot  ngfa0
!              ngfac  nguser  ntrt  ntfac  ntdc
!     paclet : ifac
!     paclgl : lstax
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        AVECTX  AVECTZ  CHKPG  CMPRSS  CSCALX  CVUCT  GMPWRN  GMP999
!        LOCAL  PAC999
!     External functions called:
!        ISCALX  JCHAR  LSCALX
!     Fortran  functions called:
!        ACHAR  ADJUSTL  ALLOCATE  ALLOCATED  COUNT  DEALLOCATE  IACHAR
!        MAX  MINVAL
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpint
   USE gmplgl
   USE memst1
   USE memst2
   USE memstb
   USE pacint
   USE paclet
   USE paclgl
   USE pacrnk
   USE systm

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE avectx( mot, longm, long, ideb, iend, kode, nomv, lnom, &
                         incr, minx, maxx, imin, imax, icol, jcol, lerr, &
                         dx, rx, ix, lx, cx )
         INTEGER, INTENT(IN)  :: longm, long, ideb, iend, kode, incr, minx, maxx
         LOGICAL, INTENT(IN)  :: lnom
         CHARACTER(LEN=*), INTENT(IN) :: mot
         CHARACTER(LEN=*), DIMENSION(minx:*), INTENT(IN) :: nomv
         INTEGER, INTENT(OUT) :: imin, imax, icol, jcol
         LOGICAL, INTENT(OUT) :: lerr
         REAL(KIND=8)    , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: dx
         REAL            , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: rx
         INTEGER         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: ix
         LOGICAL         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: lx
         CHARACTER(LEN=*), DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: cx
      END SUBROUTINE avectx

      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1, sizusv
   CHARACTER(LEN=*), INTENT(IN) :: unom(nvaru)
   INTEGER, INTENT(INOUT) :: kn
   LOGICAL, INTENT(INOUT) :: lablg, lablt
   INTEGER, INTENT(INOUT) :: tusev(sizusv)
   INTEGER, INTENT(OUT) :: iulid
   LOGICAL, INTENT(OUT) :: lonce, lerr
   CHARACTER(LEN=*), INTENT(OUT) :: lidfil
   CHARACTER(LEN=1), INTENT(OUT) :: gsym(26), tsym(26)
   CHARACTER(LEN=nocw), INTENT(OUT) :: templ(nguser)

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: iscalx, jchar
   LOGICAL, EXTERNAL :: lscalx

   REAL(KIND=8), PARAMETER :: one=1.0_8
   INTEGER :: i, icol, icot, imax, imin, incr, ioc, j, jcol, jdum, kar
   INTEGER :: maxt, mcova, mdep, mgfac, mgfa0, mint, mtfac, mtrt, mvtot, mxgl
   INTEGER :: n, nkar, nladd, nttf, status, zgfa0
   LOGICAL :: newp, none, savtst
   CHARACTER(LEN=1) :: sym
   INTEGER :: locu(10)

   INCLUDE 'pacplap.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc

   !--------------------------------------------------------------------

      ! Values from previous PLAN paragraph
   mdep  = ndep
   mcova = ncova
   mvtot = nvtot
   mgfa0 = ngfa0
   mgfac = ngfac
   mtfac = max( 0, ntfac )
   mtrt  = ntrt

      ! iulid is unit number assigned to Eyelid save file
   iulid = 99

   lerr = .false.

   nguser = iscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, nguser, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      return
   end if
   nguser = max( 1, nguser )

   ndep = iscalx( motc(2), lngm(2,1), lngm(2,2), k1, kn, ndep, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
      return
   end if
   ndep = max( 1, ndep )
   none = icol <= 0

   ncova = iscalx( motc(3), lngm(3,1), lngm(3,2), k1, kn, ncova, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      return
   end if
   nvtot = ndep + ncova
   none  = none .and. icol <= 0

   ngfa0 = iscalx( motc(4), lngm(4,1), lngm(4,2), k1, kn, ngfa0, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
      return
   end if
   ngfac = max( 1, ngfa0 )
   none  = none .and. icol <= 0

   ntfac = iscalx( motc(5), lngm(5,1), lngm(5,2), k1, kn, ntfac, icol, jcol, lerr )
   if ( lerr ) then
      ntfac = mtfac
      call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
      return
   end if
   ntfac = max( 1, ntfac )
   none  = none .and. icol <= 0

   ntrt = iscalx( motc(6), lngm(6,1), lngm(6,2), k1, kn, ntrt, icol, jcol, lerr )
   if ( lerr ) then
      ntrt = mtrt
      call gmpwrn( pargr, linter, motc(6:), nmotc-5 )
      return
   end if
   ntrt = max( 1, ntrt )
   none = none .and. icol <= 0

   ntdc = ntrt * nvtot
   nttf = ntrt * ntfac

      ! If the dimensions of the T-structure are unchanged, then store
      ! on scratch unit (iu2) treatment weights and current T-structure
      ! to be used as defaults
   savtst = ntfac == mtfac .and. ntrt == mtrt
   if ( savtst ) then
      rewind iu2
      write(iu2) at
      write(iu2) tstr
   else
      if ( allocated(at) ) deallocate(at)
      if ( allocated(tstr) ) deallocate(tstr)
      allocate ( at(1:ntrt), tstr(ntrt,ntfac), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) mrker1
         write(lgu,101) " (PacPla)."
         write(lgu,*) mrker2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if
   end if

      ! Analysis of usev (subset of vuse in appropriate order)
   call local( motc(7), lngm(7,2), k1, kn, .false., 10, ioc, locu )
   if ( ioc > 0 ) then
      call local( sep2, 1, locu(ioc)+1, kn, .false., 1, j, inul )
         ! Convert variable names to upper case letters
      if ( j > 0 ) call cvuct( car(locu(1):inul(1)) )
      call avectz( motc(7), lngm(7,1), lngm(7,2), locu(1), kn,  &
                   unom(1:nvaru), 1, nvaru,                     &
                   tusev, 1, sizusv, jdum, icol, jcol, lerr )
      if ( lerr ) then
         call gmpwrn( pargr, linter, motc(7:), nmotc-6 )
         return
      end if
      nuvar = jdum
      none  = .false.
!!!      ! Filter for duplicate variables
!!!   call filter( tusev, nuvar, 0 )
      zgfa0 = 0
   else
      zgfa0 = mgfa0
   end if
      ! Allocate usev and fill it
   if ( ioc > 0 .or. .not. allocated(usev) ) then
      if ( allocated(usev) ) deallocate(usev)
      allocate ( usev(1:nuvar), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) mrker1
         write(lgu,101) " (PacPla)."
         write(lgu,*) mrker2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if
      usev = tusev(1:nuvar)
   end if

   if ( .not. none ) then
      lablg = .false.
      lablt = .false.
   end if

   ! Some checks
   ! -----------
      ! Check for too many factors
   if ( ngfac+ntfac > 26 ) then
!!    call chkpg( 2, .false., newp )
      write(lgu,*) mrker1
      write(lgu,444) 26
      write(lgu,*) mrker2
!!    lines = lines + 2
      goto 10
   end if

      ! Check the list of variables used is consistent with the
      ! keywords given here, and set lonce to proper value
   lonce = .false.
   n = nuvar - ngfa0 - ntrt*ndep
   if ( n < 0 ) then
!!    call chkpg( 5, .false., newp )
      write(lgu,*) mrker1
      write(lgu,222) ngfa0, ntrt, ndep, ncova, nuvar
      write(lgu,*) mrker2
!!    lines = lines + 5
      goto 10
   else
      if ( n == ncova ) then
         lonce = ntrt > 1
      else if ( n /= ntrt*ncova ) then
!!       call chkpg( 5, .false., newp )
         write(lgu,*) mrker1
         write(lgu,222) ngfa0, ntrt, ndep, ncova, nuvar
         write(lgu,*) mrker2
!!       lines = lines + 5
         goto 10
      end if
   end if

   ! Allocate arrays
   ! ---------------
   if ( .not. allocated(nomv) .or. nvtot /= mvtot ) then
      if ( allocated(nomv) .and. nvtot /= mvtot ) deallocate(nomv)
      allocate ( nomv(1:nvtot), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) mrker1
         write(lgu,101) " (PacPla)."
         write(lgu,*) mrker2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if
   end if
   if ( .not. allocated(nomt) .or. ntfac /= mtfac ) then
      if ( allocated(nomt) ) deallocate(nomt)
      allocate ( nomt(1:ntfac), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) mrker1
         write(lgu,101) " (PacPla)."
         write(lgu,*) mrker2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if
   end if
   if ( .not. allocated(labt) .or. ntrt /= mtrt ) then
      if ( allocated(labt) ) deallocate(labt)
      allocate ( labt(1:ntrt), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) mrker1
         write(lgu,101) " (PacPla)."
         write(lgu,*) mrker2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if
   end if

   ! Set up most default values (except T-symbols)
   ! --------------------------
      ! Default names for T-factors
   do i = mtfac+1, ntfac
      write(nomt(i),333) i
   end do

      ! Default names for variables
      ! (Unchanged if the list of variables and  parameters remain the same)
   if ( ioc > 0 .or. ngfa0 /= mgfa0 .or. ntrt  /= mtrt .or.  &
                     ndep  /= mdep  .or. ncova /= mcova ) then
      j = ngfa0 + 1
      do i = 1, ndep
         nomv(i) = unom( tusev(j) )
         j = j + ntrt
      end do
      if ( lonce ) then
         incr = 1
      else
         incr = ntrt
      end if
      do i = ndep+1, nvtot
         nomv(i) = unom( tusev(j) )
         j = j + incr
      end do
   end if

      ! Default structure of T-factors
   if ( savtst ) then
      rewind iu2
      read(iu2) at
      read(iu2) tstr
   else
         ! Default treatment weights
      at = one
         ! Default structure if only one T-factor
      if ( ntfac == 1 ) then
         do i = 1, ntrt
            tstr(i,1) = i
         end do
      else
         tstr = -1  ! Arbitrary negative number coding for emptiness
      end if
   end if

      ! Default G-symbols (1st letter of corresponding factor name)
      ! with possible conversion to capitals and ifac updating
      ! (through zgfa0 starting point for defaults depends on
      ! whether usev was entered, so that previous symbols are left
      ! unchanged when no usev, and all symbols are reset when usev)
   if ( ngfa0 == 0 ) then
      gsym(1) = ' '
   else
      do i = zgfa0+1, ngfa0
         kar = jchar( unom(tusev(i))(1:1) )
         gsym(i) = achar( kar )
         ifac(kar) = i
      end do
   end if

   ! Complete analyses
   ! -----------------
      ! Analysis of nomv (names of depend. variables and covariates)
   call avectx( motc(8), lngm(8,1), lngm(8,2), k1, kn, 5,            &
                cnul, .false., 1, 1, nvtot, imin, imax, icol, jcol,  &
                lerr, cx=nomv )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(8:), nmotc-7 )
      return
   end if

      ! Analysis of nomt (T-factor names)
   call avectx( motc(9), lngm(9,1), lngm(9,2), k1, kn, 5,            &
                cnul, .false., 1, 1, ntfac, imin, imax, icol, jcol,  &
                lerr, cx=nomt )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(9:), nmotc-8 )
      return
   end if

      ! Default T-symbols (1st letter of corresponding factor name)
      ! with conversion of names to capitals and ifac updating
   if ( ntrt > 1 .or. icol > 0 ) then
      if ( icol > 0 ) then
         j = 1
      else
         j = max( 1, mtfac )
      end if
      do i = j, ntfac
         call cvuct( nomt(i) )
         tsym(i) = nomt(i)(1:1)
         kar = iachar( tsym(i) )
         ifac(kar) = -i
      end do
   else
      tsym(1) = ' '
   end if

      ! Analysis of gsym (G-factor symbols)
   call avectx( motc(10), lngm(10,1), lngm(10,2), k1, kn, 5,         &
                cnul, .false., 1, 1, ngfac, imin, imax, icol, jcol,  &
                lerr, cx=gsym )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(10:), nmotc-9 )
      return
   end if
   if ( icol > 0 ) then
         ! Convert to capitals and update ifac
      do i = krnka, krnkz
        if ( ifac(i) > 0 ) ifac(i) = 0
      end do
      do i = 1, ngfac
         kar = jchar( gsym(i) )
         gsym(i) = achar( kar )
         ifac(kar) = i
      end do
   end if

      ! Analysis of tsym (T-factor symbols)
   call avectx( motc(11), lngm(11,1), lngm(11,2), k1, kn, 5,         &
                cnul, .false., 1, 1, ntfac, imin, imax, icol, jcol,  &
                lerr, cx=tsym )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(11:), nmotc-10 )
      return
   end if
   if ( icol > 0 ) then
         ! Convert to capitals and update ifac
      do i = krnka, krnkz
        if ( ifac(i) < 0 ) ifac(i) = 0
      end do
      do i = 1, ntfac
         kar = jchar( tsym(i) )
         tsym(i) = achar( kar )
         ifac(kar) = -i
      end do
   end if

      ! Analysis of labt (treatment labels)
   call avectx( motc(12), lngm(12,1), lngm(12,2), k1, kn, 5,        &
                cnul, .false., 1, 1, ntrt, imin, imax, icol, jcol,  &
                lerr, cx=labt )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(12:), nmotc-11 )
      return
   end if
   if ( icol > 0 ) lablt = .true.

      ! Analysis of lat (treatment weights)
   call avectx( motc(13), lngm(13,1), lngm(13,2), k1, kn, 1,        &
                cnul, .false., 1, 1, ntrt, imin, imax, icol, jcol,  &
                lerr, dx=at )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(13:), nmotc-12 )
      return
   end if

      ! Analysis of tstr (T-factors structure)
      ! (tstr is dimensioned (ntrt,ntfac) thus treatments vary faster)
   call avectx( motc(14), lngm(14,1), lngm(14,2), k1, kn, 3,          &
                nomt, .true., ntrt, 1, nttf, mint, maxt, icot, jcol,  &
                lerr, ix=tstr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(14:), nmotc-13 )
      return
   end if

      ! Analysis of templ (group labels)
      ! First determine maximum number of labels that can be entered,
      ! as actual number of groups is not known yet
   mxgl = nguser
   call avectx( motc(15), lngm(15,1), lngm(15,2), k1, kn, 5,        &
                cnul, .false., 1, 1, mxgl, imin, imax, icol, jcol,  &
                lerr, cx=templ )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(15:), nmotc-14 )
      return
   end if
   if ( icol > 0 ) lablg = .true.

      ! Analysis of lidfil (name of Eyelid save file)
   lidfil = ' '
   call cscalx( motc(16), lngm(16,1), lngm(16,2), k1, kn, lidfil, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(16:), nmotc-15 )
      return
   end if
      ! Convert file name to capitals if not unix system
   lidfil = adjustl( lidfil )
   if ( .not. lunix .and. icol > 0 ) call cvuct( lidfil )

      ! Analysis of lstax (request for group-statistics output)
   lstax = lscalx( motc(17), lngm(17,1), lngm(17,2), k1, kn, lstax, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(17:), nmotc-16 )
      return
   end if

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   ! Check T-structure was given and complete
   ! ----------------------------------------
   if ( icot > 0 ) then
      if ( minval(tstr) < 0 ) then
!!       call chkpg( 4, .false., newp )
         write(lgu,*) mrker1
         do i = 1, ntfac
            if ( minval(tstr(1:ntrt,i)) < 0 ) then
               write(lgu,666) nomt(i), count(tstr(1:ntrt,i)>=0), ntrt
               exit
            end if
         end do
         write(lgu,*) mrker2
!!       lines = lines + 4
         lerr  = .true.
      end if
   else if ( .not. savtst .and. ntfac > 1 ) then
!!    call chkpg( 2, .false., newp )
      write(lgu,*) mrker1
      write(lgu,555)
      write(lgu,*) mrker2
!!    lines = lines + 2
      lerr  = .true.
   end if

   ! Check unicity of factor symbols
   ! -------------------------------
   do i = 1, ngfa0
      sym = gsym(i)
      do j = i+1, ngfa0
         if ( sym == gsym(j) ) goto 20
      end do
      do j = 1, ntfac
         if ( sym == tsym(j) ) goto 20
      end do
   end do
   do i = 1, ntfac-1
      sym = tsym(i)
      do j = i+1, ntfac
         if ( sym == tsym(j) ) goto 20
      end do
   end do

   ! If G-factors, check codes/cutpoints were given
   ! ----------------------------------------------
   do i = 1, ngfa0
      if ( lnom(vuse(usev(i))) == 0 ) then
         call chkpg( 7, .false., newp )
         write(imp,999) unom(usev(i))
         lines = lines + 7
         write(lgu,*) mrker1//mrker1
         write(lgu,999) unom(usev(i))
         write(lgu,*) mrker2//mrker2
      end if
   end do

   ! Normal return
   ! -------------
   return

   ! Error return
   ! ------------
10 continue
   if ( linter ) then
      ndep  = mdep
      ncova = mcova
      ngfa0 = mgfa0
      ngfac = mgfac
      ntfac = mtfac
      ntrt  = mtrt
      call chkpg( 3, .false., newp )
      write(imp,777)
      lines = lines + 3
   end if
   call gmpwrn( pargr, linter, motc(8:), nmotc-7 )
   lerr = .true.
   return

20 continue
!! call chkpg( 2, .false., newp )
   write(lgu,*) mrker1
   write(lgu,888) sym
   write(lgu,*) mrker2
!! lines = lines + 2
   lerr = .true.

   INCLUDE 'allocf.'
   INCLUDE 'gmpf111.'
   INCLUDE 'pacplaf.'

END SUBROUTINE pacpla
