SUBROUTINE pacprx( iout, lchk, lfgrp )

!***********************************************************************
!     - pacprx -
!     PRints descriptive statistics for "X" (basic data set)
!***********************************************************************
!
!     IOUT  - INPUT . Unit number for printing                - INTEGER
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!     LFGRP - INPUT . .true. if NGFA0 > 0                     - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN  NINT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memst2
   USE memsta
   USE memstb
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iout
   LOGICAL, INTENT(IN) :: lchk, lfgrp

   !  Local declarations
   !  ------------------

   INTEGER :: i, ij, j, jj, k, ko, l, linpp, ll, lnm, lx, ncount, ngpl, nladd, nlr, nx
   LOGICAL :: newp
   CHARACTER(LEN=82) :: fmtg='(2X/A9,I3:,":",T14,3(" (",A,"=",A,")")/" ===========",(T14,3(:" (",A,"=",A,")":)))'

   INCLUDE 'pacprxp.'

   !--------------------------------------------------------------------

      ! The maximum number of grouping-variables printed per line is 3,
      ! it is given by : Max = (79-13)/(nocw+nocw+4)
   write(fmtg(20:20),'(i1)') min( ngfac, 3 )
      ! ngpl is number of lines for printing of group information
   ngpl = max( 5, (ngfac-1)/3 + 4 )
   if ( lmima ) then
      nx = 3
   else
      nx = 1
   end if
   if ( lchk ) then
      call chkpg( 0, .true., newp )
      linpp = linep
   else
      linpp = 99999
   end if
   write(iout,111)
   lines = lines + 5
   lnm = max( l_noms, 1 )

   do k = 1, ngrp
      ko = gapp(k)
         ! When checking, 4 lines are added in order to print at least
         ! 1 treatment on the same page as group info. and var. name
      if ( lchk ) call chkpg( ngpl+4, .false., newp )

      if ( lfgrp ) then
            ! Note this block-if could be avoided by simply replacing
            ! ngfac by ngfa0 in the following write statement,
            ! but this may be risky at run-time with some compilers
         write(iout,fmtg) zgrp, k, ( unom( usev(i) ),   &
                          noms( max(1,min(lnm,lnom(vuse(usev(i)))-1+gcod(i,ko))) ),i=1,ngfac)
      else
         write(iout,222) zgrp, k
      end if

      ncount = nint( xbar(nlxb,ko) )
      write(iout,333) labg(k), ncount
      lines = lines + ngpl

         ! Print dependent variables and covariates
      jj = -ntrt
      do j = 1, nvtot
         jj = jj + ntrt

         if ( lchk ) call chkpg( 4, .false., newp )
         write(iout,444) nomv(j)
         lines = lines + 3

            ! nlr   = no. of lines still to be printed
            ! nladd = no. of lines in current page
         nlr = ntrt
         ij = jj
         ll = 0
         do
            nladd = min( nlr, max( linpp-lines, 1 ) )
            do l = 1, nladd
               ij = ij + 1
               ll = ll + 1
               if ( ctab(ij,ko) == ncount ) then
                  write(iout,555) labt(ll), (xbar(ij+lx*ntdc,ko),lx=0,nx)
               else
                  write(iout,666) labt(ll)
               end if
            end do
            lines = lines + nladd
            nlr = nlr - nladd
            if ( nlr <= 0 ) exit
            if ( lchk ) call chkpg( 0, .true., newp )
         end do

      end do

   end do

   INCLUDE 'pacprxf.'

END SUBROUTINE pacprx
