SUBROUTINE pacsgt( iout, lchk, gsym, tsym, lonce, allgt )

!***********************************************************************
!     - pacsgt -
!     prints design structure:
!     correspondence between treatments and variables and input objects,
!     Treatment anf Group Structures (between and within designs),
!     cross-table of groups by treatments
!***********************************************************************
!
!     IOUT  - INPUT . Unit number for printing                - INTEGER
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!     GSYM  - INPUT . Array of symbols of G-factors           - CHAR
!     TSYM  - INPUT . Array of symbols of T-factors           - CHAR
!     LONCE - INPUT . Whether covariates were given only once - LOGICAL
!     ALLGT - INPUT . Whether the G&T cross-table is complete - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        CHAR  IACHAR  MAX  MIN  SUM
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE memst1
   USE memst2
   USE memsta
   USE memstb
   USE pacint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iout
   LOGICAL, INTENT(IN) :: lchk, lonce, allgt
   CHARACTER(LEN=*), INTENT(IN) :: gsym(ngfac), tsym(ntfac)

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: sumgw, sumtw
   INTEGER, PARAMETER :: konv=iachar('A')-iachar('a') ! Conversion constant
   INTEGER :: i, ii, j, l, linpp, ll, lnm, maxil, nbeg, nend, nladd, nlr, nv
   LOGICAL :: newp
   INTEGER :: iwrk(nuvar-ngfa0), jwrk(nuvar-ngfa0)
   CHARACTER(LEN=8) :: cara(0:1)=(/ '   --   ', ' XXXXXX ' /)

   INCLUDE 'pacsgtp.'

   !--------------------------------------------------------------------

   if ( lchk ) then
      call chkpg( 13, .false., newp )
      linpp = linep
   else
      linpp = 99999
   end if
   write(iout,111)
   lines = lines + 5

   ! First, build up iwrk and jwrk
   ! -----------------------------
      ! iwrk(i) is treatment-number corresponding to ith object used
      !         (grouping-factors not counted)
      ! jwrk(i) is variable-number  corresponding to ith object used
      !         (grouping-factors not counted)
   if ( lonce ) then
      nv = ndep
   else
      nv = nvtot
   end if
   ii = 0
   do j = 1, nv
      do i = 1, ntrt
         ii = ii + 1
         iwrk(ii) = i
         jwrk(ii) = j
      end do
   end do
   do j = nv+1, nvtot
      ii = ii + 1
      iwrk(ii) = 1
      jwrk(ii) = j
   end do

   ! Print covariate-names (if any)
   ! ------------------------------
   if ( ncova > 0 ) then
      nv = (ncova-1)/5 + 1
      if ( lchk ) call chkpg( nv, .false., newp )
      write(iout,4444) (nomv(j),j=ndep+1,nvtot)
      lines = lines + nv
   end if

      ! maxil is maximum number of items printed per line
      ! (common to all printing-sections)
   maxil = (lrecp-10) / 9

   ! Print correspondence of treatments and variables / input objects
   ! ----------------------------------------------------------------
   if ( lchk ) call chkpg( 8, .false., newp )
   write(iout,222)
   lines = lines + 2

   nv = nuvar - ngfa0
   nend = 0
   do
      nbeg = nend + 1
      nend = min( nend+maxil, nv )

      if ( lchk ) call chkpg( 6, .false., newp )
      write(iout,*)   blank
      write(iout,666) msgo1,  (unom(usev(ngfa0+i)),i=nbeg,nend)
      write(iout,666) undlin, (undlin       ,i=nbeg,nend)
      write(iout,555) msgo2,  (iwrk(i)      ,i=nbeg,nend)
      write(iout,666) msgo3,  (labt(iwrk(i)),i=nbeg,nend)
      write(iout,666) msgo4,  (nomv(jwrk(i)),i=nbeg,nend)
      lines = lines + 6
      if ( nend >= nv ) exit
   end do

   ! Print treatment-structure (within design)
   ! -----------------------------------------
   if ( ntrt > 1 ) then

      if ( lchk ) call chkpg( 8, .false., newp )
      write(iout,333)
      lines = lines + 2

      nend = 0
      do
         nbeg = nend + 1
         nend = min( nend+maxil, ntrt )

         if ( lchk ) call chkpg( 6, .false., newp )
         write(iout,*)   blank
         write(iout,777) blank , ('o', i ,i=nbeg,nend)
         write(iout,666) undlin, (undlin ,i=nbeg,nend)
         write(iout,888) msgo7 , (at(i)  ,i=nbeg,nend)
         write(iout,666) msgo3 , (labt(i),i=nbeg,nend)
         lines = lines + 5
            ! nlr   = # of lines still to be printed
            ! nladd = # of lines in current page
         nlr = ntfac
         ll = 0
         do
            nladd = min( nlr, max( linpp-lines, 1 ) )
            do l = 1, nladd
               ll = ll + 1
               write(iout,777) nomt(ll), (char(iachar(tsym(ll))-konv), &
                               tstr(i,ll),i=nbeg,nend)
            end do
            lines = lines + nladd
            nlr = nlr - nladd
            if ( nlr <= 0 ) exit
            if ( lchk ) call chkpg( 0, .true., newp )
         end do

         if ( nend >= ntrt ) exit
      end do

   end if

   ! Print group-structure (between design)
   ! --------------------------------------
   if ( ngfa0 > 0 ) then

      if ( lchk ) call chkpg( 8, .false., newp )
      write(iout,444)
      lines = lines + 2

      lnm = max( l_noms, 1 )
      nend = 0
      do
         nbeg = nend + 1
         nend = min( nend+maxil, ngrp )

         if ( lchk ) call chkpg( 6, .false., newp )
         write(iout,*)   blank
         write(iout,777) blank , ('g', i ,i=nbeg,nend)
         write(iout,666) undlin, (undlin ,i=nbeg,nend)
         write(iout,999) msgo5 , (ag(i)  ,i=nbeg,nend)
         write(iout,666) msgo6 , (labg(i),i=nbeg,nend)
         lines = lines + 5
            ! nlr   = # of lines still to be printed
            ! nladd = # of lines in current page
         nlr = ngfa0 + ngfa0
         ll = 0
         do
            nladd = min( nlr, max( linpp-lines, 2 ) ) / 2
            do l = 1, nladd
               ll = ll + 1
               j = usev(ll)
               write(iout,777) unom(j),  &
                               (char(iachar(gsym(ll))-konv),gstr(i,ll),i=nbeg,nend)
               write(iout,666) blank,    &
                               (noms( max(1,min(lnm,lnom(vuse(j))-1+gcod(ll,gapp(i)))) ),i=nbeg,nend)
            end do
            lines = lines + (nladd + nladd)
            nlr = nlr - (nladd + nladd)
            if ( nlr <= 0 ) exit
            if ( lchk ) call chkpg( 0, .true., newp )
         end do

         if ( nend >= ngrp ) exit
      end do

   end if

   ! Print total group counts and total treatment weights
   ! ----------------------------------------------------
   sumgw = sum( ag(1:ngrp) )
   sumtw = sum( at(1:ntrt) )
   if ( lchk ) call chkpg( 2, .false., newp )
   write(iout,1111) sumgw, sumtw
   lines = lines + 2

   ! Print GxT cross-table
   ! ---------------------
      ! Note that only the cross-table corresponding to the first
      ! variable is printed, as the tables are supposed to be identical
      ! for all variables
      ! G's are printed as rows and T's as columns

      ! If trivial the table is skipped
   if ( ntrt == 1 .or. ngrp == 1 ) return

   if ( lchk ) call chkpg( 7, .false., newp )
   write(iout,2222)
   lines = lines + 2

      ! If complete and large, the table is replaced by a message
   if ( allgt .and. ngrp*ntrt > 64 ) then
      write(iout,3333)
      lines = lines + 2

   else
      nend = 0
      do
         nbeg = nend + 1
         nend = min( nend+maxil, ntrt )

         if ( lchk ) call chkpg( 5, .false., newp )
         write(iout,*)   blank
         write(iout,666) blank,  (labt(i),i=nbeg,nend)
         write(iout,666) undlin, (undlin ,i=nbeg,nend)
         lines = lines + 3
            ! nlr   = # of lines still to be printed
            ! nladd = # of lines in current page
         nlr = ngrp
         ll = 0
         do
            nladd = min( nlr, max( linpp-lines, 1 ) )
            do l = 1, nladd
               ll = ll + 1
               j = gapp(ll)
               write(iout,666) labg(ll), (cara(min(1,ctab(i,j))),i=nbeg,nend)
            end do
            lines = lines + nladd
            nlr = nlr - nladd
            if ( nlr <= 0 ) exit
            if ( lchk ) call chkpg( 0, .true., newp )
         end do

         if ( nend >= ntrt ) exit
      end do
   end if

   INCLUDE 'pacsgtf.'

END SUBROUTINE pacsgt
