SUBROUTINE pacyds( samew, ngprt, nvar, labg, labv, nomwr, nomv, sumsq, scalar )

!***********************************************************************
!     - pacyds -
!     prints Descriptive Statistics on the derived protocol "Y"
!     (thus computes them if not yet available)
!***********************************************************************
!
!     SAMEW - INPUT . Flags results from previous comparison  - LOGICAL
!                     can be used for current one
!     NGPRT - INPUT . Number of groups involved in printout   - INTEGER
!                     of means and std. deviation per group
!                     (NGSUP or 0)
!     NVAR  - INPUT . Number of dependent variables concerned - INTEGER
!     LABG  - INPUT . Array of group labels,                  - CHAR
!                     of length NGRP
!     LABV  - INPUT . Array of V-contrasts labels,            - CHAR
!                     of length NCV
!     NOMWR - INPUT . Array of W-contrasts & variables labels - CHAR
!                     of length NCW*NVAR
!     NOMV  - INPUT .    see definition of MEMB sub-arrays    - CHAR
!     SUMSQ - INPUT . .true. if the matrix of the sum of      - LOGICAL
!                     squares & products of the comparison
!                     is to be printed
!     SCALAR- INPUT . A scalar used to multiply the EFF'*EFF  - REAL*8
!                     matrix to obtain the matrix of the sum
!                     of squares & products of the comparison
!                     Unused if SUMSQ is .false.
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACPRY  PACYSS  PRTMAT  PRTSYM
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM  MAX  MIN  SQRT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE memst2
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: scalar
   INTEGER, INTENT(IN) :: ngprt, nvar
   LOGICAL, INTENT(IN) :: samew, sumsq
   CHARACTER(LEN=*), INTENT(IN) :: labg(ngrp), labv(ncv), nomwr(ncw*nvar), nomv(nvtot)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: ragk
   INTEGER :: iout, j, jj, jk, k, ks, lcmpt, ll, nv, nwvar
   INTEGER :: zsgs1, zsgs3, zx
   LOGICAL :: newp
   CHARACTER(LEN=40) :: cortex
   REAL(KIND=8) :: temp(ncw*nvar*(ngsup+1))
   CHARACTER(LEN=8) :: blank(1)='        '

   INCLUDE 'pacydsp.'

      ! zsgsi is an adaptation of lsgs(i) in case of a unique group
      ! zx will be associated to pooled correl./covar. matrix header

   !--------------------------------------------------------------------

      ! Copy to avoid double transmission of argument
   iout = imp
   nv = ncv
      ! Length of comparison-title for printing
   lcmpt = min( lrecp-1, len_trim(cmptit) )

   nwvar = ncw * nvar

   if ( ngsup > 1 ) then
      zsgs1 = lsgs(1)
      zsgs3 = lsgs(3)
      zx    = 0
   else
      zsgs1 = lsgs(1) - sgsoff
      zsgs3 = lsgs(3) - sgsoff
      zx    = 17
   end if

   ! Means, effects, standard deviations
   ! -----------------------------------
   if ( levout(2) ) then

      if ( levout(3) .and. .not. samew ) then
            ! Print Y-means
         call chkpg( 10, .false., newp )
         if ( newp ) then
            write(imp,222) cmptit(:lcmpt)
            lines = lines + 1
         end if
         write(imp,111)
         lines = lines + 4
         write(imp,222) '  ', msgs(1)(:zsgs1), undlin(:zsgs1)
         lines = lines + 3
         call pacpry( iout, .true., nomwr, ybar, nwdc, ngsup, nwvar, labg, supg )
      end if

         ! Print effect
      call chkpg( 6, .false., newp )
      if ( newp .and. .not. linter ) then
         write(imp,222) cmptit(:lcmpt)
         lines = lines + 1
      end if
      write(imp,222) '  ', msgs(2), undlin(:lsgs(2))
      lines = lines + 3
      call prtmat( iout, .true., eff, nv, nv, nwvar, labv, nomwr )

      if ( .not. samew ) then

            ! Compute Y-std.dev., if requested, and store them
            ! in temp, followed by pooled within groups std.dev.
         jk = 0
         ll = 0
         do ks = 1, ngprt
            k = supg(ks)
            ragk = ag(k) - one
            if ( ragk > zero ) ragk = one / ragk
            jj  = ll
            do j = 1, nwvar
               jk = jk + 1
               jj = jj + j
               temp(jk) = sqrt( yqg(jj)*ragk )
            end do
            ll = ll + nwdct
         end do
         ll = 0
         do j = 1, nwvar
            ll = ll + j
            jk = jk + 1
            if ( ys2(ll) > zero ) then
               temp(jk) = sqrt( ys2(ll) )
            else
               temp(jk) = zero
            end if
         end do

            ! Print Y-standard deviations
         if ( levout(3) .or. ngsup == 1 ) then
            call chkpg( 7, .false., newp )
            if ( newp .and. .not. linter ) then
               write(imp,222) cmptit(:lcmpt)
               lines = lines + 1
            end if
            write(imp,222) '  ', msgs(3)(:zsgs3), undlin(:zsgs3)
            lines = lines + 3
            call pacpry( iout, .true., nomwr, temp(1:nwvar*ngsup),  &
                         nwvar, ngsup, nwvar, labg, supg )
         end if

            ! Print pooled within groups Y-standard deviations
            ! (if product-form comparison)
         if ( ngsup > 1 ) then
            call chkpg( 6, .false., newp )
            if ( newp .and. .not. linter ) then
               write(imp,222) cmptit(:lcmpt)
               lines = lines + 1
            end if
            write(imp,222) '  ', msgs(4), undlin(:lsgs(4))
            lines = lines + 3
            if ( lupvw ) then
               call prtmat( iout, .true.,                           &
                            temp(nwvar*ngprt+1:nwvar*ngprt+nwvar),  &
                            1, 1, nwvar, blank, nomwr )
            else
               write(imp,333)
               lines = lines + 1
            end if
         end if

      end if

   end if

   ! Sum of squares and products of the comparison
   ! ---------------------------------------------
      ! Note that currently no corrective matrix can be used by pacyss
      ! when called by this routine as none in argument of this routine
      ! (in other words, pacyss must not be called here if variables
      ! have been adjusted for covariates)
   if ( levout(4) .and. sumsq )   &
      call pacyss( nvar, nv, eff, scalar, .false., temp(1:1), 1, nomwr, nomv )

   ! Correlation and/or covariance matrices
   ! --------------------------------------
   if ( .not. samew .and. nwvar > 1 ) then

      if ( levout(5) ) then
            ! Print pooled within groups correlation matrix
            ! (if product-form comparison)

         if ( lupvw ) then
            call prtsym( iout, .true., ys2, nwvar, nomwr, .true., corwth, lcrwth-zx )
         else
            call chkpg( 4, .false., newp )
            if ( newp .and. .not. linter ) then
               write(imp,222) cmptit(:lcmpt)
               lines = lines + 1
            end if
            write(imp,222) '  ', corwth(:12), undlin(:12)
            write(imp,333)
            lines = lines + 4
         end if

      end if

      if ( levout(6) .and. ngsup > 1 ) then
            ! Print correlation matrix per group

         ll = 1
         do ks = 1, ngsup
            k = supg(ks)
            cortex = corgrp // labg(k)
            call prtsym( iout, .true., yqg(ll), nwvar, nomwr, .true., cortex, lcrgrp+8 )
            ll = ll + nwdct
         end do

      end if

      if ( levout(7) ) then
            ! Print pooled within groups covariance matrix
            ! (if product-form comparison)

         if ( lupvw ) then
            call prtsym( iout, .true., ys2, nwvar, nomwr, .false., covwth, lcvwth-zx )
         else
            call chkpg( 4, .false., newp )
            if ( newp .and. .not. linter ) then
               write(imp,222) cmptit(:lcmpt)
               lines = lines + 1
            end if
            write(imp,222) '  ', covwth(:11), undlin(:11)
            write(imp,333)
            lines = lines + 4
         end if

      end if

   end if

   INCLUDE 'pacydsf.'

END SUBROUTINE pacyds
