SUBROUTINE pacyss( nvar, nrow, efm, scalar, acov, bmat, lbmat, nomwr, nomv )

!***********************************************************************
!     - pacyss -
!     computes and prints the matrix of Sum of Squares and products of
!     the comparison (both the solutions with and without circularity
!     assumption are given)
!***********************************************************************
!
!     NVAR  - INPUT . Number of dependent variables concerned - INTEGER
!     NROW  - INPUT . Number of rows of the matrix of effect  - INTEGER
!     EFM   - INPUT . Matrix of effect, with NROW rows and    - REAL*8
!                     and NCW*NVAR columns
!     SCALAR- INPUT . A scalar used to multiply the EFM'*EFM - REAL*8
!                     matrix to obtain the matrix of the sum
!                     of squares & products of the comparison
!     ACOV  - INPUT . If .true. the corrective matrix BMAT    - LOGICAL
!                     will be used to compute the matrix
!                     (i.e. when adjustment for covariates)
!     BMAT  - INPUT . A corrective matrix of order NROW and   - REAL*8
!                     stored in triangular form
!                     Used only if ACOV is .true.
!     LBMAT - INPUT . Actual dimension of BMAT                - INTEGER
!                     (NROW*(NROW+1)/2 or 1 if not used)
!     NOMWR - INPUT . Array of W-contrasts & variables labels - CHAR
!                     of length NCW*NVAR
!     NOMV  - INPUT .    see definition of MEMB sub-arrays    - CHAR
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        MTAAS  MTASAZ  PRTSYM
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE memst2
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: scalar
   INTEGER, INTENT(IN) :: nvar, nrow, lbmat
   LOGICAL, INTENT(IN) :: acov
   REAL(KIND=8), INTENT(IN) :: efm(nrow,ncw*nvar), bmat(lbmat)
   CHARACTER(LEN=*), INTENT(IN) :: nomwr(ncw*nvar), nomv(nvtot)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: trk
   INTEGER :: i, ii, iout, j, jj, k, ll, lll, nwvar
   REAL(KIND=8) :: temp(ncw*nvar*(ncw*nvar+1)/2)

   INCLUDE 'pacyssp.'

   !--------------------------------------------------------------------

      ! Copy to avoid double transmition of argument
   iout = imp

   nwvar = ncw * nvar
   ll = nwvar * (nwvar+1) / 2
      ! Circularity not assumed:
      ! compute efm'*(bmat)*efm and store in temp
   if ( acov ) then
      call mtasaz( efm, bmat, temp, nrow, nwvar )
   else
      call mtaas( efm, temp, nrow, nwvar )
   end if
      ! Rescale the matrix
   temp = scalar * temp
      ! Print the matrix
   call prtsym( iout, .true., temp, nwvar, nomwr,.false., ssqp, lssqp )
      ! Circularity is assumed:
      ! compute the traces of the symmetric sub-matrices ncw*ncw
      ! in temp, for each pair of variables
      ! The resultant symmetric matrix is also stored in temp
      ! in triangular form with length nvar*(nvar+1)/2
   if ( ncw > 1 .and. lupvw ) then
      lll = 0
      ll = 0
      do j = 1, nvar
         jj = ll*(ll+1)/2
         do k = 1, j
            ii = jj - (j-k)*ncw
            trk = zero
            do i = ll+1, ll+ncw
               ii = ii + i
               trk = trk + temp(ii)
            end do
            lll = lll + 1
            temp(lll) = trk
         end do
         ll = ll + ncw
      end do
         ! Print the condensed matrix
      call prtsym( iout, .true., temp, nvar, nomv, .false., ssqp//msgc, lssqp+lmsgc )
   end if

END SUBROUTINE pacyss
