SUBROUTINE propar( par1, par2, n, np1, np2, nno0, sam0, allp )

!***********************************************************************
!     - propar -
!     computes the PROduct of 2 PARtitions of a same set and checks
!     this product is complete
!***********************************************************************
!
!     PAR1  - INPUT . 1st partition-array, of length N        - INTEGER
!                     Each term is the number of the part to
!                     which the cell belongs
!                     Parts are coded 1 to NP1, 0 codes for
!                     irrelevant cells
!             OUTPUT. Product partition-array                 - INTEGER
!     PAR2  - INPUT . 2nd partition-array, of length N        - INTEGER
!                     Same coding rule as for PAR1
!     N     - INPUT . Length of PAR1, PAR2                    - INTEGER
!     NP1   - INPUT . Number of different parts in PAR1       - INTEGER
!     NP2   - INPUT . Number of different parts in PAR2       - INTEGER
!     NNO0  - OUTPUT. Number of non-zero cells in output PAR1 - INTEGER
!                     (i.e. those that were non-zero on input
!                     both in PAR1 and PAR2)
!     SAM0  - OUTPUT. .true. if zero cells (and thus non-zero - LOGICAL
!                     ones) were at same location on input in
!                     PAR1 and PAR2, .false. otherwise
!     ALLP  - OUTPUT. .true. if product is complete (i.e. the - LOGICAL
!                     NP1*NP2 parts exist), .false. otherwise
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n, np1, np2
   INTEGER, INTENT(IN) :: par2(n)
   INTEGER, INTENT(INOUT) :: par1(n)
   INTEGER, INTENT(OUT) :: nno0
   LOGICAL, INTENT(OUT) :: sam0, allp

   !  Local declarations
   !  ------------------

   INTEGER :: i, j

   !--------------------------------------------------------------------

   ! Compute product
   ! ---------------
   nno0 = 0
   sam0 = .true.
   do i = 1, n
      if ( par1(i) > 0 .and. par2(i) > 0 ) then
         par1(i) = par1(i)  +  (par2(i)-1) * np1
         nno0    = nno0 + 1
      else
         par1(i) = 0
         if ( (par1(i) == 0) .neqv. (par2(i) == 0) ) sam0 = .false.
      end if
   end do
   allp = nno0 > 0
   if ( .not. allp ) return

   ! Check for no empty part
   ! -----------------------
   chk: do j = 1, np1*np2
      do i = 1, n
         if ( par1(i) == j ) cycle chk
      end do
         ! A product number was not found: the product is not complete
      allp = .false.
      return
   end do chk

END SUBROUTINE propar
