LOGICAL FUNCTION valrdy( vnew, vold, nrow, ncoln, ncolo, tola )

!***********************************************************************
!     - valrdy -
!     is .true. on return if a given set of Vectors is ALReaDY a subset
!     of consecutive vectors in another set (note that equality, and
!     not proportionality, is checked, using an absolute tolerance
!     criteria)
!***********************************************************************
!
!     VNEW  - INPUT . The set of vectors to be checked for    - REAL*8
!                     existing in VOLD as a consecutive
!                     subset and in the same order
!                     (columns are vectors)
!     VOLD  - INPUT . The set of vectors that may already     - REAL*8
!                     contain those in VNEW as a consecutive
!                     subset and in the same order
!                     (columns are vectors)
!     NROW  - INPUT . Number of rows in VNEW and VOLD         - INTEGER
!     NCOLN - INPUT . Number of columns (vectors) in VNEW     - INTEGER
!     NCOLO - INPUT . Number of columns (vectors) in VOLD     - INTEGER
!     TOLA  - INPUT . A tolerance for zero                    - REAL*8
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: tola
   INTEGER, INTENT(IN) :: nrow, ncoln, ncolo
   REAL(KIND=8), INTENT(IN) :: vnew(nrow,ncoln), vold(nrow,ncolo)

   !  Local declarations
   !  ------------------

   INTEGER :: i, j, jj, j1, k, n

   !--------------------------------------------------------------------

   valrdy = .false.

   j1 = 0
   outer: do

      ! Find j1, a vector in vold equal to the 1st vector in vnew
      ! ---------------------------------------------------------
      n = ncolo - j1
      inner: do j = 1, n

         if ( ncolo-j1 < ncoln ) return
         j1 = j1 + 1
         do k = 1, nrow
            if ( abs( vnew(k,1)-vold(k,j1) ) > tola ) cycle inner
         end do
         ! j1 has been found; check equality of other vectors in vnew with j1+1,...
         ! ------------------------------------------------------------------------
            ! (If equality doesn't hold, look for another "j1")
         jj = j1
         do i = 2, ncoln
            jj = jj + 1
            do k = 1, nrow
               if ( abs( vnew(k,i)-vold(k,jj) ) > tola ) cycle outer
            end do
         end do
         valrdy = .true.
         return   ! Subset found
                  ! ------------
      end do inner

      return   ! There's no such j1
               ! ------------------
   end do outer

END FUNCTION valrdy
